package tp2gui;

BEGIN {
	use Exporter ();
	our ($VERSION, @ISA, @EXPORT, %EXPORT_TAGS, @EXPORT_OK);
	$VERSION=0.1;
	@ISA    = qw(Exporter);

	@EXPORT_OK = qw(
		&choose_ns_dialog
		&choose_repos_dialog
		&_get_font
		&show_error
		&please_wait_msg
		&remove_please_wait_msg
		&populate_pkglist_widget
		&get_filters
		&get_install_opts
		&create_textlog_window
		&wait_button_textlog_window
		&addtext_to_textlog_window
		&set_progbar
		&create_loglist


		&choose_filteropts_dialog
		&populate_installed_pkglist_widget
		&hide_pkglist_widget
		&hide_installed_pkglist_widget
		&restore_pkglist_widget
		&restore_installed_pkglist_widget
		&create_pkglist_widget
		&create_installed_pkglist_widget
	);
};

use tp2utils qw(
	get_ns_list
	namespace_to_obj
	);
use Tk;
require Tk::Frame;
require Tk::Label;
require Tk::Toplevel;
require Tk::Font;
require Tk::ROText;
require Tk::ProgressBar;

use Data::Dumper;
use Carp qw(cluck);

my $cui=undef;
my %LB_values=();
my %bitmaps=();
my %fonts=();
my %pkglist_entdata=();

sub menu_loose_focus {
my $win=$_[0];
my $menu=$_[1];

	return;
	my $m=$win->getobj($menu);
	$m->loose_focus;
}

sub make_main_win($$$) {
my $title=$_[0];
my $id=$_[1];
my $menus=$_[2];

	# Return a UI::Window for putting things into...
	if(!defined($cui)) {
		$cui = new Curses::UI (
			-clear_on_exit	=> 1,
			-debug		=> 0,
		);
		$cui->set_binding(sub {exit;},"\cQ");
	}
	$cui->add(
		"menu","Menubar",-menu => $menus);
	$cui->add(
		"win_main",'Window',
		-border       	=> 1,
		-titlereverse 	=> 0,
		-padtop       	=> 2,
		-padbottom    	=> 3,
		-ipad         	=> 1,
		-onfocus	=> sub { menu_loose_focus($cui,"menu");},
		-title		=> $title
	);
	$cui->add(
		"win_bbox","Window",
		-border		=> 1,
		-y		=> -1,
		-height		=> 3,
	);
	return $cui;
}

#########################################################################
# Given a window id and a title pop a dialog allowing the user to 	#
# choose the namespace to perform the action on...			#
#########################################################################

sub listbox_callback {
	my $listbox = shift;
	my @sel = $listbox->get;
	@sel = ('<none>') unless @sel;
	$LB_values{$_[0]}=$sel[0];
}

sub buttonbox_callback {
	my $this = shift;
	$LB_values{bbox}=$this->get;
}

sub _get_font($$$$) {
my ($top,$family,$size,$weight)=@_;

	if(exists($fonts{"$family,$size,$weight"})) {
		return $fonts{"$family,$size,$weight"};
	}
	my $f=$top->Font(-family=>$family,-size=>$size,-weight=>$weight);
	$fonts{"$family,$size,$weight"}=$f;
	return $f;
}

sub choose_ns_dialog($$) {
my $top=$_[0];
my $title=$_[1];
my ($selected,$action);

	my $font=_get_font($top,"Courier",10,"normal");
	my $wid=$top->Toplevel(-title => "Choose Namespace to Install Packages In..");
	my $f=$wid->LabFrame(-label => "Available Namespaces",-labelside => "acrosstop")->pack(-side=>"top",-expand => 1, -fill => "both");
	my $lb=$f->Listbox(-font=>$font,-height=>20,-selectmode=>"browse",-width=>100)->pack(-expand=>1,-fill=>"both");


	my ($nslist,$errs,$warns)=get_ns_list;
	my %L=(); my $i=1;
	for (@$nslist) {
		$L{$i}=sprintf("%-20s  %s",$_->{NAME},$_->{ROOT});
		$lb->insert("end",$L{$i});
		$i++;
	}
	my $f2=$wid->Frame()->pack(-side=>"bottom",-expand => 0, -anchor=>"s");
	$f2->Button(-text=>"Ok",-command=>sub {if(ref($lb->curselection)) {$selected=$nslist->[$lb->curselection->[0]]{NAME}; } else {$selected=$nslist->[$lb->curselection]->{NAME}}; $action="OK"; $wid->destroy;})->pack(-side=>"left",-expand=>0,-padx=>10);
	$f2->Button(-text=>"Cancel",-command=>sub {$selected=undef; $action="CANCEL"; $wid->destroy;})->pack(-side=>"left",-expand=>0,-padx=>10);
	$f2->Button(-text=>"Exit",-command=>sub {$selected=undef; $action="EXIT"; $wid->destroy;})->pack(-side=>"left",-expand=>0,-padx=>10);
	$top->update;
	$wid->transient($top);
	$lb->bind("<Double-Button-1>",sub {if(ref($lb->curselection)) {
		$selected=$nslist->[$lb->curselection->[0]]->{NAME}; } else {
		$selected=$nslist->[$lb->curselection]->{NAME};} $action="OK"; $wid->destroy;});
	$wid->grab;
	$wid->waitWindow;
	return($action,$selected);
}

sub _fatal($$) {
my $top=$_[0];
my $msg=$_[1];

	show_error($top,$msg,"stop");
	exit(0);
}
{
my $orig_tp2path=$ENV{TP2PATH}||"";

sub choose_repos_dialog($$;$) {
my $top=$_[0];
my $title=$_[1];
my $ns=$_[2];
my ($selected,$action);

	#################################################################
	# Check to see if the current namespace defines a REPOS line 	#
	# and use it if TP2 path is not set.				#
	#################################################################
	$ENV{TP2PATH}=$orig_tp2path;
	if(defined($ns)) {
		my ($nsobj,$err)=namespace_to_obj($ns);
		if(defined($nsobj)) {
			if(defined($nsobj->{REPOS})) {
				$ENV{TP2PATH}="$orig_tp2path $nsobj->{REPOS}";
			}
		}
	} 

	if(!length($ENV{TP2PATH})) {
		_fatal($top,"The 'TP2PATH' environment variable is not defined. Please set this to a list of repositories to choose from and retry the command.");
	}
	my @repos_list=split(/[, ]/,$ENV{TP2PATH});
	my @rp2=();
	for (@repos_list) {
		if(defined($_) && length($_)) {
			push @rp2,$_;
		}
	}
	@repos_list=@rp2;

	if(@repos_list==0) {
		_fatal($top,"The 'TP2PATH' environment variable is not defined. Please set this to a list of repositories to choose from and retry the command.");
	}

	print Dumper(@repos_list);
	my $font=_get_font($top,"Courier",10,"normal");
	my $wid=$top->Toplevel(-title => "Choose Repository to get Packages from ..");
	my $f=$wid->LabFrame(-label => "Available Repositories",-labelside => "acrosstop")->pack(-side=>"top",-expand => 1, -fill => "both");
	my $lb=$f->Listbox(-font=>$font,-height=>20,-selectmode=>"browse",-width=>100)->pack(-expand=>1,-fill=>"both");

	#################################################################
	# Clean the list - remove dups, leading spaces etc.		#
	#################################################################
	my $last="?";
	my @mlist=();
	for (@repos_list) {
		push @mlist,$_ if $_ ne $last;
		$last=$_;
	} 

	my %L=(); my $i=1;
	for (@mlist) {
		if(substr($_,0,1) eq "/") {
			$L{$i}=sprintf("%-12s  %s","Directory",$_);
		} else {
			my ($x,$y)=split(/:/,$_);
			$x="Http://" if $x eq "HTTP" || $x eq "WWW";
			$x="Ftp://" if $x eq "FTP";
			$L{$i}=sprintf("%-12s  %s",$x,$y);
		}
		$lb->insert("end",$L{$i});
		$i++;
	}

	#################################################################
	# Add the updates and force an update.				#
	#################################################################

	my $f2=$wid->Frame()->pack(-side=>"bottom",-expand => 0, -anchor=>"s");
	$f2->Button(-text=>"Ok",-command=>sub {if(ref($lb->curselection)) {
		$selected=$mlist[$lb->curselection->[0]]; } else {
		$selected=$mlist[$lb->curselection];}; 
		$action="OK"; $wid->destroy;})->pack(-side=>"left",-expand=>0,-padx=>10);
	$f2->Button(-text=>"Cancel",-command=>sub {$selected=undef; $action="CANCEL"; $wid->destroy;})->pack(-side=>"left",-expand=>0,-padx=>10);
	$f2->Button(-text=>"Exit",-command=>sub {$selected=undef; $action="EXIT"; $wid->destroy;})->pack(-side=>"left",-expand=>0,-padx=>10);
	$lb->bind("<Double-Button-1>",sub {if(ref($lb->curselection)) {
		$selected=$mlist[$lb->curselection->[0]];} else {
		$selected=$mlist[$lb->curselection];}; $action="OK"; $wid->destroy;});
	$top->update;
	$wid->transient($top);
	$wid->grab;
	$wid->waitWindow;
	return($action,$selected);
}

}

sub create_installed_pkglist_widget($$$) {
my $cui=$_[0];
my $callback=$_[1];
my $namespace=$_[2];
my $idw=$cui->getobj('win_main');
my $idw2=$cui->getobj('menu');

	$idw->set_binding( sub {$idw2->focus();}, CUI_TAB());
	$idw2->set_binding( sub {$idw->focus();}, CUI_TAB());
	
	my $listbox => $idw->add(
		"installed_pkglist_widget","Listbox",
		-y		=> 1,
		-x		=> 2,
		# -width		=> 95,
		# -height		=> 15,
		-title		=> "Installed Packages [$namespace]",
		-border		=> 1,
		-onblur		=> sub {$idw2->focus();},
		-multi		=> 1,
		-vscrollbar	=> 1,
		-hscrollbar	=> 1,
		-onchange	=> sub {listbox_callback($_[0],"installed_pkglist_widget"); &$callback;}
	);
	my $idw3=$idw->getobj('installed_pkglist_widget');
	$idw3->focus(undef, 1);
	$idw3->set_binding( sub {$idw2->focus();}, CUI_TAB());
	$cui->draw;
	delete $LB_values{installed_pkglist_widget} if exists($LB_values{installed_pkglist_widget});
	return "installed_pkglist_widget";
}

sub create_pkglist_widget($$) {
my $cui=$_[0];
my $callback=$_[1];
my $idw=$cui->getobj('win_main');
my $idw2=$cui->getobj('menu');
my @repos_list=sort(split(/,/,$ENV{TP2PATH}));

	if(@repos_list==0) {
		_fatal($cui,"The 'TP2PATH' environment variable is not defined. Please set this to a list of repositories to choose from and retry the command.");
	}

	$idw->set_binding( sub {$idw2->focus();}, CUI_TAB());
	$idw2->set_binding( sub {$idw->focus();}, CUI_TAB());
	
	if($idw->getobj('pkglist_widget')) {
		delete $LB_values{pkglist_widget} if exists($LB_values{pkglist_widget});
		return "pkglist_widget";
	}

	my $listbox => $idw->add(
		"pkglist_widget","Listbox",
		-y		=> 1,
		-x		=> 2,
		# -width		=> 95,
		# -height		=> 15,
		-title		=> "Available Packages",
		-border		=> 1,
		-onblur		=> sub {$idw2->focus();},
		-multi		=> 1,
		-vscrollbar	=> 1,
		-hscrollbar	=> 1,
		-onchange	=> sub {listbox_callback($_[0],"pkglist_widget"); &$callback;}
	);
	my $idw3=$idw->getobj('pkglist_widget');
	$idw3->focus(undef, 1);
	$idw3->set_binding( sub {$idw2->focus();}, CUI_TAB());
	$cui->draw;
	delete $LB_values{pkglist_widget} if exists($LB_values{pkglist_widget});
	return "pkglist_widget";
}

sub get_install_opts($$) {
my $top=$_[0];
my $options=$_[1];
my $action="";

	my @V=();
	my $wid=$top->Toplevel(-title => "Choose Package Install Options",-width=>500);
	my $lf=$wid->LabFrame(-label => "Package Installation Options ",-labelside => "acrosstop")->pack(-side=>"top",-expand => 1, -fill => "both");
	my @checks=();
	my $r=0;
	for $ckey (sort {int($a) <=> int($b)} keys(%$options)) {
		push @checks,$lf->Checkbutton(-text=>$options->{$ckey}->{label})->grid(-column=>0,-row=>$r,-sticky=>"w");
		$checks[$#checks]->{'Value'}=$options->{$ckey}->{value};
		$r++;
	}
	$wid->Frame()->pack(-side=>"bottom",-expand => 1, -fill => "x");

	$wid->Button(-text=>"Ok",-command=>sub {$action="OK"; push @V,$_->{'Value'} for (@checks); $wid->destroy;})->pack(-side=>"left",-expand=>0,-fill => "x");
	$wid->Button(-text=>"Cancel",-command=>sub {$action="CANCEL"; $wid->destroy;})->pack(-side=>"right",-expand=>0,-fill => "x");
	$top->update;
	$wid->resizable(0,0);
	$wid->transient($top);
	$wid->grab;
	$wid->waitWindow;

	if($action eq "OK") {
		my $x=0;
		for $ckey (sort {int($a) <=> int($b)} keys(%$options)) {
			$options->{$ckey}->{value}=$V[$x];
			$x++;
		}
	}
}

sub get_filters($$) {
my $top=$_[0];
my $options=$_[1];
my $action="";

	my @V=();
	my $wid=$top->Toplevel(-title => "Choose Package Filters",-width=>300);
	my $lf=$wid->LabFrame(-label => "Filter Options ",-labelside => "acrosstop")->pack(-side=>"top",-expand => 1, -fill => "both");
	$b1=$lf->Checkbutton(-text=>"Show new packages to install")->grid(-column=>0,-row=>0,-sticky=>"w");
	# $b1=$lf->Checkbutton(-text=>"Show new packages to install")->pack(-side=>"top",-expand=>0,-fill=>"x",-anchor=>"w");
	$b1->{'Value'}=$options->{new};
	$b2=$lf->Checkbutton(-text=>"Show updates to installed packages")->grid(-column=>0,-row=>1,-sticky=>"w");
	# $b2=$lf->Checkbutton(-text=>"Show updates to installed packages")->pack(-side=>"left",-expand=>1,-fill=>"x");;
	$b2->{'Value'}=$options->{updates};
	$b3=$lf->Checkbutton(-text=>"Show older revisions of installed packages")->grid(-column=>0,-row=>2,-sticky=>"w");
	# $b3=$lf->Checkbutton(-text=>"Show older revisions of installed packages")->pack(-side=>"left",-expand=>1,-fill=>"x");;
	$b3->{'Value'}=$options->{older};

	$wid->Frame()->pack(-side=>"bottom",-expand => 1, -fill => "x");

	$wid->Button(-text=>"Ok",-command=>sub {$action="OK"; @V=($b1->{'Value'},$b2->{'Value'},$b3->{'Value'}); $wid->destroy;})->pack(-side=>"left",-expand=>0,-fill => "x");
	$wid->Button(-text=>"Cancel",-command=>sub {$action="CANCEL"; $wid->destroy;})->pack(-side=>"right",-expand=>0,-fill => "x");
	$top->update;
	$wid->resizable(0,0);
	$wid->transient($top);
	$wid->grab;
	$wid->waitWindow;

	if($action eq "OK") {
		($options->{new},$options->{updates},$options->{older})=@V;
	}
}


#########################################################################
# The window will also contain a button to view the logs of the 	#
# software installation.						#
#########################################################################

sub _view_log($$$) {
my $top=$_[0];
my $file=$_[1];
my $title=$_[2];

	my $txtwin=create_textlog_window($top,$title);
	my $fd;
	open($fd,$file);
	if($fd) {
		addtext_to_textlog_window($txtwin,$_) foreach(map {chomp($_); $_;} <$fd>);
		close($fd);
	}
	wait_button_textlog_window();
}

sub create_loglist($$$) {
my $top=$_[0];
my $title=$_[1];
my $logref=$_[2];

	my @text=();
	{
		my ($clog,$fd,$cline,$preview,$result);
		for $clog (@$logref) {
			$preview=0;
			$result="Unknown";
			open($fd,$clog);
			$cline=<$fd>;
			if($cline =~ /--preview/) {
				$preview=1;
			}
			while(<$fd>) {
				$cline=$_;
			}
			close($fd);
			if($cline =~ /Result: SUCCESS/) {
				$result="Successful";
			} elsif($cline =~ /Result: FAIL/) {
				$result="Failed";
			}
			my $r=(stat($clog))[9];
			my @x=localtime($r);
			my $tt=sprintf("%02d:%02d:%02d",$x[2],$x[1],$x[0]);
			

			my @b=split(/\+/,$clog);
			if($b[0] =~ /\/install$/) {
				my $y="$result";
				$y.=" preview" if $preview==1;
				$y.=" installation of '$b[1]' @ $tt";
				push @text,$y;
			}
			if($b[0] =~ /\/remove$/) {
				my $y="$result";
				$y.=" preview" if $preview==1;
				$y.=" removal of '$b[1]' @ $tt";
				push @text,$y;
			}
		}
	}

	my $wid=$top->Toplevel(-title => "Choose Namespace to Install Packages In..");
	my $f=$wid->LabFrame(-label => "Available Namespaces",-labelside => "acrosstop")->pack(-side=>"top",-expand => 1, -fill => "both");
	my $lb=$f->Listbox(-font=>$font,-height=>20,-selectmode=>"browse",-width=>100)->pack(-expand=>1,-fill=>"both");
	my $f2=$wid->Frame()->pack(-side=>"bottom",-expand => 0, -anchor=>"s");
	$f2->Button(-text=>"View",-command=>sub {if(ref($lb->curselection)) {
		_view_log($top,$logref->[$lb->curselection->[0]],$text[$lb->curselection=>[0]]);} else {
		_view_log($top,$logref->[$lb->curselection],$text[$lb->curselection]);})->pack(-side=>"left",-expand=>0,-padx=>10);
	$f2->Button(-text=>"Return",-command=>sub {$wid->destroy();})->pack(-side=>"left",-expand=>0,-padx=>10);
	$lb->bind("<Double-Button-1>",sub {if(ref($lb->curselection)) {
		_view_log($top,$logref->[$lb->curselection->[0]],$text[$lb->curselection->[0]]);} else {
		_view_log($top,$logref->[$lb->curselection],$text[$lb->curselection]);} });

	$top->update;
	$wid->transient($top);
	$wid->grab;
	$lb->insert("end",$_) foreach (@text);
	$wid->waitWindow;
	return;
}

{
my $wid=undef;
my $but=undef;
my $top=undef;
my $prgbar=undef;

sub create_textlog_window($$;$) {
$top=$_[0];
my $title=$_[1];
my $show_progbar=$_[2] || 0;


	$wid=$top->Toplevel(-title => $title);
	my $lf=$wid->Frame()->pack(-side=>"top",-expand => 1, -fill => "both");
	my $font=_get_font($top,"Courier",10,"normal");
	my $sb=$lf->Scrollbar()->pack(-fill =>"y",-side=>"right");
	my $txtwin=$lf->ROText(-width=>80,-font=>$font,-height=>20,
		-yscrollcommand => ['set',$sb])->pack(-expand =>1,-fill =>"both",-side=>"left");
	$sb->configure(-command => ['yview',$txtwin]);

	my $fr=$wid->Frame()->pack(-side=>"bottom",-expand => 0, -fill => "x");
	if($show_progbar) {
		my $fr=$wid->Frame()->pack(-side=>"bottom",-expand => 0, -fill => "x");
		$prgbar=$fr->ProgressBar(-anchor=>"w",-width=>15,-length=>300,-to=>100,-gap=>2,-from=>0,-colors=>[0,'blue'],
			-blocks=>100)->pack(-side=>"left",-fill=>"x",-expand=>1);
	}

	$but=$fr->Button(-text=>"Ok",-command=>sub {$action="OK"; @V=($b1->{'Value'},$b2->{'Value'},$b3->{'Value'}); $wid->destroy;})->pack(-side=>"left",-expand=>0,-fill => "x");
	$but->configure(-state => "disabled");
	$top->update;
	$wid->grab;
	return($txtwin);
}

sub wait_button_textlog_window {

	$but->configure(-state => "normal", -command => sub {$wid->destroy});
	$top->update;
	$wid->grab;
	$wid->waitWindow;
}

sub addtext_to_textlog_window($$) {
my $txt=$_[0];
my $text=$_[1];

	$txt->insert("end","$text\n");
	$txt->see("end");
	$top->update;
}

sub set_progbar($) {
	$prgbar->value($_[0]);
	$top->update;
}
}

sub populate_installed_pkglist_widget($$$) {
my ($cui,$pkglist,$UNFILTERED_REPOS)=@_;
my $idw=$cui->getobj('win_main');
my $pkgw=$idw->getobj($pkglist);

	#################################################################
	# Clear out any existing entries for the widget first...	#
	#################################################################

	die("No pkglist widget found!") if !defined($pkgw);

	$verwidth=0;
	$pkgwidth=0;
	my $x;
	for $x (@$UNFILTERED_REPOS) {
		if(length($x->{NAME})>$pkgwidth) {
			$pkgwidth=length($x->{NAME});
		}
		if(length($x->{PKGVER})>$verwidth) {
			$verwidth=length($x->{PKGVER});
		}
	}

	$pkgw->remove_entries;
	my @L=();
	my %h=();
	my $xx=0;
	for $x (@$UNFILTERED_REPOS) {
		push @L,$xx;
		$h{$xx}=sprintf("%-*s  %-*s  %s",$pkgwidth,$x->{NAME},
			$verwidth,$x->{PKGVER},$x->{DESCRIPTION});
		$xx++;
	}
	$pkgw->values(@L);
	$pkgw->labels(\%h);
}

sub populate_pkglist_widget($$$) {
my ($top,$pkglist,$UNFILTERED_REPOS)=@_;

	#################################################################
	# Clear out any existing entries for the widget first...	#
	#################################################################

	my $font=_get_font($top,"Courier",10,"normal");
	$pkglist->delete(0,"end");
	my %entdata=();
	my $verwidth=0;
	my $pkgwidth=0;
	my $x;
	for $x (@$UNFILTERED_REPOS) {
		if(length($x->{pkgname})>$pkgwidth) {
			$pkgwidth=length($x->{pkgname});
		}
		if(length($x->{pkgvers})>$verwidth) {
			$verwidth=length($x->{pkgvers});
		}
	}
	my $xx=0;

	for $x (@$UNFILTERED_REPOS) {
		my $t=sprintf("%-*s  %-*s  %s",$pkgwidth,$x->{pkgname},
			$verwidth,$x->{pkgvers},$x->{pkgldesc});
		$pkglist->insert("end",$t);
		$entdata{$xx}=[$x->{pkgname},$x->{pkgvers},$x->{pkgldesc}];
		$xx++;
	}
	$pkglist->bind("<ButtonPress-1>",[\&pkglist_callback,$UNFILTERED_REPOS,Ev('@'),\$entdata]);
}

sub pkglist_callback {
my $pkglist=$_[0];
my $pkgs=$_[1];
my $pos=$_[2];
my $ed1=$_[3];

	my @sel;
	if(ref($pkglist->curselection)) {
		@sel=$pkglist->curselection->[0];
	} else {
		@sel=$pkglist->curselection;
	}
	return if !@sel;

	#################################################################
	# Get the selection index at the cursor position...		#
	#################################################################

	my $at_cursor=$pkglist->index($pos);
	if(! scalar(grep {$_ == $at_cursor} @sel)) {
		return;
	}

	#################################################################
	# If check all selected packages and if another of the same	#
	# name is selected then we remove it from the list of selected	#
	# packages.							#
	#################################################################
	
	for (@sel) {
		next if $_ == $at_cursor;
		if($pkgs->[$_]->{pkgname} eq $pkgs->[$at_cursor]->{pkgname}) {
			$pkglist->selectionClear($_,$_);
		}
	}
}

sub _get_xpm($$) {
my $top=$_[0];
my $bit=$_[1];

	if(!exists($bitmaps{$bit})) {
		my $f="$ENV{TP2_LIBS}/../pixmaps/$bit.xpm";
		if(! -f $f) {
			$bit=undef;
		} else {
			$bitmaps{$bit}=$top->Pixmap(-file=>$f); 
			$bit=$bitmaps{$bit};
		}
	} else {
		$bit=$bitmaps{$bit};
	}
	return $bit;
}

{
my $wid;

sub please_wait_msg($$;$) {
my $top=$_[0];
my $msg=$_[1];
my $bit=$_[2];

	$bit="load" if !defined($bit);
	$bit=_get_xpm($top,$bit); 
	$wid=$top->Toplevel(-title => "Operation in progress...",-width=>300);
	my $f=$wid->Frame->pack(-expand=>1,-fill=>"both");
	if($bit) {
		$f->Label(-image => $bit)->pack(-side=>"left",-expand=>1,-fill=>"both");
	}
	$f->Label(-text => $msg)->pack(-side=>"right",-expand=>1,-fill=>"both");
	$top->update;
	$wid->grab;
}

sub remove_please_wait_msg() {
	$wid->grabRelease;
	$wid->destroy;
}
}

sub show_error($$;$) {
my $top=$_[0];
my $msg=$_[1];
my $bit=$_[2];

	$wid=$top->Toplevel(-title => "Error!",-width=>500);
	my $f=$wid->Frame->pack(-expand=>1,-fill=>"both",-side=>"top",-anchor=>"nw");
	$bit="warning" if !defined($bit);
	$bit=_get_xpm($top,$bit); 
	if($bit) {
		$f->Label(-image => $bit)->pack(-side=>"left",-expand=>0,-fill=>"none",-anchor=>"nw");
	}
	$f->Message(-text=>$msg,-width=>500)->pack(-side=>"right",-expand=>1,-fill=>"both",-anchor=>"ne");
	$wid->Button(-text=>"Ok",-command=>sub {$wid->destroy;})->pack(-side=>"bottom",-expand=>0,-fill=>"x");
	$top->update;
	$wid->resizable(0,0);
	$wid->transient($top);
	$wid->grab;
	$wid->waitWindow;
}

1;

