package tp2utils;

BEGIN {
	use Exporter ();
	our ($VERSION, @ISA, @EXPORT, %EXPORT_TAGS, @EXPORT_OK);
	$VERSION=0.1;
	@ISA    = qw(Exporter);

	@EXPORT_OK = qw(
		&repackage
		&dprint
		&does_script_exist
		&vercmp
		&remove_file_from_package
		&parse_pkgbuild_conf
		&validate_pkgbuild_conf
		&generate_pkg_config
		&spool_files_to_tmpdir
		&is_installed
		&file_sha1
		&make_dir_path
		&get_pkg_default_repos
		&get_pkg_name
		&remove_dir_path
		&check_ns_security
		&manage_ns_logs
		&manage_ns_spool
		&manage_repackage_space
		&pkgfile_config_to_obj
		&namespace_to_obj
		&update_namespace_file_meta
		&get_file_meta
		&install_pkg_files
		&_get_space_actually_used
		&_get_summary_counts
		&_get_removed_space
		&_get_summary_remove_counts
		&extract_to_spool
		&remove_from_spool
		&check_namespace_write
		&pkg_installed_ok
		&pkg_installed_failed
		&execute_script
		&install_firstboot
		&get_package_from_path
		&get_repos_file
		&checkinstall_code_from_repos_file
		&find_package
		&follow_dependencies
		&show_software_required
		&spool_software
		&do_software_installs
		&pkg_install_state
		&fs_impact
		&fs_impact_actual
		&check_for_all_clashes
		&get_owning_namespace
		&get_owning_package
		&get_ns_pkglist
		&get_ns_list
		&get_ns_log_list
		&file_installed_by
		&gain_lock
		&release_lock
		&rm_file_meta
		&pkg_remove_cleanup
		&remove_pkg_files_1
		&remove_pkg_files_2
		&return_dependents_list
		&rm_on_reboot
		&TP2_READLOCK
		&TP2_WRITELOCK
		&check_incompatibles
		&TP2_LATEST_PACKAGE
		&TP2_ALL_PACKAGES
		&_get_tar_cmd
	);
	@EXPORT = qw(errormsg logmsg warnmsg msg);
	%EXPORT_TAGS = (
		'locking'   => [qw(TP2_READLOCK TP2_WRITELOCK &gain_lock &release_lock)],
	);
}
sub TP2_LATEST_PACKAGE() {1};
sub TP2_ALL_PACKAGES() {2};

sub show_software_required($$$);
sub spool_software($$$$$;$);

my ($WWW_avail,$FTP_avail);
my %GROUP_MAP;
my %GROUP_MAP2;
my %USER_MAP;
my %USER_MAP2;
my %SHA1_MAP=();

use warnings;
use XML::Parser;
use Data::Dumper;
use Digest::SHA1;
use File::Basename;
use File::Find;
use Fcntl;
use Fcntl qw( :flock);
use Sys::Hostname;
use English;
use Cwd;
use Config;
use POSIX qw(getgid);
eval 'use Net::FTP;';
if($@) {
	$FTP_avail=0;
} else {
	$FTP_avail=1;
}

eval 'use LWP::Simple;';
if($@) {
	$WWW_avail=0;
} else {
	$WWW_avail=1;
}

my %repos_contents_hash=();

sub dprint(@) {
	return if !exists($ENV{DEBUG}) || $ENV{DEBUG} != 1;
	print "\n" if $need_newline;
	$need_newline=0;
	print "DEBUG:",@_;
}

sub _remove_dir_path {

	if(-d $File::Find::name) {
		rmdir($File::Find::name) or
			errormsg("Can not remove directory \"$File::Find::name\" - $!",8);
	} else {
		unlink($File::Find::name) or
			errormsg("Can not remove file \"$File::Find::name\" - $!",8);
	}
}

{
my $tar_cmd=undef;

sub _get_tar_cmd {
my (@P,$c,$r);

	return($tar_cmd) if defined($tar_cmd);
	@P=split(/:/,$ENV{PATH});
	for $c (@P) {
		if(-f "$c/tar" && -x "$c/tar" && -r "$c/tar") {
			$r=`$c/tar --help 2>&1`;
			if($r =~ /gnu/) {
				$tar_cmd="$c/tar";
				return $tar_cmd;
			} else {
				dprint "$c/tar found - but not suitable!\n";
			}
		}
		if(-f "$c/gtar" && -x "$c/gtar" && -r "$c/gtar") {
			$r=`$c/gtar --help 2>&1`;
			if($r =~ /gnu/) {
				$tar_cmd="$c/gtar";
				return $tar_cmd;
			} else {
				dprint "$c/gtar found - but not suitable!\n";
			}
		}
	}
	errormsg("No suitable tar or gtar command found.",1);
}

}

#########################################################################
# This routine is called by _tpboot_ll to clean up any files left over	#
# from packages that failed to remove all files, (most likely due to	#
# text file busy issues).						#
#########################################################################

sub rm_on_reboot($) {
my $nsobj=$_[0];
my ($name,$x,$y);
my @still_here=();
my $deleted=0;
my (@T,$user,$system);

	if(!ref($nsobj)) {
		($x,$y)=namespace_to_obj($nsobj);
		if(!defined($x)) {
			return(0,$y,0);
		}
		($nsobj,$name)=($x,$nsobj);
	} else {
		$name=$nsobj->{NAME};
	}

	#################################################################
	# Does the required file exist, if not exit OK.			#
	#################################################################

	if(! -f "/var/adm/tp2/ns/$name/pkg/rm on reboot") {
		@T=times(); $user=$T[0]+$T[2]; $system=$T[1]+$T[3];
		return(1,"Result: SUCCESS".sprintf(" [ %0.2f secs User CPU , %0.2f secs System CPU ]",$user,$system),0);
	}
	my $root=$nsobj->{ROOT};
	my ($fd,$cline2,$cline);
	if(!open($fd,"/var/adm/tp2/ns/$name/pkg/rm on reboot")) {
		return(0,"Unable to open file:\n$!",0);
	}

	while($cline=<$fd>) {
		chomp $cline;
		$cline2=$cline;
		#########################################################
		# Clean up the name to ensure nothing untoward actually	#
		# happens ...						#
		#########################################################

		$cline =~ s/\.\.//g;	# stop attempts to reach out
					# of namespace.
		if($cline2 ne $cline) {
			warnmsg("Removed relative path element from \"$cline2\".");
		}
		if(! -f "$root/$cline") {
			next;	# file does not exist, silently ignore
		}
		if(!unlink("$root/$cline")) {
			errormsg("Left $cline in place - could not remove:\n$!");
			push @still_here,$cline;
		} else {
			$deleted++;
			logmsg("Successfully removed $cline.");
		}
	}
	close($fd);

	#################################################################
	# If still_here contains entries then write out new version of	#
	# log file.							#
	#################################################################

	my $changed=0;
	$changed=1 if $deleted;
	if(@still_here) {
		open($fd,">/var/adm/tp2/ns/$name/pkg/rm on reboot.\$\$") or
			return(0,"Unable to write to temporary file:\n$!");
		for (@still_here) {
			if(!print $fd "$_\n") {
				close($fd);
				unlink("/var/adm/tp2/ns/$name/pkg/rm on reboot.\$\$");
				return(0,"Unable to write to temporary file:\n$!",$changed);
			}
		}
		if(!close($fd)) {
			unlink("/var/adm/tp2/ns/$name/pkg/rm on reboot.\$\$");
			return(0,"Unable to write to temporary file:\n$!",$changed);
		}
		if(!rename("/var/adm/tp2/ns/$name/pkg/rm on reboot.\$\$",
			"/var/adm/tp2/ns/$name/pkg/rm on reboot")) {
			unlink("/var/adm/tp2/ns/$name/pkg/rm on reboot.\$\$");
			return(0,"Unable to rename temporary file:\n$!",$changed);
		}
	} else {
		if(!unlink("/var/adm/tp2/ns/$name/pkg/rm on reboot")) {
			return(0,"Unable to rename temporary file:\n$!",$changed);
			
		}
	}
	#################################################################
	# Write out success/failure message and CPU utilsation.		#
	#################################################################

	@T=times(); $user=$T[0]+$T[2]; $system=$T[1]+$T[3];
	if(!scalar(@still_here)) {
		msg("Result: SUCCESS".sprintf(" [ %0.2f secs User CPU , %0.2f secs System CPU ]",$user,$system));
	} else {
		msg("Result: FAILED".sprintf(" [ %0.2f secs User CPU , %0.2f secs System CPU ]",$user,$system));
	}
	if(@still_here) {
		return(0,"Deleted $deleted files, ".scalar(@still_here)." left in place.",$changed);
	}
	return(1,"Deleted $deleted files, ".scalar(@still_here)." left in place.",$changed);
}

sub remove_dir_path($) {
my $cp=cwd;
	return if ! -d $_[0];
	chdir "/tmp";
	find({ wanted => \&_remove_dir_path, bydepth => 1, no_chdir => 1},$_[0]);
	chdir $cp;
}

sub _load_groups() {
my @F;

	while(@F=getgrent()) {
		$GROUP_MAP{$F[2]}=$F[0];
		$GROUP_MAP2{$F[0]}=$F[2];
	}
	endgrent();
}

sub _load_users() {
my @F;

	while(@F=getpwent()) {
		$USER_MAP{$F[2]}=$F[0];
		$USER_MAP2{$F[0]}=$F[2];
	}
	endpwent();
}

sub _getgrgid($) {
	if(!%GROUP_MAP) { _load_groups; }
	return $GROUP_MAP{$_[0]} if exists($GROUP_MAP{$_[0]});
	return "";
}

sub _getgrnam($) {
	if(!%GROUP_MAP2) { _load_groups; }
	return $GROUP_MAP2{$_[0]} if exists($GROUP_MAP2{$_[0]});
	return "";
}

sub _getpwnam($) {
my @F;

	if(!%USER_MAP2) { _load_users; }
	return $USER_MAP2{$_[0]} if exists($USER_MAP2{$_[0]});
	return "";
}

sub copyfile($$) {
my ($sname,$dname)=@_;
my ($rfd,$wfd,$cdata,$csize,$fsize,$fcount);
my (@sb,$togo,$toread,$aread);

	return(undef,"No such file: $sname") if ! -f $sname;
	return(undef,"Destination file exists: $dname") if -f $dname;
	@sb=stat($sname);
	if(!exists($sb[7])) {
		return(undef,"Unable to stat file $sname: $!");
	}
	sysopen($rfd,"$sname",O_RDONLY) or
		return(undef,"Unable to open file for reading: $sname");
	sysopen($wfd,"$dname",O_CREAT|O_WRONLY) or
		return(undef,"Unable to open file for writing: $dname : $!");

	$csize=0;
	$togo=$sb[7]-$csize;
	while($togo) {
		$toread=$togo>10240 ? 10240 : $togo;
		$aread=sysread($rfd,$cdata,$toread);
		if(!defined($aread)) {
			unlink $dname;
			return(undef,"Error whilst reading from $sname.");
		}
		last if !$aread;
		$toread=syswrite($wfd,$cdata,$aread);
		if(!defined($toread) || $toread!=$aread) {
			unlink $dname;
			return(undef,"Error whilst writing to $sname.");
		}
		$togo-=$aread;
	}
	close($wfd);
	close($rfd);
	return($sb[7],undef);
}

sub make_dir_path($) {
my ($cpath,$cd);

	$cpath="$_[0]";
	$cd="/";
	for (split(/\//,$cpath)) {
		next if /^$/;
		$cd.="$_/";
		if(! -d "$cd") {
			if(!mkdir($cd)) {
				errormsg("Unable to create directory \"$cd\" : $!",7);
			}
		}
	}
	return 1;
}

sub file_sha1($) {
my ($sha1,$fd);

	open($fd,$_[0]) or return(undef);
	$sha1=Digest::SHA1->new or return(undef);
	$sha1->addfile($fd);
	close($fd);
	return $sha1->hexdigest;
}

sub logmsg($) {
	return if ! $::ARG_verbose;
	if(defined($::LOGMSGFD)) {
		select($::LOGMSGFD);
		$|=1;
	}
	if($_[0] =~ /\.\.\.$/) {
		print "\n" if $need_newline;
		print "Log  : $_[0]";
		$need_newline=1;
	} elsif($_[0] =~ /^\.\.\./) {
		my $x=$_[0];
		$x =~ s/^\.\.\.//;
		print " $x\n";
		$need_newline=0;
	} else {
		print "\n" if $need_newline;
		print "Log  : $_[0]\n";
		$need_newline=0;
	}
	if(defined($::LOGMSGFD)) {
		select(STDOUT);
		$|=1;
	}
}

sub msg($) {
my $x;

	if(!defined($ARG_verbose)) {
		$x=0;
	} else {
		$x=$::ARG_verbose;
	}
	$::ARG_verbose=1;
	logmsg($_[0]);
	$::ARG_verbose=$x;
}

sub warnmsg($) {
my $x=$_[0];

	if(defined($::LOGMSGFD)) {
		select($::LOGMSGFD);
		$|=1;
	}
	if(defined($::LOGMSGFD)) {
		print "\n" if $need_newline;
	} else {
		print STDERR "\n" if $need_newline;
	}
	$need_newline=0;
	if(defined($::LOGMSGFD)) {
		print "Warn : $x\n";
	} else {
		print STDERR "Warn : $x\n";
	}
	if(defined($::LOGMSGFD)) {
		select(STDOUT);
		$|=1;
	}
}

sub errormsg {
my $x=$_[0];

	return if !defined($x);
	if(defined($::LOGMSGFD)) {
		select($::LOGMSGFD);
		$|=1;
	}
	print STDERR "\n" if $need_newline;
	print "\n" if $need_newline && defined($::LOGMSGFD);
	$need_newline=0;
	my @xx=split(/\n/m,$x);
	print STDERR "Error: $_\n" foreach(@xx);
	if(defined($::LOGMSGFD)) {
		print "Error: $_\n" foreach(@xx);
	}
	exit($_[1]) if defined($_[1]);
	if(defined($::LOGMSGFD)) {
		select(STDOUT);
		$|=1;
	}
}

#########################################################################
# Work out any changes to the list of dependencies that a particular	#
# package has required, as based on instructions in the specified 	#
# file.									#
#									#
# Arguments:								#
# 0	- comma separated list of original dependencies.		#
# 1	- filename of potentially new instructions.			#
#									#
#########################################################################

sub update_deps_from_file($$) {
my ($odeps,$file)=@_;
my ($ndeps,$fd,$cline,@cdeps,@F);

	$ndeps="";
	$ndeps=$odeps if defined($odeps);
	return($ndeps) if ! defined($file) || ! -f $file || ! -r $file;

	#################################################################
	# File exists and is readable, so process it.			#
	#################################################################

	@cdeps=split(/\s+/,$ndeps);
	open($fd,$file) or return($ndeps);
	while(<$fd>) {
		chomp;
		s/^\s+//;
		@F=split(/\s+/,$_);
		if(defined($F[0])) {
			$F[0]=uc($F[0]);
			if($F[0] eq "=DEPENDS") {
				$ndeps=defined($F[1]) ? $F[1] : "";
				last;
			}
			if($F[0] eq "-DEPENDS") {
				if(!defined($F[1]) || $F[1] eq "*") {
					$ndeps="";
				} else {
					my ($cc,$cc2,@tdeps);
					for $cc (@F[1..$#F]) {
						my @tdeps=();
						for $cc2 (@cdeps) {
							if($cc =~ /^$cc2$/ || $cc =~ /$cc2,/) {
								1;
							} else {
								push @tdeps,$cc2;
							}
						}
						@cdeps=@tdeps;
					}
				}
			}
			if($F[0] eq ">DEPENDS") {
				my ($cc,@tdeps,$f,$cc2);
				for $cc (@F[1..$#F]) {
					$f=0;
					my @tdeps=();
					for $cc2 (@cdeps) {
						if($cc =~ /^$cc2$/ || $cc =~ /$cc2,/) {
							push @tdeps,$cc;
							$f=1;
						} else {
							push @tdeps,$cc2;
						}
					}
					if(!$f) {
						push @tdeps,$cc;
					}
					@cdeps=@tdeps;
				}
			}
		}
	}
	close($fd);
	return $ndeps;
}

#########################################################################
# Will return a reference to a hash which stores the complete contents	#
# of the configuration file. The hash is in the following format:	#
#									#
# name			-> Name						#
# description		-> Description					#
# version		-> Package version				#
# [repository]		-> existing writable directory			#
# [preinstall]		-> script					#
# [preremove]		-> script					#
# [postremove]		-> script					#
# [postinstall]		-> script					#
# [checkinstall]	-> script					#
# [os]			-> pattern					#
# [architecture]	-> pattern					#
# [dependencies->pkgname->{minvar,maxver}]				#
# [incompatibles->pkgname->{minvar,maxver}]				#
# [altrepos]		-> Space separated list				#
# files			-> list (each a hash reference):		#
#				[perms],[owner],[group],dest		#
# dirs			-> list (each a hash reference):		#
#				[perms],[owner],[group],dest		#
#									#
# Will return:								#
# (hash,undef)	- everything ok						#
# (nudef,txt)	- error - see txt					#
#########################################################################

sub _handle_start {
shift;
$celement=shift;
%attrs=@_;

	$data="";
}

sub _handle_end {
my $expat=shift;
my $celement=shift;
my @ss=qw(name description preinstall preremove postinstall checkinstall postremove
		firstboot os architecture version altrepos repository);

	if(scalar(grep {/^$celement$/} @ss)) {
		$obj->{lc($celement)}=$data;
		return;
	}

	#################################################################
	# Handle file,dir and dependencies entries separately.		#
	#################################################################

	if($celement eq "dir") {
		if(!exists($obj->{dirs})) {
			$obj->{dirs}=[];
		}
		my $chash={};
		$chash->{dest}=$data;
		$chash->{perms}=$attrs{perms} if exists($attrs{perms});
		$chash->{owner}=$attrs{owner} if exists($attrs{owner});
		$chash->{group}=$attrs{group} if exists($attrs{group});
		push @{$obj->{dirs}},$chash;
	}

	if($celement eq "file") {
		if(!exists($obj->{files})) {
			$obj->{files}=[];
		}
		my $chash={};
		$chash->{dest}=$data;
		$chash->{perms}=$attrs{perms} if exists($attrs{perms});
		$chash->{owner}=$attrs{owner} if exists($attrs{owner});
		$chash->{group}=$attrs{group} if exists($attrs{group});
		$chash->{volatile}=$attrs{volatile} if exists($attrs{volatile});
		$chash->{configfile}=$attrs{configfile} if exists($attrs{configfile});
		push @{$obj->{files}},$chash;
	}

	if($celement eq "pkg") {
		my ($minver,$maxver);

		if(exists($attrs{minver})) {
			$minver=$attrs{minver};
		} else {
			$minver="0.0.1";
		}
		if(exists($attrs{maxver})) {
			$minver=$attrs{maxver};
		} else {
			$maxver="99999999.999999.999999";
		}

		if(!exists($obj->{dependencies})) {
			$obj->{dependencies}={};
		}
		if(!exists($obj->{incompatibles})) {
			$obj->{incompatibles}={};
		}
		if(scalar(grep {/^dependencies$/} ($expat->context))) {
			$obj->{dependencies}->{$data}->{minver}=$minver;
			$obj->{dependencies}->{$data}->{maxver}=$maxver;
		} else {
			$obj->{incompatibles}->{$data}->{minver}=$minver;
			$obj->{incompatibles}->{$data}->{maxver}=$maxver;
		}
	}
}

sub _handle_char {

	$data.=$_[1];
}

sub parse_pkgbuild_conf($) {
my $cname=$_[0];
my $P;
local ($obj,$celement,$data,%attrs);

	if(! -f "$cname") {
		return(undef,"File \"$cname\" not found.");
	}
	if(! -r "$cname") {
		return(undef,"File \"$cname\" not readable.");
	}
	$P=new XML::Parser(
			Handlers => {	Start	=> \&_handle_start,
					End	=> \&_handle_end,
					Char	=> \&_handle_char},
			ErrorContext => 2,
		);
		
	if(!defined($P)) {
		return(undef,"Unable to initiate XML parser.");
	}
	$obj={};
	eval {$P->parsefile($cname);};
	if($@) {
		return(undef,$@);
	}
	return($obj,undef);
}

#########################################################################
# Validates the specified configuration file from above has valid 	#
# entries, and no unknown tags.						#
# Also then validate all files referenced exist, including pre/post	#
# files.								#
#########################################################################

sub validate_pkgbuild_conf($) {
my $obj=$_[0];
my @errors=();
my @warnings=();
my @must_be_here=qw(name description version files dirs);
my @might_be_here=qw(preinstall postinstall preremove postremove 
			firstboot checkinstall os architecture dependencies
			incompatibles altrepos);
my @scriptlist=qw(checkinstall preinstall firstboot postinstall preremove postremove);

	for (@must_be_here) {
		push @errors,"Mandatory element \"$_\" not found." if !exists($obj->{$_});
	}

	for (sort(keys(%$obj))) {
		my $s=$_;
		next if scalar(grep {/^$s$/} @must_be_here);
		next if scalar(grep {/^$s$/} @might_be_here);
		push @warnings,"Unknown element \"$s\" found.";
	}

	#################################################################
	# Now some specific validation - check the scripts referenced	#
	# by the scriptlist exist and are executable.			#
	#################################################################

	for (@scriptlist) {
		next if !exists($obj->{$_});

		if(! -e $obj->{$_}) {
			push @errors,"File \"".$obj->{$_}."\" specified for \"$_\" does not exist.";
			next;
		}
		if(-e $obj->{$_} && ! -f $obj->{$_}) {
			push @errors,"File \"".$obj->{$_}."\" specified for \"$_\" is not a regular file.";
			next;
		}
		if(! -r $obj->{$_}) {
			push @errors,"File \"".$obj->{$_}."\" specified for \"$_\" is not readable.";
			next;
		}
	}

	#################################################################
	# Validate files ... dirs can not be unfortunately.		#
	#################################################################

	$obj->{ilist}={};
	for (@{$obj->{files}}) {
		my ($perms,$owner,$group,$dest);
		
		$dest=$_->{dest};
		if($dest =~ /=/) {
			($dest,$junk)=($dest =~ /(.*)=(.*)/);
			if(substr($junk,0,1) eq "/" || substr($junk,0,1) eq ".") {
				push @errors,"Destination specified for \"$dest\" is absolute ($junk)";
				next;
			}
			if($dest =~ /^\// || $dest =~ /^\.\//) {
				push @errors,"File \"$dest\" is not relative!";
				next;
			}
		}
		if(! -e $dest) {
			push @warnings,"File \"$dest\" does not exist.";
			$obj->{ilist}->{$_}=1;
			next;
		}
		if(! -f $dest) {
			push @warnings,"Object \"$dest\" is not a regular file.";
			$obj->{ilist}->{$_}=1;
			next;
		}
	}
	return(\@errors,\@warnings);
}

#########################################################################
# Given a valid object and the name of a directory, will create the 	#
# !!..config..!! file for this object in this directory.		#
#									#
# Will return 1 if OK, 0 if not able to create file.			#
#########################################################################

sub generate_pkg_config($$) {
my $obj=$_[0];
my $fname=$_[1];
my ($fd2,$fd);
my %mapp=qw(
	name			NAME
	description		DESCRIPTION
	repackaged		REPACKAGED
	version			PKGVER
	altrepos		ALTREPOS
	firstboot		FIRSTBOOT
	preinstall		PREINSTALL
	postinstall		POSTINSTALL
	checkinstall		CHECKINSTALL
	preremove		PREREMOVE
	postremove		POSTREMOVE
	os			OS
	architecture		ARCHITECTURE
	);

	if(!open($fd,">$fname/!!..config..!!")) {
		return 0;
	}
	
	#################################################################
	# Write out the simple entries first...				#
	#################################################################

	for (keys(%mapp)) {
		my $v;

		next if ! exists($obj->{$_});
		if($_ eq "architecture" && uc($obj->{$_}) eq "GENERIC") {
			next;
		}
		if($_ eq "architecture") {
			$v=$obj->{$_};
			$v =~ s/[\-\/]{1}/_/g;
		} else {
			$v=$obj->{$_};
		}
		$v =~ s/\n//g;
		$v =~ s/\s+/ /g;
		if(!print $fd $mapp{$_}." $v\n") {
			unlink("$fname/!!..config..!!");
			return 0;
		}
	}

	#################################################################
	# Now deal with dependencies and incompatibles entries...	#
	#################################################################

	my ($clist,$str);

	$str="";
	for (sort(keys(%{$obj->{dependencies}}))) {
		$str.="$_|".$obj->{dependencies}->{$_}->{minver}."|".$obj->{dependencies}->{$_}->{maxver}." ";
	}
	if(length($str)) {
		print $fd "DEPENDENCIES $str\n";
	}
	$str="";
	for (sort(keys(%{$obj->{incompatibles}}))) {
		$str.="$_|".$obj->{incompatibles}->{$_}->{minver}."|".$obj->{incompatibles}->{$_}->{maxver}." ";
	}
	if(length($str)) {
		print $fd "INCOMPATIBLES $str\n";
	}

	my $cscript;
	for $cscript ("firstboot","preinstall","postinstall","preremove","postremove","checkinstall") {
		if(defined($obj->{$cscript})) {
			print $fd "#!#!--START:$cscript--#!#!\n";
			open($fd2,$obj->{$cscript});
			while(<$fd2>) {
				print $fd $_;
			}
			close($fd2);
			print $fd "#!#!--END:$cscript--#!#!\n";
		}
	}

	print $fd "__FILELIST__\n";
	for  (@{$obj->{dirs}}) {
		$str="D|";
		if(exists($_->{perms})) { $str.=$_->{perms}."|"; } else { $str.="|"; }
		if(exists($_->{owner})) { $str.=$_->{owner}."|"; } else { $str.="|"; }
		if(exists($_->{group})) { $str.=$_->{group}."|"; } else { $str.="|"; }
		$str.=$_->{dest};
		print $fd "$str\n";
	}

	for (@{$obj->{files}}) {
		my ($dest,$junk);

		if(exists($obj->{ilist}->{$_})) { next; }
		$dest=$_->{dest};
		if($dest =~ /=/) {
			($dest,$junk)=($dest =~ /(.*)=(.*)/);
		}
		$str="F|";
		if(exists($_->{perms})) { $str.=$_->{perms}."|"; } else { $str.="|"; }
		if(exists($_->{owner})) { $str.=$_->{owner}."|"; } else { $str.="|"; }
		if(exists($_->{group})) { $str.=$_->{group}."|"; } else { $str.="|"; }
		if(exists($_->{volatile}) && lc(substr($_->{volatile},0,1)) eq "y") {
			$str.="|";
		} elsif(exists($_->{configfile}) && lc(substr($_->{configfile},0,1)) eq "y" ) {
			$x=file_sha1($dest);
			$str.="CFGFILE=$x|";
		} else {
			$x=file_sha1($dest);
			if(!defined($x)) {
				print STDERR "Unable to get sha1 of $dest\n";
				close($fd); 
				unlink("$fname/!!..config..!!");
				return 0;
			}
			$str.=file_sha1($dest) . "|";
		}
		$str.=(stat($dest))[7] . "|";
		if(defined($junk)) {
			$str.=$junk;
		} else {
			$str.=$_->{dest};
		}
		print $fd "$str\n";
	}
	close($fd);

	return 1;
}

#########################################################################
# This is different from teh above routine because it takes a		#
# !!..config..!! read object and recreates one after changes have 	#
# been made - used for repackaging.					#
#########################################################################

sub regenerate_pkg_config($$) {
my $obj=$_[0];
my $fname=$_[1];
my ($fd2,$fd);
my @fields=qw(
	NAME
	DESCRIPTION
	REPACKAGED
	PKGVER
	ALTREPOS
	FIRSTBOOT
	PREINSTALL
	POSTINSTALL
	CHECKINSTALL
	PREREMOVE
	POSTREMOVE
	OS
	ARCHITECTURE
	DEPENDENCIES
	INCOMPATIBLES
	);

	if(!open($fd,">$fname/!!..config..!!")) {
		return 0;
	}
	
	#################################################################
	# Write out the simple entries first...				#
	#################################################################

	for (@fields) {
		my $v;

		next if ! exists($obj->{$_});
		if($_ eq "ARCHITECTURE" && uc($obj->{$_}) eq "GENERIC") {
			next;
		}
		if($_ eq "ARCHITECTURE") {
			$v=$obj->{$_};
			$v =~ s/[\-\/]{1}/_/g;
		} else {
			$v=$obj->{$_};
		}
		$v =~ s/\n//g;
		$v =~ s/\s+/ /g;
		if(!print $fd $_." $v\n") {
			unlink("$fname/!!..config..!!");
			return 0;
		}
	}

	#################################################################
	# Deal with the script sections ...				#
	#################################################################

	my $cscript;
	for $cscript ("FIRSTBOOT","PREINSTALL","POSTINSTALL","PREREMOVE","POSTREMOVE","CHECKINSTALL") {
		if(defined($obj->{$cscript})) {
			print $fd "#!#!--START:",lc($cscript),"--#!#!\n";
			print $fd $obj->{$cscript};
			print $fd "#!#!--END:",lc($cscript),"--#!#!\n";
		}
	}

	print $fd "__FILELIST__\n";
	for  (@{$obj->{DIRS}}) {
		print $fd "$_\n";
	}

	for (@{$obj->{FILES}}) {
		print $fd "$_\n";
	}
	close($fd);
	return 1;
}

sub spool_files_to_tmpdir($$) {
my $obj=$_[0];
my $tmpdir=$_[1];
my ($junk,$fname,$destdir,$dest);

	for (@{$obj->{files}}) {
		
		if(exists($obj->{ilist}->{$_})) { next; }
		$dest=$_->{dest};
		if($dest =~ /=/) {
			($dest,$junk)=($dest =~ /(.*)=(.*)/);
			$destdir=dirname($junk);
			$destfile=basename($junk);
		} else {
			$destdir=dirname($dest);
			$destfile=basename($dest);
		}

		#########################################################
		# Create the destination directory path.		#
		#########################################################

		if(! -d "$tmpdir/$destdir") {
			make_dir_path("$tmpdir/$destdir");
		}

		my ($cszie,$err);
		($csize,$err)=copyfile($dest,"$tmpdir/$destdir/$destfile");
	}
	return 1;
}

sub get_pkg_default_repos($) {
my $obj=$_[0];

	if(!exists($obj->{repository})) {
		return "/tmp";
	}
	return $obj->{repository};
}

sub get_pkg_name($) {
my $obj=$_[0];
my $pkg;

	$pkg=$obj->{name};

	if(exists($obj->{os})) {
		my $os=$obj->{os};
		if(index($os,".") == -1 && index($os,"*") == -1 && index($os,"+") == -1) {
			$pkg.="+$os";
		}
	}
	if(exists($obj->{architecture})) {
		my $arch=$obj->{architecture};
		$arch =~ s/[\-\/]{1}/_/g;
		if(uc($arch) ne "GENERIC" && index($arch,".") == -1 && index($arch,"*") == -1 && index($arch,"+") == -1) {
			$pkg.="+$arch";
		}
	}
	$pkg.="+".$obj->{version};
	$pkg.=".tp2";
	return $pkg;
}

#########################################################################
# Check namespace security. The following routine will return two 	#
# references to lists. The first is for errors and the 2nd for warnings.#
#									#
# THIS ROUTINE SHOULD BE CALLED BEFORE ANY COMMAND DOES ANYTHING!	#
#########################################################################

sub check_ns_security() {
my ($dfd,$cdir,$cline,$found,$clog);
my ($fd,%nsdirs,@sb,$mode);
my (@errors,@warnings);

	@errors=();
	@warnings=();
	%nsdirs=();

	if(! -d "/var/adm/tp2/ns") {
		push @errors,"Top level namespace directory does not exist!";
		return(\@errors,\@warnings);
	}

	if(!opendir($dfd,"/var/adm/tp2/ns")) {
		push @errors,"Top level namspace directory is not readable!";
		return(\@errors,\@warnings);
	}

	foreach $cdir (readdir($dfd)) {
		next if $cdir eq "." || $cdir eq "..";
		next if ! -d "/var/adm/tp2/ns/$cdir";
		@sb=stat(_);
		$mode=$sb[2] & 0777;
		if($mode != 0755) {
			push @errors,"Namespace directory for $cdir does not have 755 permissions.";
			next;
		}
		#########################################################
		# Now grab the contents of the config file in this	#
		# namespace, checking the ROOT setting.			#
		#########################################################

		if(! -e "/var/adm/tp2/ns/$cdir/config") {
			push @errors,"Namespace directory for $cdir does not have a configuration file.";
			next;
		}

		if(! -f "/var/adm/tp2/ns/$cdir/config") {
			push @errors,"Namespace directory \"config\" for $cdir is not a regular file.";
			next;
		}
		@sb=stat(_);
		$mode=$sb[2] & 0777;
		if($mode != 0644) {
			push @errors,"Namespace configuration file for $cdir does not have 644 permissions.";
			next;
		}
		if(!open($fd,"/var/adm/tp2/ns/$cdir/config")) {
			push @errors,"Unable to open file /var/adm/tp2/ns/$cdir/config: $!";
			next;
		}
		$found=0;
		while($cline=<$fd>) {
			my @F;

			chomp $cline;
			@F=split(/\s+/,$cline);
			if($F[0] eq "ROOT") {
				$found=1;
				if(exists($nsdirs{$F[1]})) {
					push @errors,"Namespace $cdir uses a ROOT of $F[1] - which is already defined for namespace ".$nsdirs{$F[1]};
					last;
				}
				$nsdirs{$F[1]}=$cdir;
				last;
			}
		}
		if(!$found) {
			push @errors,"Namespace $cdir does not define a ROOT!";
		}
		#########################################################
		# Final check - ensure the subdirectories exist for	#
		# namespace:						#
		# log (0755) meta (0755) pkg (0755)			#
		#########################################################

		for $clog ("log","meta","pkg") {
			if(! -d "/var/adm/tp2/ns/$cdir/$clog") {
				push @errors,"Namespace $cdir does not have a $clog directory.";
			} else {
				@sb=stat(_);
				$mode=$sb[2] & 0777;
				if($mode != 0755) {
					push @errors,"Namespace $clog directory for $cdir does not have 755 permissions.";
				}
			}
		}
	}
	closedir($dfd);
	return(\@errors,\@warnings);
}

#########################################################################
# Will check to ensure that the current user has access to update the	#
# specified namespace. To do so they must own the meta,log and pkg	#
# sub-directories, and the ROOT directory for the namespace must exist.	#
#									#
# returns (1,undef) if OK, or (0,msg) if error.				#
#########################################################################

sub check_namespace_write($) {
my ($uname,@F,$ROOT,@sb,$cdir);
my $namespace=$_[0];

	if(! -d "/var/adm/tp2/ns/$namespace") {
		return(0,"Namespace \"$namespace\" does not exist.");
	}
	$uname=_getpwnam($UID);
	if(!open($fd,"/var/adm/tp2/ns/$namespace/config")) {
		return(0,"Namepace \"$namespace\" config file can not be opened.");
	}
	while(<$fd>) {
		chomp;
		@F=split;
		if($F[0] eq "ROOT") {
			$ROOT=$F[1];
			last;
		}
	}
	close($fd);
	if(!defined($ROOT)) {
		return(0,"Missing ROOT setting for namespace $namespace.");
	}
	$ROOT=~ s/\/+$//; $ROOT="/" if $ROOT eq "";	# remove trailing /'s

	if(! -d $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace does not exist.");
	}
	if(! -w $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace not writable.");
	}

	for $cdir ("pkg","meta","log") {
		@sb=stat("/var/adm/tp2/ns/$namespace/$cdir");
		if($sb[4] != $UID) {
			return(0,"Namespace \"$cdir\" directory for $namespace is not owned by user.");
		}
	}

	return(1,undef);
}

#########################################################################
# Converts a !!..config..!! file specified in the argument to an 	#
# hash containing the relevant details. This hash will use elements	#
# in UPPERCASE, including:						#
# DESCRIPTION, NAME, PKGVER, [ALTREPOS], [DEPENDENCIES],[INCOMPATIBLES]	#
#									#
# The script sections are included these will use the following	keys:	#
# POSTINSTALL, PREINSTALL, PREREMOVE, POSTREMOVE, CHECKINSTALL		#
#									#
# The return value is a two element list:				#
# (obj,undef) or (undef,errorstring)					#
#########################################################################

sub pkgfile_config_to_obj($) {
my $cfile=$_[0];
my ($state,$fd,$cline,$obj,$cdata);
my ($keyword,$value,$type);

	$obj={};
	$obj->{DIRS}=[];
	$obj->{FILES}=[];
	if(! -f "$cfile") {
		return(undef,"Can not find file $cfile.");
	}
	if(!open($fd,$cfile)) {
		return(undef,"Can not open file $cfile.");
	}
	$state=0;
	while($cline=<$fd>) {
		chomp $cline;
		if($state==0) {
			if($cline =~ /^#!#!--START:/ || $cline eq "__FILELIST__") {
				$state=1;
			} else {
				($keyword,$value)=($cline =~ /^([A-Z0-9]+)\s+(.*)/);
				$obj->{$keyword}=$value;
				next;
			}
		}
		if($state==1) {
			if($cline =~ /^#!#!--START:/) {
				($type)=($cline =~ /^#!#!--START:([a-z]+)-/);
				$cdata="";
				while($cline=<$fd>) {
					last if $cline =~ /^#!#!--END:/;
					$cdata.=$cline;
				}
				$obj->{uc($type)}=$cdata;
			}
			if($cline eq "__FILELIST__") {
				while($cline=<$fd>) {
					chomp $cline;
					if(substr($cline,0,1) eq "D") {
						push @{$obj->{DIRS}},$cline;
					}
					if(substr($cline,0,1) eq "F") {
						push @{$obj->{FILES}},$cline;
					}
				}
			}
		}
	}
	close($fd);
	return ($obj,undef);
}

#########################################################################
# if a file has been replaced in the meta informatino, then we re-add	#
# or replace the meta based on the current contents if a rollback is	#
# required.								#
#########################################################################

sub replace_namespace_file_meta($$$) {
my ($namespace,$fullpath,$package)=@_;
my ($moddedname,$sha1obj,$fpsha1,$cdir,@F,@entries);
my ($cfile,$fd,$f,$f2,$uname,$gname,$perms);

	$moddedname=$fullpath;
	$moddedname=~ s/\/{2,}/\//g;
	$sha1obj=Digest::SHA1->new;
	$sha1obj->add($moddedname);
	$fpsha1=$sha1obj->hexdigest;
	$cdir="/var/adm/tp2/ns/$namespace/meta/".substr($fpsha1,0,2);
	if(! -d "$cdir" || ! -f "$cdir/$fpsha1") {
		return(1,undef);
	}

	#################################################################
	# Open the file and read in entries...				#
	#################################################################

	@entries=();
	$cfile="$cdir/$fpsha1";
	if(!open($fd,"$cfile")) {
		return(0,"Unable to read existing meta data entry $cfile");
	}
	@entries=<$fd>; close($fd);
	chomp foreach (@entries);

	#################################################################
	# Scan the entries looking for this one, setting $f to it.	#
	#################################################################

	$f2=0; $f=-1;
	while($f2<=$#entries) {
		@F=split(/\|/,$entries[$f2]);
		if($F[0] eq $moddedname && $F[5] eq $package) {
			$f=$f2; last;
		}
		$f2++;
	}
	return if $f== -1;

	#################################################################
	# We only get here if $f2 points to element for this file that	#
	# needs to be updated, so firstly get the sha1 sum of the file.	#
	#################################################################

	$sha1obj=Digest::SHA1->new;
	open($fd,$fullpath) or
		return(0,"Unable to read file $fullpath to update meta information.");
	$sha1obj->addfile($fd);
	close($fd);
	@sb=stat($fullpath);
	if(scalar(@sb)<6) {
		return(0,"Unable to get file $fullpath stat information.");
	}
	
	$gname=_getgrgid($sb[5]); $uname=getpwuid($sb[4]);
	$gname=$sb[5] if !defined($gname);
	$uname=$sb[4] if !defined($uname);
	$perms=sprintf("%o",oct($sb[2] & 07777));
	$entries[$f2]="$moddedname|" . $sha1obj->hexdigest . "|$uname|$gname|$perms|$package";

	#################################################################
	# Only this file on the system with this sha1, so remove file.	#
	#################################################################

	if(!open($fd,">$cdir/$fpsha1.tmp")) {
		return(0,"Unable to open temporary meta file.");
	}
	foreach (@entries) {
		if(! print $fd "$_\n") {
			unlink("$cdir/$fpsha1.tmp");
			return(0,"Unable to write to temporary meta file.");
		}
	}
	close($fd);
	if(!rename("$cdir/$fpsha1.tmp","$cdir/$fpsha1.new")) {
		unlink("$cdir/$fpsha1.tmp");
		return(0,"Unable to rename temporary meta file.");
	}
	if(!rename("$cdir/$fpsha1","$cdir/$fpsha1.old")) {
		unlink("$cdir/$fpsha1.new");
		return(0,"Unable to rename oringal meta file.");
	}
	if(!rename("$cdir/$fpsha1.new","$cdir/$fpsha1")) {
		return(0,"Unable to rename oringal meta file.");
	}
	unlink("$cdir/$fpsha1.old");
	return(1,undef);
}

#########################################################################
# if a file has been aded to the meta, but needs to be removed, since	#
# the package installed failed then this routine is called with		#
# namespace,fullpath,package.						#
#########################################################################

sub remove_namespace_file_meta($$$) {
my ($namespace,$fullpath,$package)=@_;
my ($moddedname,$sha1obj,$fpsha1,$cdir,@F,@entries);
my ($cfile,$fd,$f,$f2);

	$moddedname=$fullpath;
	$moddedname=~ s/\/{2,}/\//g;
	$sha1obj=Digest::SHA1->new;
	$sha1obj->add($moddedname);
	$fpsha1=$sha1obj->hexdigest;
	$cdir="/var/adm/tp2/ns/$namespace/meta/".substr($fpsha1,0,2);
	if(! -d "$cdir" || ! -f "$cdir/$fpsha1") {
		return(1,undef);
	}

	#################################################################
	# Open the file and read in entries...				#
	#################################################################

	@entries=();
	$cfile="$cdir/$fpsha1";
	if(!open($fd,"$cfile")) {
		return(0,"Unable to read existing meta data entry $cfile");
	}
	@entries=<$fd>; close($fd);
	chomp foreach (@entries);

	#################################################################
	# Scan the entries looking for this one, setting $f to it.	#
	#################################################################

	$f2=0; $f=-1;
	while($f2<=$#entries) {
		@F=split(/\|/,$entries[$f2]);
		if($F[0] eq $moddedname && $F[5] eq $package) {
			$f=$f2; last;
		}
		$f2++;
	}
	if($f>-1) {
		splice(@entries,$f,1);
	}

	#################################################################
	# Only this file on the system with this sha1, so remove file.	#
	#################################################################

	if(!scalar(@entries)) {
		unlink("$cdir/$fpsha1");
		return(1,undef);
	}

	if(!open($fd,">$cdir/$fpsha1.tmp")) {
		return(0,"Unable to open temporary meta file.");
	}
	foreach (@entries) {
		if(! print $fd "$_\n") {
			unlink("$cdir/$fpsha1.tmp");
			return(0,"Unable to write to temporary meta file.");
		}
	}
	close($fd);
	if(!rename("$cdir/$fpsha1.tmp","$cdir/$fpsha1.new")) {
		unlink("$cdir/$fpsha1.tmp");
		return(0,"Unable to rename temporary meta file.");
	}
	if(!rename("$cdir/$fpsha1","$cdir/$fpsha1.old")) {
		unlink("$cdir/$fpsha1.new");
		return(0,"Unable to rename oringal meta file.");
	}
	if(!rename("$cdir/$fpsha1.new","$cdir/$fpsha1")) {
		return(0,"Unable to rename oringal meta file.");
	}
	unlink("$cdir/$fpsha1.old");
	return(1,undef);
}

#########################################################################
# This will update or remove the meta information for the specified 	#
# file in the specified namespace. If more than a single package has	#
# this file defined then it will just be removed from the specified	#
# package, whilst the remaining entries will have the SHA1 sum modified	#
# to the latest value to ensure all are correct.			#
# Arguments:								#
# 0 - Namespace reference						#
# 1 - relative filename to remove meta information for			#
# 2 - package name to remove information for				#
# Returns:								#
# 0 - 0 = error, 1 = OK							#
# 1 - undef when OK, msg when error					#
#########################################################################

sub rm_file_meta($$$) {
my ($nsobj,$file,$pkgname)=@_;
my $r=$nsobj->{ROOT};
my $namespace=$nsobj->{NAME};
my (@new,$new_sha1,@entries,@F,$moddedname,$fpsha1,$sha1obj,$cdir,$x,$cfile,$fd,$wfd);

	$moddedname="$r/$file";
	$moddedname=~ s/\/{2,}/\//g;
	$sha1obj=Digest::SHA1->new;
	$sha1obj->add($moddedname);
	$fpsha1=$sha1obj->hexdigest;
	$cdir="/var/adm/tp2/ns/$namespace/meta/".substr($fpsha1,0,2);
	if(! -d "$cdir") {
		return(1,undef);
	}
	@entries=();
	$cfile="$cdir/$fpsha1";
	if(-f $cfile) {
		if(!open($fd,"$cfile")) {
			return(0,"Unable to read existing meta data entry $cfile");
		}
		@entries=<$fd>; close($fd);
		chomp foreach (@entries);
	}

	#################################################################
	# Get the last sha1 info, since this will be used to update any	#
	# other references to this file if more than one.		#
	#################################################################

	@F=split(/\|/,$entries[$#entries]);
	$new_sha1=$F[1];

	@new=();
	for (@entries) {
		@F=split(/\|/,$_);
		if($F[0] ne $moddedname) {
			push @new,$_;
		} else {
			if($F[5] ne $pkgname) {
				$F[1]=$new_sha1;
				push @new,join("|",@F);
			}
		}
	}

	#################################################################
	# So entries are the new entries to write to the file.		#
	#################################################################

	if(scalar(@new)) {
		open($wfd,">$cfile.tmp") or
			return(0,"Unable to open file $cfile.tmp: $!");
		print $wfd "$_\n" foreach(@new); close($wfd);
		rename ($cfile,"$cfile.tmp2") or 
			return(0,"Unable to replace $cfile: $!");
		if(! rename("$cfile.tmp","$cfile")) {
			rename("$cfile.tmp2","$cfile");
			return(0,"Unable to replace $cfile: $!");
		}
		unlink("$cfile.tmp2");
	} else {
		unlink("$cfile");

		#########################################################
		# Enhancement: Check if the directory is empty and if	#
		# 	       so remove the meta data directory.	#
		#########################################################
		my ($dfd,$cfile,$count);
		if(opendir($dfd,$cdir)) {
			$count=0;
			while($cfile=readdir($dfd)) {
				$count++ if -f "$cdir/$cfile";
			}
			closedir($dfd);
			rmdir($cdir) if ! $count;
		}
	}
	return(1,undef);
}

#########################################################################
# The below function attempts to clean up after a failed package	#
# removal. 								#
# Arguments:								#
# 0 - name or reference for namespace					#
# 1 - name or refernece for package					#
# Returns:								#
# (0,msg)	- Clean up failed - see msg				#
# (1,undef)	- Clean up worked					#
#########################################################################

sub pkg_remove_cleanup($$) {
my ($nsobj,$pkgobj)=@_;
my ($x,$y,$z,$name,$pkgname);
my ($fd,$action,$cfile,$root);

	#################################################################
	# If not passed a hash then generate one from the appropriate	#
	# subroutine call.						#
	#################################################################

	if(!ref($nsobj)) {
		($x,$y)=namespace_to_obj($nsobj);
		if(!defined($x)) {
			return(0,$y);
		}
		($nsobj,$name)=($x,$nsobj);
	} else {
		$name=$nsobj->{NAME};
	}

	if(!ref($pkgobj)) {
		$x=$y=undef;
		if(-f "/var/adm/tp2/ns/$name/pkg/$pkgobj removing \$\$\$") {
			($x,$y)=pkgfile_config_to_obj("/var/adm/tp2/ns/$name/pkg/$pkgobj removing \$\$\$");
		}
		if(-f "/var/adm/tp2/ns/$name/pkg/$pkgobj \$\$\$ removed") {
			($x,$y)=pkgfile_config_to_obj("/var/adm/tp2/ns/$name/pkg/$pkgobj \$\$\$ removed");
		}
		if(!defined($x)) {
			return(0,"Unable to find package configuration file") if !defined($y);
			return(0,$y);
		}
		($pkgobj,$pkgname)=($x,$pkgobj);
	} else {
		$pkgname=$pkgobj->{NAME};
	}

	$root=$nsobj->{ROOT};
	my $stage=0;
	if(-f "/var/adm/tp2/ns/$name/pkg/$pkgname removing \$\$\$") {
		$stage=1;
	} else {
		$stage=2;
	}

	#################################################################
	# If stage==1 then do the necessary renames now...		#
	#################################################################
	if($stage==1) {
		dprint "Stage 1 recovery taking place\n";
		if(-f "/var/adm/tp2/ns/$name/pkg/$pkgname removing \$\$\$ trans" &&
			open($fd,"/var/adm/tp2/ns/$name/pkg/$pkgname removing \$\$\$ trans")) {
			while(<$fd>) {
				chomp;
				dprint "Transaction entry:$_\n";
				$action=substr($_,0,2);
				$cfile=substr($_,3);
				if($action eq "RM") {
					if(!rename("$root/$cfile.remove.\$\$","$root/$cfile")) {
						return(0,"Unable to rename $cfile.remove.\$\$ to $cfile.");
					}
					dprint "moved $root/$cfile.remove.\$\$ to $root/$cfile\n";
				}
				#########################################
				# Restore user-altered config file.	#
				#########################################
				if($action eq "RS") {
					my ($newf,$origf)=split(/\|/,$cfile);
					if(-f "$ROOT/$newf") {
						if(-f "$ROOT/$origf") { unlink "$ROOT/$origf"; }
						if(!rename("$ROOT/$newf","$ROOT/$origf")) {
							return(0,"Unable to rename file $newf back to $origf.");
						}
					}
				}
				if($action eq "UM" || $action eq "RM") {
					my ($mfile,@F,$f);
					$f=0;
					for $mfile (@{$pkgboj->{FILES}}) {
						@F=split(/\|/,$mfile);
						if($F[6] eq $cfile) {
							$f=1; last;
						}
					}
					if($f) {
						($x,$y)=update_namespace_file_meta($name,"$root/$cfile",$F[4],$F[2],$F[3],$F[1],$pkgname);
						if(!$x) {
							errormsg("Unable to update meta information for $cfile:\n$y");
						}
					}
					dprint "Updated namespace meta for $root/$cfile\n";
				}
				#################################
				# Ignore directories 		#
				#################################
			}
			close($fd);
		}

		#########################################################
		# All files renamed and all meta-info added back ...	#
		#########################################################
		unlink("/var/adm/tp2/ns/$name/pkg/$pkgname removing \$\$\$ trans");
		if(!rename("/var/adm/tp2/ns/$name/pkg/$pkgname removing \$\$\$",
			"/var/adm/tp2/ns/$name/pkg/$pkgname")) {
			return(0,"Unable to rename package configuration file.");
		}
		return(1,undef);
	}

	#################################################################
	# Below work is for stage 2 installations - must attempt to	#
	# complete removal now...					#
	#################################################################

	($x,$y)=remove_pkg_files_2(NAMESPACE	=> $name,
				   OBJ		=> $pkgobj );
	return($x,$y);
}

sub get_file_meta($$) {
my ($fullpath,$namespace)=@_;
my ($moddedname,$sha1obj,$fpsha1,$cdir,$cline,$ll,$fd);

	my @z;
	$moddedname=$fullpath;
	$moddedname=~ s/\/{2,}/\//g;
	if(exists($SHA1_MAP{$fullpath})) {
		@z=grep {/^$moddedname\|/} @F;
		if(! @z) {
			dprint "get_file_meta: Unable to find $moddedname in file.\n";
			return undef;
		}
		return($z[$#z]);
	}

	$sha1obj=Digest::SHA1->new;
	$sha1obj->add($moddedname);
	$fpsha1=$sha1obj->hexdigest;
	$cdir="/var/adm/tp2/ns/$namespace/meta/".substr($fpsha1,0,2);
	if(! -f "$cdir/$fpsha1" || ! -r "$cdir/$fpsha1") {
		dprint "get_file_meta: File $cdir/$fpsha1 does not exist [file=$moddedname].\n";
		return(undef);
	}
	return(undef) if !open($fd,"$cdir/$fpsha1");
	my @F=(map {chomp;$_;} <$fd>); close($fd);
	$SHA1_MAP{$fullpath}=[@F];
	@z=grep {/^$moddedname\|/} @F;
	if(! @z) {
		dprint "get_file_meta: Unable to find $moddedname in file.\n";
		return undef;
	}
	return($z[$#z]);
}

#########################################################################
# The below routine is called whenever a file is updated. It will take	#
# the following arguments:						#
# namespace:	The namespace being updated				#
# fullpath:	Full path to add into namespace				#
# sha1		The sha1sum of the file contents			#
# user		The uid of the file					#
# group		The gid of the file					#
# perms		The permissions to use					#
# package	The package the file now belongs to			#
#									#
# Return value: (1,undef) if OK or (0,msg) if error.			#
#########################################################################

sub update_namespace_file_meta($$$$$$$) {
my ($namespace,$fullpath,$sha1,$user,$group,$perms,$package)=@_;
my ($fpsha1,$sha1obj,$moddedname,$cdir,$cfile,$x,@entries);
my (@F,$f);

	$moddedname=$fullpath;
	$moddedname=~ s/\/{2,}/\//g;
	$sha1obj=Digest::SHA1->new;
	$sha1obj->add($moddedname);
	$fpsha1=$sha1obj->hexdigest;
	$cdir="/var/adm/tp2/ns/$namespace/meta/".substr($fpsha1,0,2);
	if(! -d "$cdir") {
		$x=mkdir($cdir);
		if(!$x) {
			return(0,"Unable to create directory $cdir: $!");
		}
		chmod 0755,$cdir;
	}
	@entries=();
	$cfile="$cdir/$fpsha1";
	if(-f $cfile) {
		if(!open($fd,"$cfile")) {
			return(0,"Unable to read existing meta data entry $cfile");
		}
		@entries=<$fd>; close($fd);
		chomp foreach (@entries);
	}

	#################################################################
	# Even if an entry exists the original one is removed to ensure	#
	# it appears at the end of the file. This ensures that if	#
	# multiple packages include the same file the most recent	#
	# checksum is the last entry for that file.			#
	#################################################################
	
	my @newentries=();
	$f=0;
	for $cent (@entries) {
		@F=split(/\|/,$cent);
		if($F[0] eq $moddedname && $F[5] eq $package) {
			next;
		} 
		push @newentries,$cent;
	}
	if(!defined($user)) {
		$user="";
	}
	if(!defined($group)) {
		$group="";
	}
	push @newentries,"$moddedname|$sha1|$user|$group|$perms|$package";
	@entries=@newentries;

	#################################################################
	# Now replace/create the meta file.				#
	#################################################################
	if(-f $cfile) {
		$x=rename "$cfile","$cfile.old";
		if(!$x) {
			return(0,"Unable to temporarily rename existing file.");
		}
	}
	if(!open($fd,">$cfile")) {
		if(-f "$cfile.old") {
			rename("$cfile.old",$cfile);
			return(0,"Unable to open meta file $cfile for writing.");
		}
	}
	foreach (@entries) {
		$x=$_; chomp $x;
		if(! print $fd "$x\n") {
			close($fd);
			unlink($cfile);
			if(-f "$cfile.old") {
				rename("$cfile.old",$cfile);
				return(0,"Unable to write to meta file $cfile.");
			}
		}
	}
	if(!close($fd)) {
		if(! print $fd $x) {
			close($fd);
			unlink($cfile);
			if(-f "$cfile.old") {
				rename("$cfile.old",$cfile);
				return(0,"Unable to write to meta file $cfile.");
			}
		}
	}
	chmod 0644,$cfile;
	if(-f "$cfile.old") {
		unlink "$cfile.old";
	}
	return(1,undef);
}

sub namespace_to_obj($) {
my $ns=$_[0];
my (@F,$obj,$name,$setting);

	$obj={};
	if(! -f "/var/adm/tp2/ns/$ns/config") {
		return(undef,"Namespace configuration file for namespace $ns does not exist.");
	}
	if(!open($fd,"/var/adm/tp2/ns/$ns/config")) {
		return(undef,"Unable to open namespace configuration file for $ns.");
	}
	while(<$fd>) {
		($name,$setting)=(/(^[A-Z_0-9]+)\s+(.*)/);
		next if !defined($name);
		$obj->{$name}=$setting;
	}
	close($fd);
	$obj->{NAME}=$ns;
	return($obj,undef);
}

#########################################################################
# This routine will install all the files for a package. As per the 	#
# design document it does so in a very controlled manner to allow a 	#
# recovery if a problem arises.						#
#									#
# Also it will only attempt an install of a file if THE FILE TO 	#
# INSTALL HAS DIFFERENT CONTENT via SHA1 check.				#
#									#
# The routine uses keyname:value pairs since it requires a number of	#
# parameters:								#
# SPOOLDIR	Directory where package contents are currently spooled	#
# 		to. Including in this top directory will be the		#
#		!!..config..!! config file.				#
# NAMESPACE	Namespace to install file into. The value of the	#
#		ROOTINSTALLABLE attribute is not checked here, the	#
#		caller is expected to do that.				#
# [OBJ]		Reference to package config obect - optional.		#
# [HOOK]	The name of a routine to call after every successfull	#
#		file install - spaced files installed, space used.	#
# [PREVIEW]	If exists and set to 1, then NO FILES are installed.	#
#		However, will call the HOOK function, if any file	#
#		does need to be installed.				#
# [LOGMSG]	Alternative routine to call with log messages.		#
# [WARNMSG]	Alternative routine to call with warnings.		#
# [ERRORMSG]	Alternative routine to call with errors.		#
#									#
# Returns (1,undef)	- files installed as requested.			#
# 	  (0,msg)	- error ... see msg				#
#									#
# Config file handling:							#
#									#
# If the file already exists, but the SHA1 info is set to CFGFILE=...	#
# then it indicates the file is user-updatable and the following rules	#
# are considered when they are 'updated' (as per pacman and rpm):	#
#									#
# [1] original=X, current=X, new=X					#
#     All three files are the same, no action taken.			#
# [2] original=X, current=X, new=Y					#
#     The current file is un-altered from the original but the new one 	#
#     is different. Since the user did not ever modify the file, and  	#
#     thenew one may contain improvements/bugfixes, install it.		#
# [3] original=X, current=Y, new=X					#
#     Both package versions contain the exact same file, but the one 	#
#     on the filesystem has been modified since - so leave it.		#
# [4] original=X, current=Y, new=Y					#
#     The new one is identical to the current one - so no action.	#
# [5] original=X, current=Y, new=Z					#
#     All three are different, so save current to .previous and install	#
#     the new one.							#
#									#
# Since configuration files tend to be important in the case of [3],	#
# and [5] a message will be given to indicate the action.		#
#									#
#########################################################################

{
my %space_used=();
my $files_installed=$files_replaced=$files_kept=0;

sub install_pkg_files {
my %args=@_;
my ($spooldir,$namespace,$hook,$package);
my ($obj,$bytes,$msg,$fd,$cline,@F,$ROOT);
my ($wfd,$xx,$cdir,$cfile);
my ($uname,$gname,$nsobj,$preview);
my ($warnmsg,$errmsg,$logmsg);

	$files_installed=$files_replaced=$files_kept=0;
	$warnmsg=\&warnmsg;
	$errormsg=\&errormsg;
	$logmsg=\&logmsg;

	$warnmsg=$args{WARNMSG} if exists($args{WARNMSG});
	$errmsg=$args{ERRORMSG} if exists($args{ERRORMSG});
	$logmsg=$args{LOGMSG} if exists($args{LOGMSG});

	$uname=getpwuid($UID); $gname=_getgrgid(getgid());
	$hook=undef; $preview=0;
	if(!exists($args{SPOOLDIR}) || !exists($args{NAMESPACE})) {
		return(0,"Missing mandatory arguments.");
	}

	$preview=$args{PREVIEW} if exists($args{PREVIEW});
	$spooldir=$args{SPOOLDIR};
	$namespace=$args{NAMESPACE};
	$hook=$args{HOOK} if exists($args{HOOK});

	if(exists($args{OBJ})) {
		$obj=$args{OBJ};
	} else {
		($obj,$err)=pkgfile_config_to_obj("$spooldir/!!..config..!!");
		return(0,$err) if defined($err);
	}

	($nsobj,$err)=namespace_to_obj($namespace);

	if(!exists($nsobj->{ROOT})) {
		return(0,"Missing ROOT setting for namespace $namespace.");
	}
	$ROOT=$nsobj->{ROOT};
	$ROOT=~ s/\/+$//; $ROOT="/" if $ROOT eq "";	# remove trailing /'s

	if(! -d $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace does not exist.");
	}
	if(! -w $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace not writable.");
	}

	$package=$obj->{NAME};

	#################################################################
	# Before continuing, copy the !!..config..!! file to the	#
	# "pkgname install $$$" in the namespace pkg directory.		#
	# Also open a file for writing called 				#
	# "pkgname install $$$ trans" to recovery the transaction state	#
	# of the installation.						#
	#################################################################

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$") {
		return(0,"Previous failed installation not cleaned up - use tp2clean first!");
	}
	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package") {
		&$logmsg("Existing package - implicit upgrade/downgrade will be performed.");
	}
	($bytes,$msg)=copyfile("$spooldir/!!..config..!!","/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$");
	if(defined($msg)) {
		return(0,"Unable to create temporary file in package configuration directory for namespace $namespace.");
	}

	if(!open($wfd,">/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans")) {
		unlink("/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$");
		return(0,"Unable to create transaction file for package installation.");
	}

	#################################################################
	# Ensure all changes to wfd are flushed ASAP			#
	#################################################################

	$xx=select $wfd;
	$|=1;
	select $xx;
	#################################################################
	# Now all directories are created, the order being based on	#
	# the shortest length paths first.				#
	#################################################################

	for $cdir (sort {my (@c,@d); @c=split(/\|/,$a); @d=split(/\|/,$b); length($c[4]) <=> length($d[4])} @{$obj->{DIRS}}) {
		my $t;
		@F=split(/\|/,$cdir);
		$t="$ROOT/$F[4]";
		if(! -d $t && !$preview) {
			if(!mkdir $t) {
				close($wfd);
				return(0,"Unable to create directory \"$t\". : $!");
			}
			if(!print $wfd "RD $F[4]\n") {
				close($wfd);
				return(0,"Unable to write to transaction file.");
			}
			if($F[1] ne "") {
				if(!chmod oct("0$F[1]"),$t) {
					&$warnmsg("Unable to set permissions of $t to $F[1].");
				}
			}
			if($UID==0) {
				my ($gg,$uu);
				if($F[3] ne "") {
					if($F[3] !~ /^[0-9]+$/) {
						$gg=_getgrnam($F[3]);
						if(!defined($gg)) {
							&$warnmsg("Can not change group to $F[3] for $t.");
							$gg=-1;
						}
					} else {
						$gg=$F[3];
					}
				} else {
					$gg=-1;
				}
				if($F[2] ne "") {
					if($F[2] !~ /^[0-9]+$/) {
						$uu=_getpwnam($F[2]);
						if(!defined($uu)) {
							&$warnmsg("Can not change owner to $F[2] for $t.");
							$uu=-1;
						}
					} else {
						$uu=$F[2];
					}
				} else {
					$uu=-1;
				}
				if($gg != -1 || $uu !=  -1) {
					if(!chown $uu,$gg,$t) {
						&$warnmsg("Unable to change permissions to $uu:$gg for $i.");
					}
				}
			}
		}
	}

	#################################################################
	# At this point all necessary directories referenced explicitly	#
	# in the package configuration will have been created, so	#
	# now attempt to install the files...				#
	#################################################################

	my $total_files=scalar(@{$obj->{FILES}});

	%space_used=(); # new files will allocate all space, existing
		   # files just the different [in blocks], and
		   # removed files are -size
		   # This can then be used to remove an 
		   # install summary, so that option is required

	#################################################################
	# As of 0.7.0 we estimate the impact of the actual install	#
	# as well ... which is different from the max install space	#
	# required [which saves all files, even if no changes are 	#
	# actually to take place].					#
	#################################################################

	$fcount=0;
	for $cfile (@{$obj->{FILES}}) {
		my $instfile;

		@F=split(/\|/,$cfile);
		$t="$ROOT/$F[6]";
		$t =~ s/\/{2,}/\//g;
		if(! -d dirname($t) && !$preview) {
			my $cpath="$ROOT/";
			my $cpath2="";
			my @D;
			@D=split(/\//,dirname($F[6]));
			for $xx (@D) {
				next if $xx eq "";
				$cpath.="$xx/";
				$cpath2.="$xx/";
				if(! -d $cpath) {
					if(!mkdir $cpath) {
						close($wfd);
						return(0,"Unable to create directory \"$cpath\".");
					}
					$cpath2 =~ s/\/$//;
					if(!print $wfd "RD $cpath2\n") {
						close($wfd);
						return(0,"Unable to write to transaction file.");
					}
				}
			}
		}

		$instfile=1;
		#########################################################
		# The directory to place the file now exists, so also	#
		# attempt to add the file ... actions depend on 	#
		# file is new or already exists.			#
		#########################################################
		my ($install_action,$install_file);
		$install_file=$F[6]; $install_action="Install";
		my $new_file_size=$F[5];
		my $cfile_size=0; my $cblock_size=0;
		if(-f $t) {
			my @sbuff=stat(_); $cfile_size=$sbuff[7]; $cblock_size=$sbuff[11];

			my $nfile=dirname($t)."/".basename($t).".install.\$\$";
			my $csha1;
			if($F[4] =~ /^CFGFILE=/) {
				my ($orig,$curr,$new,$om,@OM,);
				&$logmsg("Handling $t as a configuration file:");
				$new=$F[4]; $new =~ s/^CFGFILE=//;
				$curr=file_sha1($t);
				$om=get_file_meta($t,$namespace);
				if(!defined($om)) {
					$orig="00000000000000000000000000000000";
				} else {
					@OM=split(/\|/,$om);
					$orig=$OM[1];
					$orig =~ s/^CFGFILE=//;
				}

				&$logmsg("  Original: $orig, Current: $curr, New: $new");
				if($orig eq $new && $orig eq $curr) {
					&$logmsg("  Same contents - no changes.");
					$install_action="Keep";
					$instfile=0;
				}
				if($orig eq $curr && $orig ne $new) {
					&$logmsg("  Original/Current unaltered - so replace.");
					$install_action="Update";
					$instfile=1;
				}
				if($orig eq $new && $orig ne $curr) {
					&$logmsg("  Original/New same, current updated - so leave it.");
					$install_action="Keep";
					$instfile=0;
				}
				if($curr eq $new && $curr ne $orig) {
					&$logmsg("  Current/New same, no changes.");
					$install_action="Keep";
					$instfile=0;
				}
				if($orig ne $curr && $orig ne $new) {
					&$logmsg("  All different, archive current to backup.");
					$install_action="Update";
					$instfile=2;
				}
			}
			if($F[4] ne "" && $F[4] !~ /^CFGFILE=/) {
				$csha1=file_sha1($t);
				if(defined($csha1) && $csha1 eq $F[4]) {
					$instfile=0;
					$install_action="Keep";
				} else {
					$install_action="Update";
					&$logmsg("Overwriting    $t [$csha1 -> $F[4]]");
				}
			}

			#################################################
			# Rename current file to .previous ready to	#
			# install new version.				#
			#################################################
			if($instfile==2) {
				my $space=0;
				$files_replaced++;
				if(-f "$t.previous.$$") {
					$space=(stat(_))[7];
					unlink("$t.previous.$$") if !$preview;
					$cfile_size-=$space;
				}

				if(!$preview) {
					if(!rename("$t","$t.previous.$$")) {
						close($wfd);
						return(0,"Unable to rename existing file $t.");
					}
					print $wfd "RS $cfile.restore.$$|$cfile\n";
				}
			}
			$files_kept++ if $instfile==0;
			if($instfile==1) {
				$files_replaced++;
				my ($current_blocks,$new_blocks);
				$current_blocks=int($cfile_size/$cblock_size);
				$new_blocks=int($F[5]/$cblock_size);
				$cfile_size=$cblock_size * ($new_blocks-$current_blocks);
				if(!$preview) {
					if(!print $wfd "RP $F[6]\n") {
						close($wfd);
						return(0,"Unable to write to transaction file.");
					}
					if(!rename($t,$nfile)) {
						close($wfd);
						return(0,"Unable to rename existing file $t.");
					}
				}
				#################################################
				# The size of the new size - size of new file	#
				# indicates amount of additonal space used...	#
				#################################################
			}
		} else {
			$files_installed++;
			$cfile_size=$F[5];
			&$logmsg("Installing     $t [0000000000000000000000000000000000000000 -> $F[4]]");
			if(!$preview) {
				if(!print $wfd "RM $F[6]\n") {
					close($wfd);
					return(0,"Unable to write to transaction file.");
				}
			}
		}
		if(!$preview && $instfile) {
			($bytes,$msg)=copyfile("$spooldir/$F[6]",$t);
			if(defined($msg)) {
				close($wfd);
				return(0,"Unable to install file $t:\n$msg");
			}
			my $blksz=(stat($t))[11];
			my $space_added;
			if($cfile_size % $blksz==0) {
				$space_added=$cfile_size;
			} else {
				$space_added=(1+int($cfile_size/$blksz))*$blksz;
			}
			$space_used{$t}=int($space_added/1024);
			#########################################################
			# The file has been installed - so set the permissions.	#
			#########################################################
			if($F[1] ne "") {
				if(!chmod oct("0".$F[1]),$t) {
					&$warnmsg("Unable to set permissions on $t to $F[1].");
				}
			}
			if($UID==0) {
				my ($fuid,$fgid);
				if($F[2] ne "") {
					$fuid=_getpwnam($F[2]);
					$fuid=-1 if !defined($fuid);
				}
				if($F[3] ne "") {
					$fgid=_getgrnam($F[3]);
					$fgid=-1 if !defined($fgid);
				}
				dprint "$F[2] == $fuid, $F[3] == $fgid\n";
				if(!chown $fuid,$fgid,$t) {
					&$warnmsg("Unable to set ownership/group to $F[2]/$F[3]");
				}
			}
		}

		$fcount++; $fsize+=$bytes;

		#########################################################
		# Now that the file has been installed the next step	#
		# is to update the meta-information for the file.	#
		#########################################################

		if(!$preview) {
			if($UID==0) {
				($err,$msg)=update_namespace_file_meta($namespace,$t,$F[4],$F[2],$F[3],$F[1],$package);
			} else {
				($err,$msg)=update_namespace_file_meta($namespace,$t,$F[4],$uname,$gname,$F[1],$package);
			}
			if(defined($msg)) {
				close($wfd);
				return(0,$msg);
			}
		}

		#########################################################
		# If we have a hook then call it after every successful	#
		# file installation.					#
		#########################################################

		if(defined($hook)) {
			&$hook($fcount,$fsize,$total_files,$install_file,$install_action);
		}
	}
	close($wfd);

	#################################################################
	# We exit with the transaction log intact, allowing the caller	#
	# to recover or clear-up after a successful install - depending	#
	# on the return code.						#
	#################################################################

	return(1,undef);
}

sub _get_space_actually_used {
	return {%space_used};
}

sub _get_summary_counts {
	return ($files_installed,$files_replaced,$files_kept);
}

}

#########################################################################
# This routine will extract the contents of the specified file into a	#
# temporary directory for the specified name space. Argumenta are:	#
# 0:	Absolute path of package file					#
# 1:	Namespace							#
#									#
# Returns(path,undef)	Name of temporary directory extracted to.	#
# or     (undef,msg)	Error reason of failure.			#
#########################################################################

sub extract_to_spool($$) {
my ($pkgfile,$namespace)=@_;
my ($nsobj,$err,$tmpdir,$ROOT);

	# Ensure the named file exists, and is readable.		#

	if(! -f $pkgfile) {
		return(undef,"Package file $pkgfile does not exist.");
	}
	if(! -r $pkgfile) {
		return(undef,"Package file $pkgfile is not readable.");
	}

	#################################################################
	# Get the TMPDIR directory for the specified namespace.	If not	#
	# specified will use a temporary directory relative to the	#
	# namespace root.						#
	#################################################################

	($nsobj,$err)=namespace_to_obj($namespace);
	if(!defined($nsobj)) {
		return(undef,$err);
	}

	if(!exists($nsobj->{ROOT})) {
		return(undef,"Missing ROOT setting for namespace $namespace.");
	}
	if(exists($nsobj->{TMPDIR})) {
			$tmpdir=$nsobj->{TMPDIR};
	}
	$ROOT=$nsobj->{ROOT};
	if(!defined($tmpdir)) {
		$tmpdir=$ROOT;
	}
	$tmpdir=~ s/\/+$//;	# remove trailing /'s

	#################################################################
	# Now create a temporary sub-directory here.			#
	#################################################################

	mkdir "$tmpdir/pkginstall.$$" or
		return(undef,"Unable to create temporary directory $tmpdir/pkginstall.$$: $!");

	my $pwd;
	$pwd=cwd;
	if(!chdir("$tmpdir/pkginstall.$$")) {
		remove_dir_path("$tmpdir/pkginstall.$$");
		return(undef,"Unable to make $tmpdir/pkginstall.$$ current directory.");
	}

	#################################################################
	# Extract the file contents...					#
	#################################################################

	$cmd=_get_tar_cmd()." xzf $pkgfile";
	system($cmd);
	if($?>>8) {
		chdir $pwd;
		remove_dir_path("$tmpdir/pkginstall.$$");
		return(undef,"Unable to extact file contents to $tmpdir/pkginstall.$$");
	}

	chdir $pwd;
	return("$tmpdir/pkginstall.$$",undef);
}

#########################################################################
# The following routine is called after the successful installation	#
# of the package files. It does the following:				#
#									#
# [1] Change the temporary directory name from "pkgname install $$$"	#
#     to "pkgname $$$ installed" - indicating files installed OK.	#
# [2] Check transaction lof "pkgname install $$$ trans", and for	#
#     each "RP file" (replace) entry, if a backup file called 		#
#     "file.install.$$" is found it should be removed. 			#
#     NOTE	Any file that can not be removed is placed in the	#
#		"/var/adm/tp2/ns/namespace/pkg/rm on boot" file.	#
# [3] The "pkgname install $$$ trans" file is removed.			#
# [4] If a package configuration file already exists then it removed.	#
#     [Calls the package_upgrade_clean first!]				#
# [5] The "pkgname $$$ installed" package name is changed to "pkgname".	#
#									#
# The routine uses keyname:value pairs since it requires a number of	#
# parameters:								#
# SPOOLDIR	Directory where package contents are currently spooled	#
# 		to. Including in this top directory will be the		#
#		!!..config..!! config file.				#
# PKGFILE	If the SPOOLDIR is missing then PKGFILE should contain	#
#		the name of the package file being processed.		#
# NAMESPACE	Namespace to install file into. The value of the	#
#		ROOTINSTALLABLE attribute is not checked here, the	#
#		caller is expected to do that.				#
# [OBJ]		Reference to package config obect - optional.		#
# [HOOK]	The name of a routine to call after every successfull	#
#		(filename, current file count, total files, action)	#
# [PREVIEW]	Set to 1 and nothing changes.				#
#									#
# Returns (1,undef) == OK, (0,msg) == ERROR				#
#########################################################################

sub pkg_installed_ok {
my %args=@_;
my ($spooldir,$namespace,$nsobj,$hook,$package,$pkgfile,$ROOT);
my ($obj,$err);

	$hook=undef;
	if(!exists($args{SPOOLDIR}) && !exists($args{PKGFILE})) {
		return(0,"Missing mandatory arguments.");
	}
	if(!exists($args{NAMESPACE})) {
		return(0,"Missing mandatory arguments.");
	}

	$spooldir=$args{SPOOLDIR} if exists($args{SPOOLDIR});
	$pkgfile=$args{PKGFILE} if exists($args{PKGFILE});
	$namespace=$args{NAMESPACE};
	$hook=$args{HOOK} if exists($args{HOOK});

	if(exists($args{OBJ})) {
		$obj=$args{OBJ};
	} else {
		if(defined($spooldir)) {
			($obj,$err)=pkgfile_config_to_obj("$spooldir/!!..config..!!");
		} else {
			($obj,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$pkgfile");
		}
		return(0,$err) if defined($err);
	}

	($nsobj,$err)=namespace_to_obj($namespace);
	if(!defined($nsobj)) {
		return(undef,$err);
	}
	if(!exists($nsobj->{ROOT})) {
		return(0,"Missing ROOT setting for namespace $namespace.");
	}
	$ROOT=$nsobj->{ROOT};
	$ROOT=~ s/\/+$//; $ROOT="/" if $ROOT eq "";	# remove trailing /'s

	if(! -d $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace does not exist.");
	}
	if(! -w $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace not writable.");
	}

	my ($oldcfg,$newcfg);
	my $upgrade=0; my $remove_count=0;
	$package=$obj->{NAME};
	if(exists($args{PREVIEW}) && $args{PREVIEW}==1) {
		if(-f "/var/adm/tp2/ns/$namespace/pkg/$package") {
			($oldcfg,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$package");
			if(!defined($oldcfg)) {
				warnmsg("Unable to read in previous config following package update.");
				warnmsg("Error was - \"$err\".");
				warnmsg("No antiquated files will be removed.");
			}
			($newcfg,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$");
			if(!defined($newcfg)) {
				warnmsg("Unable to read in current config following package update.");
				warnmsg("Error was - \"$err\".");
				warnmsg("No antiquated files will be removed.");
			}
			if(defined($newcfg) && defined($oldcfg)) {
				# use variables package,namespace,ROOT
				my (@F,@newfiles,$xxx);

				@newfiles=map {@F=split(/\|/); $F[6];} @{$newcfg->{FILES}};
				foreach $cfile (@{$oldcfg->{FILES}}) {
					@F=split(/\|/,$cfile);
					$xxx=$F[6];
					if(!scalar(grep {/^$xxx$/} @newfiles)) {
						if($F[4] !~ /^CFGFILE=/) {
							my ($plist,$err)=file_installed_by(
										$nsobj,$xxx,TP2_ALL_PACKAGES);
							if(defined($plist) && scalar(@$plist)==1) {
								if(!$remove_count) {
									logmsg("Files being removed due to package upgrade/downgrade:");
								}
								logmsg("  Removing $xxx - not needed by version " . $obj->{PKGVER} . " of $package.");
								$remove_count++;
							}
						}
					}
				}
				logmsg("  Files removed due to package upgrade/downgrade: $remove_count.") if $remove_count;
			}
		}
		unlink "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$";
		unlink "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans";
		return(1,undef);
	}

	#################################################################
	# Step 1: Rename the "pkgname install $$$" directory to		#
	#	  "pkgname $$$ installed".				#
	#################################################################

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$") {
		if(!rename("/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$",
			   "/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed")) {
			   return(0,"Unable to rename temporary package config file!");
		}
	}

	#################################################################
	# Step 2: If the "pkgname install $$$ trans" file exists, then	#
	#	  scan it for "RP file" entries and if any are found,	#
	#	  then remove the "file.install.$$" if exists, and if	#
	#	  fail append them to the "pkg/rm on reboot" list.	#
	#################################################################

	my ($fd,$wfd,$caction,$cfile,@F);
	my $count=0;

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans") {
		if(!open($fd,"/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans")) {
			return(0,"Unable to open transaction recovery file.");
		}
		if(!open($wfd,">>/var/adm/tp2/ns/$namespace/pkg/rm on reboot")) {
			return(0,"Unable to open reboot cleanup transaction file.");
		}
		my @_data=<$fd>; close($fd);
		my $_total_files=@_data;
		my $_cfile=0;
		for (@_data) {
			chomp;
			($caction,$cfile)=(/([A-Z]+) (.*)/);
			$_cfile++;
			if(defined($hook)) {
				&$hook($cfile,$_cfile,$_total_files,"Config");
			}
			if($caction eq "RP" && -f "$ROOT/$cfile.install.\$\$") {
				if(!unlink "$ROOT/$cfile.install.\$\$") {
					print $wfd "$cfile.install.\$\$\n";
					$count++;
				}
			}
		}
		close($wfd);
	}
	logmsg("Added $count files to post-boot clean list for namespace.") if $count;

	#################################################################
	# Step 3: Remove the pkgname install $$$ trans" if it exists.	#
	#################################################################

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans") {
		unlink("/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans") or
			return(0,"Unable to remove transaction file.");
	}

	#################################################################
	# Step 4: Remove existing package file, if applicable.		#
	#	  This involves a bit more however - the existing (now	#
	#	  old) package is renamed and then examined at the	#
	#	  very end, and any files no longer present can be	#
	#	  removed.						#
	#################################################################

	$upgrade=0; $remove_count=0;
	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package") {
		($oldcfg,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$package");
		if(!defined($oldcfg)) {
			warnmsg("Unable to read in previous config following package update.");
			warnmsg("No antiquated files will be removed.");
		}
		($newcfg,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed");
		if(!defined($newcfg)) {
			warnmsg("Unable to read in current config following package update.");
			warnmsg("No antiquated files will be removed.");
		}
		if(defined($newcfg) && defined($oldcfg)) {
			# use variables package,namespace,ROOT
			my (@F,@newfiles,$xxx);
			my $_total_files=@{$oldcfg->{FILES}};
			my $_cfile=0;

			@newfiles=map {@F=split(/\|/); $F[6];} @{$newcfg->{FILES}};
			my %_newfiles=();
			$_newfiles{$_}=1 foreach(@newfiles);
			foreach $cfile (@{$oldcfg->{FILES}}) {
				@F=split(/\|/,$cfile);
				$xxx=$F[6];
				$_cfile++;
				if(defined($hook)) {
					&$hook($xxx,$_cfile,$_total_files,"Cleanup");
				}
				if(!exists($_newfiles{$xxx})) {
					if($F[4] !~ /^CFGFILE=/) {
						if(!$remove_count) {
							logmsg("Files being removed due to package upgrade/downgrade:");
						}
						logmsg("  Removing $xxx - not needed by version " . $obj->{PKGVER} . " of $package.");
						$remove_count++;
						#########################
						# If not a config file 	#
						# then remove, if this	#
						# is only package 	#
						# using it...		#
						#########################
						my ($plist,$err)=file_installed_by(
									$nsobj,$xxx,TP2_ALL_PACKAGES);
						if(defined($plist)) {
							($upgrade,$err)=rm_file_meta($nsobj,$xxx,$package);
							if(!$upgrade) {
								warnmsg("Unable to change meta-data for $xxx in package $package.");
							}
							if(scalar(@$plist)==1) {
								unlink("$ROOT/$xxx") 
							}
						} else {
							errormsg("Missing meta information for $xxx:\n$err");
						}
					}
				}
			}
			logmsg("  Files removed due to package upgrade/downgrade: $remove_count.") if $remove_count;
		}
		unlink("/var/adm/tp2/ns/$namespace/pkg/$package");
	}

	#################################################################
	# Step 5: Finally renamed the temporary package name to the	#
	#	  correct name.						#
	#################################################################

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed") {
		if(!rename("/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed",
			   "/var/adm/tp2/ns/$namespace/pkg/$package")) {
			return(0,"Unable to rename temporary package configuration file.");
		}
	}
	return(1,undef);
}

#########################################################################
# The function below is similar in concept to pkg_install_ok, but is 	#
# instead called if the package installation FAILS.			#
# Hence it performs the following actions:				#
#									#
# [1]	If the package configuration file "pkg $$$ installed" is 	#
#	found, then the package has been installed (mostly), so 	#
#	jump into the pkg_installed_ok to complete the installation -	#
#	and then exit.							#
# [2]	If the "pkg install $$$ trans" is found, the following actions	#
#	are performed:							#
#	RM entries - specified files are removed.			#
#	RP entries - The exisitng file is renamed to entry.remove.$$	#
#		     and the entry.install.$$ is renamed to entry.	#
#		     entry.remove.$$ is then removed, or added to the	#
#		     "pkg/rm on reboot" file.				#
#	RS entries - Remove file given as 2nd argument and then rename	#
#		     file given as 1st argument to the name of the	#
#		     2nd (used for replaced configuration files).	#
#	RD entries - Sored into longest length first - then each	#
#		     directory is removed.				#
# [3]	Remove the "pkg install $$$ trans" if exists.			#
# [4]	Remove the "pkg install $$$" if exists.				#
# [5]	Remove a corresponding tmpdir/pkginstall.NN directory if 	#
#	found [the directory will contain a !!..config..!! file to	#
#	parse to ensure it is indeed for the package in question].	#
#									#
# The routine uses keyname:value pairs since it requires a number of	#
# parameters:								#
# SPOOLDIR	Directory where package contents are currently spooled	#
# 		to. Including in this top directory will be the		#
#		!!..config..!! config file.				#
# PKGFILE	If the SPOOLDIR is missing then PKGFILE should contain	#
#		the name of the package file being processed.		#
# NAMESPACE	Namespace to install file into. The value of the	#
#		ROOTINSTALLABLE attribute is not checked here, the	#
#		caller is expected to do that.				#
# [OBJ]		Reference to package config obect - optional.		#
# [HOOK]	The name of a routine to call after every successfull	#
#		file install - spaced files installed, space used.	#
# [PREVIEW]	Set to 1 if doing preview install.			#
#									#
# Returns (1,undef) == REMOVED, (2,undef) == COMMITTed, (0,msg) == ERROR#
#########################################################################

sub pkg_installed_failed {
my %args=@_;
my ($spooldir,$namespace,$nsobj,$hook,$package,$ROOT);
my ($ok,$msg,$obj,$err,$pkgfile,$errs,$tmpdir);

	$errs=0;
	$hook=undef;
	if(!exists($args{SPOOLDIR}) && !exists($args{PKGFILE})) {
		return(0,"Missing mandatory arguments.");
	}
	if(!exists($args{NAMESPACE})) {
		return(0,"Missing mandatory arguments.");
	}

	$spooldir=$args{SPOOLDIR} if exists($args{SPOOLDIR});
	$pkgfile=$args{PKGFILE} if exists($args{PKGFILE});
	$namespace=$args{NAMESPACE};
	$hook=$args{HOOK} if exists($args{HOOK});

	if(exists($args{OBJ})) {
		$obj=$args{OBJ};
	} else {
		if(defined($spooldir)) {
			($obj,$err)=pkgfile_config_to_obj("$spooldir/!!..config..!!");
		} else {
			($obj,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$pkgfile");
		}
		return(0,$err) if defined($err);
	}

	($nsobj,$err)=namespace_to_obj($namespace);
	if(!defined($nsobj)) {
		return(undef,$err);
	}
	if(!exists($nsobj->{ROOT})) {
		return(0,"Missing ROOT setting for namespace $namespace.");
	}
	$ROOT=$nsobj->{ROOT};
	$ROOT=~ s/\/+$//; $ROOT="/" if $ROOT eq "";	# remove trailing /'s

	if(! -d $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace does not exist.");
	}
	if(! -w $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace not writable.");
	}

	#################################################################
	# Work out the temporary directory for the namespace.		#
	#################################################################

	if(exists($nsobj->{TMPDIR})) {
			$tmpdir=$nsobj->{TMPDIR};
	}
	if(!defined($tmpdir)) {
		$tmpdir=$ROOT;
	}
	$tmpdir=~ s/\/+$//;	# remove trailing /'s

	$package=$obj->{NAME};
	if(exists($args{PREVIEW}) && $args{PREVIEW}==1) {
		unlink "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$";
		unlink "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans";
		return(1,undef);
	}

	#################################################################
	# Step 1: If the "pkg $$$ installed" exists, then call the	#
	#	  pkg_installed_ok to complete install, not rollback.	#
	#################################################################

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed") {
		($ok,$msg)=pkg_installed_ok(%args);
		if($ok) {
			return(2,undef);
		} else {
			return(0,"Unable to complete rollforward for $package:\n$msg");
		}
	}

	my ($fd,$wfd,$caction,$cfile,@F);
	my $count=0;
	my @dlist=();

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans") {
		if(!open($fd,"/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans")) {
			return(0,"Unable to open transaction recovery file.");
		}
		if(!open($wfd,">>/var/adm/tp2/ns/$namespace/pkg/rm on reboot")) {
			return(0,"Unable to open reboot cleanup transaction file.");
		}
		while(<$fd>) {
			chomp;
			($caction,$cfile)=(/([A-Z]+) (.*)/);
			if($caction eq "RM" && -f "$ROOT/$cfile") {
				if(!unlink "$ROOT/$cfile") {
					print $wfd "$cfile\n";
					$count++;
				}
				($ok,$msg)=remove_namespace_file_meta($namespace,"$ROOT/$cfile",$package);
				if(!$ok) {
					$errs=1;
					errormsg("Error encountered whilst restoring meta information:\n$msg");
				}
			}
			if($caction eq "RD") {
				push @dlist,$cfile;
			}
			if($caction eq "RS") {
				my ($newf,$origf)=split(/\|/,$cfile);
				if(-f "$ROOT/$newf") {
					if(-f "$ROOT/$origf") { unlink "$ROOT/$origf"; }
					if(!rename("$ROOT/$newf","$ROOT/$origf")) {
						return(0,"Unable to rename file $newf back to $origf.");
					}
				}
			}
			if($caction eq "RP") {
				if(-f "$ROOT/$cfile" && -f "$ROOT/$cfile.remove.\$\$") {
					unlink "$ROOT/$cfile.remove.\$\$";
					if(!rename("$ROOT/$cfile","$ROOT/$cfile.remove.\$\$")) {
						return(0,"Unable to rename newly installed file.");
					}
				}
				if(-f "$ROOT/$cfile.install.\$\$") {
					if(!rename("$ROOT/$cfile.install.\$\$","$ROOT/$cfile")) {
						return(0,"Unable to original file prior to install.");
					}
				}
				if(-f "$ROOT/$cfile.remove.\$\$") {
					if(!unlink "$ROOT/$cfile.remove.\$\$") {
						print $wfd "$cfile.remove.\$\$";
					}
				}
				($ok,$msg)=replace_namespace_file_meta($namespace,"$ROOT/$cfile",$package);
				if(!$ok) {
					$errs=1;
					errormsg("Error encountered whilst restoring meta information:\n$msg");
				}
			}
		}

		for (sort {length($b) <=> length($a)} @dlist) {
			if(!rmdir("$ROOT/$_")) {
				print $wfd "$_\n";
			}
		}
		close($fd);
		close($wfd);
	}

	#################################################################
	# Step 3: If the "pkg install $$$ trans" exists, it should be	#
	#	  removed.						#
	#################################################################

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans") {
		if(!unlink("/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$ trans")) {
			return(0,"Unable to remove transaction file.");
		}
	}

	#################################################################
	# Step 4: If the "pkg install $$$" exists, it should be		#
	#	  removed.						#
	#################################################################

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$") {
		if(!unlink("/var/adm/tp2/ns/$namespace/pkg/$package install \$\$\$")) {
			return(0,"Unable to remove temporary package configuration file.");
		}
	}

	#################################################################
	# Step 5: Check the tmpdir directory for pkginstall.NN dirs,	#
	#	  and if found check the !!..config..!! files and if	#
	#	  matching the name of this package, then delete the	#
	#	  directory.						#
	#################################################################

	{
		my ($dfd,@check,$x);
		@check=();
		if(opendir($dfd,$tmpdir)) {
			while($x=readdir($dfd)) {
				next if $x !~ /^pkginstall\.\d+$/;
				next if ! -d "$tmpdir/$x";
				next if ! -f "$tmpdir/$x/!!..config..!!";
				push @check,$x;
			}
			closedir($dfd);

			#################################################
			# Load in each config file and only delete the	#
			# directory if relevant to the package.		#
			#################################################
			my $cdir;
			for $cdir (@check) {
				my ($obj,$err)=pkgfile_config_to_obj("$tmpdir/$cdir/!!..config..!!");
				if(defined($obj) && $obj->{NAME} eq $package) {
					logmsg("Clearing down temp directory '$tmpdir/$cdir' for package '$package'.");
					remove_dir_path("$tmpdir/$cdir");
				}
			}
		}
	}
	
	if($errs) {
		return(0,"Errors found during cleanup.");
	}
	return(1,undef);
}

sub remove_from_spool($) {
my ($namespace)=@_;
my ($nsobj,$err,$tmpdir,$ROOT);

	#################################################################
	# Get the TMPDIR directory for the specified namespace.	If not	#
	# specified will use a temporary directory relative to the	#
	# namespace root.						#
	#################################################################

	($nsobj,$err)=namespace_to_obj($namespace);
	if(!defined($nsobj)) {
		return(undef,$err);
	}

	if(!exists($nsobj->{ROOT})) {
		return(undef,"Missing ROOT setting for namespace $namespace.");
	}
	if(exists($nsobj->{TMPDIR})) {
			$tmpdir=$nsobj->{TMPDIR};
	}
	$ROOT=$nsobj->{ROOT};
	if(!defined($tmpdir)) {
		$tmpdir=$ROOT;
	}
	$tmpdir=~ s/\/+$//;	# remove trailing /'s

	#################################################################
	# Now remove this directory.					#
	#################################################################

	if(-d "$tmpdir/pkginstall.$$") {
		remove_dir_path("$tmpdir/pkginstall.$$");
	}
}

#########################################################################
# The function below is called to execute a specific stript. It expects	#
# the following variables to exist via a hash.				#
#									#
# SPOOLDIR	Directory where package contents are currently spooled	#
# 		to. Including in this top directory will be the		#
#		!!..config..!! config file.				#
# NAMESPACE	Namespace to install file into. The value of the	#
#		ROOTINSTALLABLE attribute is not checked here, the	#
#		caller is expected to do that.				#
# [OBJ]		Reference to package config obect - optional.		#
# SCRIPT	Should be postinstall,preinstall,preremove,postremove	#
#		or checkinstall.					#
#									#
# This routine sets ensures the follownig variables are set when the	#
# script is called:							#
# PKGROOT	The namespace root directory.				#
# PKGSPACE	The namespace name.					#
# PKGNAME	The package name					#
# PKGVERSION	The version of the package				#
# PKGACTION	Will be POSTINSTALL for postinstall etc.		#
# PKGMSGFILE	File to write to to send messages back to this program.	#
# PKGDEPENDS	List of dependencies used for this product. 		#
#									#
# Note:		PKGDEPENDS must be set in the caller, it is not set	#
#		by this routine, but the other values are.		#
#									#
# Return values (status,msg,undef|msg)					#
# Status:	0 - Failure in configuration - see msg.			#
#		1 - Ran script - result code is msg.			#
#		2 - No script is available.				#
#									#
#		msg is the name of the message file if it exists and	#
#		the code has been run and returned 0.			#
#########################################################################

sub does_script_exist {
my %args=@_;
my ($spooldir,$namespace,$nsobj,$hook,$package,$ROOT);
my ($ok,$msg,$text,$fd,$cname,$scr,$rc);

	$hook=undef;
	if(!exists($args{SPOOLDIR}) || !exists($args{NAMESPACE})) {
		return(-1,"Missing mandatory arguments.");
	}
	$scr=uc($args{SCRIPT});
	if(! scalar(grep {/^$scr$/} qw(FIRSTBOOT PREINSTALL POSTINSTALL PREREMOVE POSTREMOVE CHECKINSTALL))) {
		return(-1,"Script type \"".$args{SCRIPT}."\" not known.");
	}

	$spooldir=$args{SPOOLDIR};
	$namespace=$args{NAMESPACE};
	$hook=$args{HOOK} if exists($args{HOOK});

	if(exists($args{OBJ})) {
		$obj=$args{OBJ};
	} else {
		($obj,$err)=pkgfile_config_to_obj("$spooldir/!!..config..!!");
		return(-1,$err) if defined($err);
	}

	($nsobj,$err)=namespace_to_obj($namespace);
	if(!defined($nsobj)) {
		return(-1,$err);
	}
	if(!exists($nsobj->{ROOT})) {
		return(-1,"Missing ROOT setting for namespace $namespace.");
	}
	$ROOT=$nsobj->{ROOT};
	$ROOT=~ s/\/+$//; $ROOT="/" if $ROOT eq "";	# remove trailing /'s

	if(! -d $ROOT) {
		return(-1,"Specified ROOT ($ROOT) for namespace $namespace does not exist.");
	}
	if(! -w $ROOT) {
		return(-1,"Specified ROOT ($ROOT) for namespace $namespace not writable.");
	}

	#################################################################
	# Get the contents of the script.
	#################################################################

	if(!exists($obj->{$scr})) {
		return(0,undef);
	}
	return(1,undef);
}

#################################################################################
# args are SCRIPT, NAMESPACE and SPOOLDIR. We create a ".first boot for pkgname"#
# file - this does not contain the code just the fact that it needs to be run.	#
#################################################################################

sub install_firstboot {
my %args=@_;
my ($err,$obj,$fd);

	if(!exists($args{SPOOLDIR}) || !exists($args{NAMESPACE})) {
		return(0,"Missing mandatory arguments.");
	}
	my $namespace=$args{NAMESPACE};
	my $spooldir=$args{SPOOLDIR};
	if(exists($args{OBJ})) {
		$obj=$args{OBJ};
	} else {
		($obj,$err)=pkgfile_config_to_obj("$spooldir/!!..config..!!");
		return(0,$err) if defined($err);
	}
	my $package=$obj->{NAME};
	if(-f "/var/adm/tp2/ns/$namespace/pkg/.first boot for $package") {
		return(1,undef);
	}
	open($fd,">/var/adm/tp2/ns/$namespace/pkg/.first boot for $package") or
		return(0,"Unable to write to file '/var/adm/tp2/ns/$namespace/pkg/.first boot for $package'.");
	close($fd);
	return(1,undef);
}

sub execute_script {
my %args=@_;
my ($spooldir,$namespace,$nsobj,$hook,$package,$ROOT);
my ($ok,$msg,$text,$fd,$cname,$scr,$rc);

	$hook=undef;
	if(!exists($args{SPOOLDIR}) || !exists($args{NAMESPACE})) {
		return(0,"Missing mandatory arguments.");
	}
	$scr=uc($args{SCRIPT});
	if(! scalar(grep {/^$scr$/} qw(FIRSTBOOT PREINSTALL POSTINSTALL PREREMOVE POSTREMOVE CHECKINSTALL))) {
		return(0,"Script type \"".$args{SCRIPT}."\" not known.");
	}

	$spooldir=$args{SPOOLDIR};
	$namespace=$args{NAMESPACE};
	$hook=$args{HOOK} if exists($args{HOOK});

	if(exists($args{OBJ})) {
		$obj=$args{OBJ};
	} else {
		($obj,$err)=pkgfile_config_to_obj("$spooldir/!!..config..!!");
		return(0,$err) if defined($err);
	}

	($nsobj,$err)=namespace_to_obj($namespace);
	if(!defined($nsobj)) {
		return(undef,$err);
	}
	if(!exists($nsobj->{ROOT})) {
		return(0,"Missing ROOT setting for namespace $namespace.");
	}
	$ROOT=$nsobj->{ROOT};
	$ROOT=~ s/\/+$//; $ROOT="/" if $ROOT eq "";	# remove trailing /'s

	if(! -d $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace does not exist.");
	}
	if(! -w $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace not writable.");
	}

	#################################################################
	# Get the contents of the script.
	#################################################################

	if(!exists($obj->{$scr})) {
		return(2,undef);
	}
	
	$package=$obj->{NAME};
	$cname="$spooldir/$package-$scr";
	$scr=$obj->{$scr};
		
	open($fd,">$cname") or return(0,"Unable to open temporay file in spooldir for writing.");
	if(! print $fd "$scr\n") {
		unlink($cname);
		return(0,"Error writing to temporary file in spool directory.");
	}
	if(! close($fd)) {
		unlink($cname);
		return(0,"Error writing to temporary file in spool directory.");
	}

	#################################################################
	# Make the code executable and run it ... allowing any output	#
	# to go the screen for present.					#
	#################################################################

	chmod 0755,$cname;
	$ENV{PKGROOT}=$ROOT;
	$ENV{PKGSPACE}=$namespace;
	$ENV{PKGNAME}=$package;
	$ENV{PKGVERSION}=$obj->{PKGVER};
	$ENV{PKGACTION}=uc($args{SCRIPT});
	$ENV{PKGMSGFILE}="$cname.msg";
	system($cname);
	$rc=$?>>8;
	unlink($cname);
	if(!$rc) {
		return(1,$rc,undef);
	} else {
		return(1,$rc,"$cname.msg");
	}
}

#########################################################################
# file_clash is a routine that takes the following arguments:		#
# package configuration object.						#
# namespace configuration object.					#
# strictness.								#
# checks to see if any of the files referenced are currently part	#
# of other packages in this namespace. It will return a reference	#
# to the list of clashes. Each reference element will be the namespace-	#
# relative filename and a comma-separated list of packages that have	#
# that file installed.							#
#									#
# The third argument is 1 for "strict" and 0 for "loose".		#
#									#
# A strict clash is simply a clash of names, no matter what the 	#
# content. A loose context only produces a clash if the file contents	#
# differ.								#
#########################################################################

sub file_clash($$$) {
my ($obj,$nsobj,$strict)=@_;
my ($cfile,$moddedname,$sha1obj,$csha1,$ROOT,$package,$cdir);
my ($namespace,@F,@L,$fd,@C);

	$namespace=$nsobj->{NAME};
	$ROOT=$nsobj->{ROOT};
	$package=$obj->{NAME};
	$clist=[];
	for $cfile (@{$obj->{FILES}}) {
		my $sha1_for_target;

		@F=split(/\|/,$cfile);
		$sha1_for_target=$F[4];
		$moddedname="$ROOT/$F[6]";
		$moddedname =~ s/\/{2,}/\//g;

		$sha1obj=Digest::SHA1->new;
		$sha1obj->add($moddedname);
		$csha1=$sha1obj->hexdigest;
		$cdir="/var/adm/tp2/ns/$namespace/meta/".substr($csha1,0,2);
		next if ! -f "$cdir/$csha1";

		#########################################################
		# If there is a clash, we need to see if the entry	#
		# refers to this or other packages.			#
		# In fact only check the last entry since that will	#
		# have the current SHA1 of the installed file.		#
		#########################################################

		if(!open($fd,"$cdir/$csha1")) {
			warnmsg("Unable to open metafile to check $cfile!");
			next;
		}
		@L=<$fd>; close($fd);
		chomp foreach (@L);
		@C=();
		if($strict) {
			foreach (@L) {
				@F=split(/\|/);
				next if $F[0] ne $moddedname;
				next if $F[5] eq $package;
				push @C,$F[5];
			}
		} else {
			my ($ll,@L2);
			foreach (@L) {
				@F=split(/\|/);
				next if $F[0] ne $moddedname;
				next if $F[5] eq $package;
				$ll=$F_;
				@L2=@F;

			}
			if(defined($ll) && $ll ne $sha1_for_target) {
				push @C,$L2[5];
			}
		}
		if(@C) {
			push @$clist,"$moddedname|".join(",",@C);
		}
	}
	return $clist;
}

#########################################################################
# Takes a Repository path in any of the following formats:		#
# DIR:/directory			- local directory		#
# FTP:host/directory			- anonymous FTP			#
# FTP:host/directory[user,pass]		- FTP via user,pass		#
# WWW:host/directory			- access via HTTP		#
#									#
# This will return:							#
# (1,repos_ref) 	OK - contents of repos file in reference.	#
# (0,msg) 		ERROR - see msg					#
#########################################################################

sub get_repos_file($) {
my $P=$_[0];
my ($fd,$cdata,@DATA,@F);
my ($proto,$uri,$d);

	($proto,$uri)=($P =~ /^([A-Z]+):(.*)/);
	if(!defined($proto) || uc($proto) eq "DIR") {
		if(!defined($proto)) {
			$proto="DIR";
			$uri=$P;
		}
		if(! -d "$uri") {
			return(0,"No such directory, or directory not accessible.");
		}
		if(! -r "$uri") {
			return(0,"Directory is not readable.");
		}
		if(! -f "$uri/.repos") {
			return(0,"Directory missing repository index.");
		}
		if(! -r "$uri/.repos") {
			return(0,"Repository index is not readable.");
		}

		open($fd,"$uri/.repos") or
			return(0,"Unable to open repository index for reading.");
		
		@DATA=<$fd>; close($fd);
		$d=join("",@DATA);
		return(1,\$d);
	}
	if(uc($proto) eq "HTTP" || uc($proto) eq "WWW") {
		return(0,"Protocol ".uc($proto)." is not available.") if ! $WWW_avail;
		my ($host,$dir,$d);
		($host,$dir)=($uri =~ /^([^\/]+)(\/.*)/);
		$dir="" if !defined($dir);
		$d=get("http://$host/$dir/.repos");
		if(defined($d)) {
			return(1,\$d);
		} else {
			return(0,"Unable to retrieve file via http://$host/$dir/.repos");
		}
	}

	if(uc($proto) eq "FTP") {
		return(0,"FTP protocol functionality not available.") if ! $FTP_avail;
		my ($host,$user,$pass,$dir,$x,$ftp);
		if(index($uri,"[")>-1) {
			($user,$pass)=($uri =~ /\[(.*),(.*)\]/);
		}
		if(!defined($user)) {
			$user="anonymous";
		}
		if(!defined($pass)) {
			$pass="-anonymous@";
		}
		if(index($uri,"[")>-1) {
			($host,$dir)=($uri =~ /^([^\/]+)(\/.*)\[/);
		} else {
			($host,$dir)=($uri =~ /^([^\/]+)(\/.*)/);
		}
		if(!defined($host) || !defined($dir)) {
			return(0,"Malformed FTP repository path.");
		}

		eval {$ftp=Net::FTP->new($host);};
		if(!defined($ftp)) {
			return(0,"Unable to gain FTP connection to $host :\n$@");
		}
		$x=$ftp->login($user,$pass);
		if(!defined($x)) {
			return(0,"Unable to access FTP site using login credentials.");
		}
		open($fd,">/tmp/.repos.$$") or
			return("Unable to write to temporary file.");
		close($fd);

		$ftp->binary;
		$x=$ftp->get(".repos","/tmp/.repos.$$");
		if(!defined($x)) {
			return(0,"Repository index file not found, or cannot be retrieved.");
		}
		$ftp->quit;

		open($fd,"/tmp/.repos.$$") or
			return(0,"Unable to open repository index for reading.");
		
		@DATA=<$fd>; close($fd);
		$d=join("",@DATA);
		return(1,\$d);
	}
	errormsg("Protocol ".uc($proto)." is not yet supported.");
}

#########################################################################
# The checkinstall_code_from_repos_file takes the full path of the file	#
# containing the .repos contents, whilst the 2nd arg is the name of the	#
# packege file in the repository. Alternatively third data can be the	#
# repository text itself if available, undef otherwise. 		#
# This will return:							#
# (checkcode,under) or (undef,errmsg)					#
#########################################################################

sub checkinstall_code_from_repos_file($$$) {
my ($reposfile,$pkgfile,$reposdata)=@_;
my ($fd,$str,@F,@F2);

	if(!defined($reposdata)) {
		open($fd,$reposfile) or
			return(undef,"Unable to open $reposfile: $!");
		while(<$fd>) {
			chomp;
			@F=split(/\|/,$_);
			if($F[0] eq $pkgfile) {
				close($fd);
				if(!defined($F[8])) {
					return("",undef);
				}
				return (pack("H*",$F[8]),undef);
			}
		}
		close($fd);
		return(undef,"Unable to find package file \"$pkgfile\" in repository.");
	}
	@F2=split(/\n/,$reposdata);
	for (@F2) {
		@F=split(/\|/,$_);
		if($F[0] eq $pkgfile) {
			if(!defined($F[8])) {
				return("",undef);
			}
			return (pack("H*",$F[8]),undef);
		}
	}
	return(undef,"Unable to find package file \"$pkgfile\" in repository.");
}

#########################################################################
# This function takes a repository and retrieves the named file.	#
# It makes use of the following arguments:				#
# reposity - any of the forms mentioned above.				#
# file     - file to retrieve						#
# destfile - file to write to						#
#									#
# Will return:								#
#	(0,msg)		- error occured, see msg			#
#	(1,undef)	- file retrieved as required.			#
#########################################################################

sub get_package_from_path($$$) {
my ($P,$rfile,$lfile)=@_;
my ($fd,$cdata,@F);
my ($proto,$uri,$msg,$ok);

	#################################################################
	# Grab the protocol, and handle each separately.		#
	#################################################################

	($proto,$uri)=($P =~ /^([A-Z]+):(.*)/);
	if(!defined($proto) || uc($proto) eq "DIR") {
		if(!defined($proto)) {
			$proto="DIR";
			$uri=$P;
		}
		if(! -d "$uri") {
			return(0,"No such directory, or directory not accessible.");
		}
		if(! -r "$uri") {
			return(0,"Directory is not readable.");
		}
		if(! -f "$uri/$rfile") {
			return(0,"Directory missing repository index.");
		}
		if(! -r "$uri/$rfile") {
			return(0,"Repository index is not readable.");
		}

		($ok,$msg)=copyfile("$uri/$rfile","$lfile");
		if(!defined($ok)) {
			return(0,"Unable to copy file:\n$msg");
		}
		return(1,undef);
	}
	if(uc($proto) eq "HTTP" || uc($proto) eq "WWW") {
		my ($r);
		return(0,"Protocol ".uc($proto)." is not available.") if ! $WWW_avail;
		$r=getstore("http://$uri/$rfile","$lfile");
		if(! -f "$lfile") {
			return(0,"Unable to copy file.");
		}
		return(1,undef);
	}
	if(uc($proto) eq "FTP") {
		my ($host,$user,$pass,$dir,$x,$ftp);
		return(0,"FTP protocol not available.") if ! $FTP_avail;

		if(index($uri,"[")>-1) {
			($user,$pass)=($uri =~ /\[(.*),(.*)\]/);
		}
		if(!defined($user)) {
			$user="anonymous";
		}
		if(!defined($pass)) {
			$pass="-anonymous@";
		}
		if(index($uri,"[")>-1) {
			($host,$dir)=($uri =~ /^([^\/]+)(\/.*)\[/);
		} else {
			($host,$dir)=($uri =~ /^([^\/]+)(\/.*)/);
		}
		if(!defined($host) || !defined($dir)) {
			return(0,"Malformed FTP repository path.");
		}

		eval {$ftp=Net::FTP->new($host);};
		if(!defined($ftp)) {
			return(0,"Unable to gain FTP connection to $host.");
		}
		$x=$ftp->login($user,$pass);
		if(!defined($x)) {
			return(0,"Unable to access FTP site using login credentials.");
		}
		open($fd,">/tmp/.repos.$$") or
			return("Unable to write to temporary file.");
		close($fd);

		$ftp->binary;
		$x=$ftp->cwd($dir);
		if(!defined($x)) {
			return(0,"Repository directory [$dir] not found.");
		}
		$x=$ftp->get($rfile,$lfile);
		if(!defined($x)) {
			return(0,"Repository file not found, or cannot be retrieved.");
		}
		$ftp->quit;
		return(1,undef);
	}
	errormsg("Protocol ".uc($proto)." is not yet supported.");
}

#########################################################################
# The following routine will return the URI of a depot that includes	#
# a package given name,minver,maxver.					#
# Arguments:								#
# 1. repository list  (space separated)					#
# 2. package name							#
# 3. min version							#
# 4. max version							#
# 5. requested version (undef gets latest,-1 gets very latest)		# 
#									#
# Difference between latest and "very latest"				#
# ===========================================				#
# The 'latest' package will return the latest version of the package	#
# from the first depot where the package exists, which may not		#
# actually be newer than that currently installed. Very latest scans	#
# _all_ depots and returns the latest version from any of the depots.	#
#									#
# The operating system type is gotton from Config{osname}, whilst the	#
# architecture is retrieved from `uname -m`.				#
#									#
# Returns:	(0,msg)		  Error attempting to find package	#
#		(1,repos,pkgfile) Found it in specified repository.	#
#		(2,msg)		  Unable to find package		#
#########################################################################

sub vercmp($$) {
my ($e,$f)=@_;
my (@EE,@FF,$x);

	$x=0;
	@EE=split(/\./,$e);
	@FF=split(/\./,$f);
	while(exists($EE[$x])) {
		if(!exists($FF[$x])) {
			return 1;
		}
		if($EE[$x] !~ /^\d+$/ || $FF[$x] !~ /^\d+$/) {
			if($EE[$x] eq $FF[$x]) {
				$x++; next;
			}
			# If just one is alphabetic make that older.
			if($EE[$x] =~ /^\d+$/) {
				return 1;
			}
			if($FF[$x] =~ /^\d+$/) {
				return -1;
			}
			return $EE[$x] cmp $FF[$x];
		} else {
			if($EE[$x]==$FF[$x]) {
				$x++; next;
			}
			return $EE[$x] <=> $FF[$x];
		}
	}
	if(!exists($FF[$x])) {
		return 0;
	}
	return -1;
}

sub do_ver_cmp($$) {
	return vercmp($_[0],$_[1]);
}

sub find_package($$$$$) {
my ($repos,$pkg,$minver,$maxver,$reqver)=@_;
my $osname=$Config{osname};
my $arch=`uname -m`; chomp $arch;
my ($ok,$d,$crepos,@R,%lhash,%lhash2);

	$arch =~ s/[\-\/]{1}/_/g;
	dprint "find_package: repos =$repos\n";
	dprint "find_package: pkg   =$pkg\n";
	dprint "find_package: minver=",defined($minver) ? $minver : "undefined","\n";
	dprint "find_package: maxver=",defined($maxver) ? $maxver : "undefined","\n";
	dprint "find_package: reqver=",defined($reqver) ? $reqver : "undefined","\n";

	%lhash=();
	($minver,$maxver)=($reqver,$reqver) if defined($reqver) && $reqver ne "-1";
	$minver="0.0.1" if !defined($minver);
	$maxver="99999999.999999.999999" if !defined($maxver);
	if(substr($minver,0,1) eq ">") {
		$minver=substr($minver,1);
		@R=split(/\./,$minver);
		$R[$#R]++;
		$minver=join(".",@R);
		dprint "find_package: minver=$minver\n";
	}

	@R=split(/\s+/,$repos);
	my $highest_in_repos=undef;
	my $highest_repos_is=undef;
	for $crepos (@R) {
		#########################################################
		# Check out hash, and if we already have the repos	#
		# contents and it is under 2 mins old, do not get a 	#
		# newer one.						#
		#########################################################
		if(!exists($repos_contents_hash{$crepos}) ||
			$repos_contents_hash{$crepos}->[0]+120<time) {
			($ok,$d)=get_repos_file($crepos);
			if(!$ok) {
				warnmsg("Error retrieving from repository $crepos:");
				warnmsg("$d");
				next;
			}
			$repos_contents_hash{$crepos}=[time,$d];
		}
		my (@pkglist,@pkglist2,$cpkg,@F,@F2,@P);
		@P=split(/\n/m,${$repos_contents_hash{$crepos}->[1]});
		@pkglist=();
		for (@P) {
			$cpkg=$_;
			@F=split(/\|/,$cpkg);
			push @pkglist,$cpkg if $F[1] eq "$pkg";
		}
		dprint "pkglist after initial selection:",join(",",@pkglist),"\n";
				
		#########################################################
		# Filter out packages not relevant to this operating	#
		# system.						#
		#########################################################

		@pkglist2=@pkglist;
		@pkglist=();
		for (@pkglist2) {
			$cpkg=$_;
			@F=split(/\|/,$cpkg);
			if($F[3] eq "") {
				push @pkglist,$cpkg;
				next;
			}
			@F2=split(/,/,$F[3]);
			if(scalar(grep {/^$osname$/} @F2)) {
				push @pkglist,$cpkg;
			} else {
				dprint "Ejecting package '$cpkg' - OSs does not contain '$osname'.\n";
			}
		}
		next if ! scalar(@pkglist);

		#########################################################
		# Finally filter out packages not relevant to this	#
		# architecture.						#
		#########################################################

		@pkglist2=();
		for (@pkglist) {
			$cpkg=$_;
			@F=split(/\|/,$cpkg);
			if($F[4] eq "") {
				push @pkglist2,$cpkg;
				next;
			}
			@F2=split(/,/,$F[4]);
			if(scalar(grep {/^$arch$/} @F2)) {
				push @pkglist2,$cpkg;
			} else {
				dprint "Ejecting package '$cpkg' - architectures does not contain '$arch'.\n";
			}
		}
		next if ! scalar(@pkglist2);

		dprint "package list after architecture/os filtering: ",join(",",@pkglist2),"\n";

		#########################################################
		# Filter out packages that are not of the correct	#
		# versions...						#
		#########################################################

		@pkglist2=();
		for (@pkglist) {
			$cpkg=$_;
			@F=split(/\|/,$cpkg);
			if(defined($reqver) && $reqver ne "-1") {
				if($F[2] eq $reqver) {
					push @pkglist2,$cpkg;
				}
				next;
			}
			if(defined($reqver) && $reqver eq "-1") {
				if(do_ver_cmp($F[2],$minver)>=0 && do_ver_cmp($F[2],$maxver)<=0) {
					push @pkglist2,$cpkg;
				}
				next;
			}
			if(do_ver_cmp($F[2],$minver)>=0 && do_ver_cmp($F[2],$maxver)<=0) {
				push @pkglist2,$cpkg;
			} else {
				if(!defined($highest_in_repos)) {
					$highest_in_repos=$F[2];
					$highest_repos_is=$crepos;
					next;
				}
				if(do_ver_cmp($F[2],$highest_in_repos)>0) {
					$highest_in_repos=$F[2];
					$highest_repos_is=$crepos;
				}
			}
		}
		next if ! scalar(@pkglist2);


		#########################################################
		# At this point pkglist2 contains only packages of the	#
		# correct versions, and also match the architecture	#
		# and or operating system. Out of all thses (if any)	#
		# chose the one with the highest version by default.	#
		#							#
		# If very latest is required just store the settings	#
		# for comparision after all repositories.		#
		#########################################################

		@pkglist=sort {my (@C,@D); @C=split(/\|/,$a); @D=split(/\|/,$b); vercmp($D[2],$C[2]);} @pkglist2;
		if(defined($reqver) && $reqver eq "-1") {
			if(scalar(@pkglist)) {
				my @FF=split(/\|/,$pkglist[0]);
				if(!exists($lhash{$FF[2]})) {
					$lhash{$FF[2]}=$pkglist[0];
					$lhash2{$FF[2]}=$crepos;
				}
			}
		} else {
			if(scalar(@pkglist)) {
				return(1,$crepos,$pkglist[0]);
			}
		}
	}

	#################################################################
	# Get here if no packages are matching or very latest package	#
	# has been requested.						#
	#################################################################

	if(defined($reqver) && $reqver eq "-1" && scalar(keys(%lhash))) {
		@pkglist=sort {vercmp($b,$a);} (keys(%lhash));
		return(1,$lhash2{$pkglist[0]},$lhash{$pkglist[0]});
	}

	#################################################################
	# Nothing available to install.					#
	#################################################################

	if(!defined($highest_repos_is)) {
		return(2,"Unable to find package $pkg in repositories.",undef);
	} else {
		return(2,$highest_in_repos,$highest_repos_is);
	}
}

#########################################################################
# The following routine will be called to ascertain whether the		#
# specified package is installed in the specified namespace. The args	#
# passed include the following:						#
# nsobj   - Namespace object to check.					#
# pkgname - The name of the package to check whether installed.		#
# minver  - Minimum level to check for (0.0.1 if undefined)		#
# maxver  - Maximum level to check for (99999999.999999.999999 		#
#	    if undefined)						#
# reinst  - if 1 then reinstall saem version; set to 0 if undefined.	#
# downgrade - set to 1 to reinstall older version, 0 if undefined.	#
#									#
# Will return the following:						#
# 0,undef - Package is not currently installed [or installed and 	#
# 	    downgrade/re-install is ok].				#
# 1,ver   - Package is installed.					#
# 2,undef - Package is partially installed.				#
#########################################################################

sub is_installed($$$$$$) {
my ($nsobj,$pkgname,$minver,$maxver,$reinstall,$downgrade)=@_;
my ($ROOT,$NAME);

	$ROOT=$nsobj->{ROOT};
	$NAME=$nsobj->{NAME};

	$minver="0.0.1" if !defined($minver);
	$maxver="99999999.999999.999999" if !defined($maxver);
	$reinstall=0 if ! defined($reinstall);
	$downgrade=0 if ! defined($downgrade);

	if(-f "/var/adm/tp2/ns/$NAME/pkg/$pkgname") {

		#########################################################
		# Get the object and check versions...			#
		#########################################################

		($obj,$msg)=pkgfile_config_to_obj("/var/adm/tp2/ns/$NAME/pkg/$pkgname");
		if(!defined($obj)) {
			return (2,undef);
		}
		my $vv;
		if(exists($obj->{PKGVER})) {
			$vv=do_ver_cmp($minver,$obj->{PKGVER});
			if($vv <= 0) {
				return (0,undef) if $downgrade ;
				return (0,undef) if $vv==0 && $reinstall;
			}
			if(do_ver_cmp($minver,$obj->{PKGVER}) >0 &&
				do_ver_cmp($obj->{PKGVER},$maxver) <=0) {
				return (0,undef);
			}
			return (1,$obj->{PKGVER});
		}
		#########################################################
		# If no version info, also assume installed.		#
		#########################################################
		return (0,undef);
	}

	#################################################################
	# Check for partially installed files...			#
	#################################################################

	if(-f "/var/adm/tp2/ns/$NAME/pkg/$pkgname install \$\$\$" ||
		-f "/var/adm/tp2/ns/$NAME/pkg/$pkgname \$\$\$ installed" ||
		-f "/var/adm/tp2/ns/$NAME/pkg/$pkgname \$\$\$ removed" || 
		-f "/var/adm/tp2/ns/$NAME/pkg/$pkgname removing \$\$\$") {
		return (2,undef);
	}
	#################################################################
	# Package is not installed.					#
	#################################################################
	return (0,undef);
}

#########################################################################
# The routine below is called to generate a list of ALL files that	#
# meet the dependencies of the initial package. This list is just	#
# a hash reference, where each key is the name of a package.		#
# Each element value is: [notinstallcount, [list requirements], repos,	#
#	reposline]							#
# Argument are:								#
# cpkgname   - details of the package dependencies trying to resolve.	#
# hashedinfo - currently kept information.				#
# nsobj      - current namespace object.				#
# deptree    - current dependency list (for error output only)		#
# [minver]   - minimum version						#
# [maxver]   - maximum version						#
# [reqver]   - requested version					#
# [checkinstalls] - whether to run checkinstall scripts or not.		# 
# [reinstall]     - If present and set will allow the same level of	#
#                   software to be reinstalled.				#
# [downgrade]     - Set to 1 to back date software versions.		#
#									#
# [altreposlist]  - white space separated list of repositories to	#
#		    consider if package not find via TP2_PATH.		#
# [repos]	  - override the settings of TP2_PATH and scan these	#
#		    repositories instead.				#
#									#
# Handling 'altrepos' attributes.					#
# 									#
# Also now handle incompatibles if present in the repository index.	#
#									#
#########################################################################

sub follow_dependencies {
my %args=@_;
my ($cpkgname,$hashedinfo,$nsobj,$minver,$maxver,$reqver,$deptree,$checkinstalls,$reinstall);
my ($downgrade,$traversed);
my ($ok,$msg,$x,$data,$pkg,$pkginfo);
my ($altrepos,$altreposlist,$iversion);
my $userepos=$ENV{TP2PATH};

	$cpkgname=$args{cpkgname};
	$userepos=$args{repos} if exists($args{repos});
	my $toppkg=0;
	if(!exists($args{deptree}) || $args{deptree} eq $cpkgname) {
		$toppkg=1;
	}
	if(!exists($args{deptree})) {
		$deptree="$cpkgname";
	} else {
		$deptree=$args{deptree};
	}
	$traversed={};
	$traversed=$args{traversed} if exists($args{traversed});
	$checkinstalls=$reinstall=$downgrade=0;
	$checkinstalls=$args{checkinstalls} if exists($args{checkinstalls});
	$reinstall=$args{reinstall} if exists($args{reinstall});
	$downgrade=$args{downgrade} if exists($args{downgrade});
	$altrepos="";
	$altrepos=$args{altreposlist} if exists($args{altreposlist});
	$hashedinfo=$args{hashedinfo};
	$nsobj=$args{nsobj};
	
	if(exists($traversed->{$cpkgname})) {
		return;
	}
	$traversed->{$cpkgname}=1;
	logmsg("Getting dependencies for $cpkgname.");
	$minver=$maxvar=undef;
	if(exists($args{minver})) {
		$minver=$args{minver};
	}
	if(exists($args{maxver})) {
		$maxver=$args{maxver};
	}
	if(exists($args{reqver})) {
		$maxver=$args{reqver};
		$minver=$args{reqver};
		$reqver=$args{reqver};
	}

	#################################################################
	# Exit immediately if a nested call has recorded an error.	#
	#################################################################
	if(exists($hashedinfo->{'%ERROR%'})) {
		return;
	}

	#################################################################
	# If cpkgname already exists in the hash then it has been, or	#
	# is being processed, so do nothing.				#
	#################################################################
	if(exists($hashedinfo{$cpkgname})) {
		return;
	}

	#################################################################
	# If the package in question is already installed, then do	#
	# nothing and return.						#
	#################################################################

	($x,$iversion)=is_installed($nsobj,$cpkgname,$minver,$maxver,$reinstall,$downgrade);
	if($x==1) {
		return;
	}

	#################################################################
	# Abort if a partially installed dependency is found.		#
	#################################################################
	if($x==2) {
		$hashedinfo->{'%ERROR%'}="Dependency \"$cpkgname\" is only partially installed.";
		return;
	}

	#################################################################
	# Ok, attempt to get the package details via the userepos 	#
	# settings.							#
	#################################################################

	# dprint "minver=$minver,maxver=$maxver,reqver=$reqver\n";
	($ok,$msg,$pkginfo)=find_package($userepos,$cpkgname,$minver,$maxver,$reqver);
	if($ok==0) {
		$hashedinfo->{'%ERROR%'}="Error searching for package \"$cpkgname\":\n$msg";
		return;
	}

	#################################################################
	# If not found, then if altrepos contains anything, then scan	#
	# those...							#
	#################################################################

	if($ok==2) {
		if($altrepos ne "") {
			($ok,$msg,$pkginfo)=find_package($altrepos,$cpkgname,$minver,$maxver,$reqver);
			if($ok==0) {
				$hashedinfo->{'%ERROR%'}="Error searching for package \"$cpkgname\":\n$msg";
				return;
			}
		}
	}
	if($ok==2) {
		my $t="";
		my @TT=();
		push @TT,"Minimum version $minver" if defined($minver);
		push @TT,"Requried version $reqver" if defined($reqver);
		push @TT,"Maximum version $maxver" if (defined($maxver) && $maxver ne "99999999.999999.999999");
		if(@TT) {
			$t=" [" . join(",",@TT) . "]";
		}
		my (@m1);
		@m1=split(/,/,$deptree);
		$hashedinfo->{'%ERROR%'}="Unable to find package \"$cpkgname$t\". Dependency trail:\n".join(" -> ",@m1);
		return;
	}

	#################################################################
	# If get this far then pkginfo contains an actual package 	#
	# details whilst msg contains the repository name.		#
	#################################################################

	@m2=split(/\|/,$pkginfo);
	$deps=$m2[6]; $deps =~ s/\s+/ /g;
	$altreposlist=$m2[10];
	my $incompatibles=$m2[11];
	$incompatibles="" if !defined($incompatibles);
	$hashedinfo->{$cpkgname}=[0,[],$msg,$pkginfo];

	#################################################################
	# If incompatibles is set then any CURRENTLY installed package	#
	# is checked and if meet specifications then the installation	#
	# will be abandoned.						#
	#################################################################

	if($incompatibles ne "") {
		my ($in_pkg,$in_minver,$in_maxver);
		my (@in_F,$in_x,$ver);

		for $in_pkg (split(/\s+/,$incompatibles)) {
			next if $in_pkg eq "";
			@in_F=split(/,/,$in_pkg);
			($in_x,$ver)=is_installed($nsobj,$in_F[0],$in_F[1],$in_F[2],0,0);
			if($in_x>0) {
				$hashedinfo->{'%ERROR%'}="Package \"$cpkgname\" is incompatible with installed package \"$in_F[0]\".";
				return;
			}
		}
	}

	#################################################################
	# If checkinstalls is set then need to get/call check_install	#
	# script via the reposfile/pkgage...				#
	#################################################################

	if($checkinstalls) {
		my ($check_code,$err)=checkinstall_code_from_repos_file(undef,$m2[0],${$repos_contents_hash{$msg}->[1]});
		if(defined($err)) {
			warnmsg("Unable to get check installation script for $m2[0] from repository $msg:\n$err");
		} elsif(defined($check_code) && length($check_code)) {
			my ($rc,$t,$fd2)=(undef,"/tmp/.check_code.$$",undef);

			if(!open($fd2,">$t")) {
				$hashedinfo->{'%ERROR%'}="Unable to write to file \"$t\": $!";
				return;
			}
			if(! print $fd2 $check_code) {
				close($fd2); unlink $fd2;
				$hashedinfo->{'%ERROR%'}="Error writing to file \"$t\": $!";
				return;
			}
			close($fd2);
			if(!chmod(0555,$t)) {
				$hashedinfo->{'%ERROR%'}="Unable to make file \"$t\" executable: $!";
				return;
			}

			#################################################
			# Before running command dump out environment	#
			# settings for the script.			#
			#################################################

			$ENV{PKGROOT}=$nsobj->{ROOT};
			$ENV{PKGSPACE}=$nsobj->{NAME};
			$ENV{PKGNAME}=$m2[1];
			$ENV{PKGVERSION}=$m2[2];
			$ENV{PKGACTION}="CHECKINSTALL";
			$ENV{PKGDEPENDS}=$deps;
			$ENV{PKGMSGFILE}="/var/adm/tp2/ns/".$nsobj->{NAME}."/spool/checkinstall-$$-msg";
			unlink($ENV{PKGMSGFILE}) if -f $ENV{PKGMSGFILE};
			system($t);
			unlink $t;
			$rc=$?>>8;
			if($rc==3) {
				warnmsg("Checkinstall for $m2[1] has removed itself from install list.");
				delete $hashedinfo->{$cpkgname};
				my ($ckey,@nlist,$rpkg);
				$rpkg=$m2[1];
				foreach $ckey (keys %$hashedinfo) {
					@nlist=grep { ! /^$rpkg,/ } @{$hashedinfo->{$ckey}->[1]};
					$hashedinfo->{$ckey}->[0]=scalar(@nlist);
					$hashedinfo->{$ckey}->[1]=\@nlist;
				}
				return;
			}
			if($rc==1) {
				warnmsg("Return code 1 from check install script for $m2[0].");
			}
			if($rc==2) {
				$hashedinfo->{'%ERROR%'}="Fatal error returned from check install script for $m2[0].";
				unlink($ENV{PKGMSGFILE}) if -f $ENV{PKGMSGFILE};
				return;
			}
			if(-f $ENV{PKGMSGFILE}) {
				$deps=update_deps_from_file($deps,$ENV{PKGMSGFILE});
			}
		}
	}

	@depl=split(/\s+/,$deps);
	for $cpkg (@depl) {
		my ($cn,$min,$max,$ver)=split(/,/,$cpkg);
		($x,$ver)=is_installed($nsobj,$cn,$min,$max,0,0);
		if($x==2) {
			$hashedinfo->{'%ERROR%'}="Dependency \"$cn\" is only partially installed.";
			return;
		}
		if($x==0) {
			$hashedinfo->{$cpkgname}->[0]++;
			push @{$hashedinfo->{$cpkgname}->[1]},"$cn,$min,$max";
			follow_dependencies(
				cpkgname	=> $cn,
				hashedinfo	=> $hashedinfo,
				nsobj		=> $nsobj,
				minver		=> $min,
				maxver		=> $max,
				deptree		=> "$deptree,$cn",
				checkinstalls	=> $checkinstalls,
				altreposlist	=> $altreposlist,
				repos		=> $userepos,
				traversed	=> $traversed,
				);
		}
	}
	return;
}

#########################################################################
# Works out whether any packages chosen for install are incompatible	#
# with those packages already installed.				#
# Arguments:								#
# 0 - reference to hash of packages to isntall				#
# 1 - reference to list of packages currently installed			#
#									#
# Returns:								#
# 0 - reference to a list of errors.					#
#########################################################################

sub check_incompatibles($$) {
my ($hashedinfo,$pkg_list)=@_;
my ($cpkg,$cinc,@F,$cn,$cmin,$cmax);
my ($creposl,@S,$cver,@errors);

	@errors=();
	foreach $cpkg (sort {$a->{NAME} cmp $b->{NAME}} @$pkg_list) {
		next if !exists($cpkg->{INCOMPATIBLES});
		dprint "Checking package \"".$cpkg->{NAME}."\"\n";
		dprint "INCOMPATS: ",$cpkg->{INCOMPATIBLES},"\n";
		for $cinc (split(/\s+/,$cpkg->{INCOMPATIBLES})) {
			next if $cinc eq "";
			@F=split(/\|/,$cinc); $cn=$F[0];
			if(!exists($hashedinfo->{$cn})) {
				next;
			}
			$cmin=defined($F[1]) ? $F[1] : "0.0.1";
			$cmax=defined($F[2]) ? $F[2] : "99999999.999999.999999";
			dprint "Is package $cn\[$cmin,$cmax\] selected?\n";

			#################################################
			# cpkg might not like the $cn, so check		#
			# whether this covers just specific versions.	#
			#################################################

			$creposl=$hashedinfo->{$cn}->[3];
			@S=split(/\|/,$creposl); $cver=$S[2];
			dprint "$cn\[$cver\] to install\n";
			dprint "vercmp($cver,$cmin) = ",vercmp($cver,$cmin),"\n";
			dprint "vercmp($cver,$cmax) = ",vercmp($cver,$cmax),"\n";
			if(vercmp($cver,$cmin)>=0 && 
				vercmp($cver,$cmax)<=0) {
				push @errors,"Installed package ".$cpkg->{NAME}." is not compatible with $cn\[$cver\]";
			}
		}
	}
	return(\@errors);
}

sub show_software_required($$$) {
my ($hashedinfo,$cpkgname,$traversed)=@_;
my ($clist,@F,$crepos,$creposl,$str);

	#################################################################
	# If this element has been visited before ignore it.		#
	#################################################################

	return if exists($traversed->{$cpkgname});

	#################################################################
	# If the current package has no dedendencies then just print	#
	# the name and do nothing further.				#
	#################################################################

	$traversed->{$cpkgname}=1;
	$clist=$hashedinfo->{$cpkgname}->[1];
	$crepos=$hashedinfo->{$cpkgname}->[2];
	$creposl=$hashedinfo->{$cpkgname}->[3];
	@F=split(/\|/,$creposl);
	$str=sprintf("%-*s [from $crepos, version $F[2]]",20,$cpkgname);
	logmsg($str);
	for (@$clist) {
		logmsg(sprintf("%*s  requires $_",20," "));
	}
	logmsg("");
	for (@$clist) {
		my ($cn,$max,$min)=split(/,/,$_);
		show_software_required($hashedinfo,$cn,$traversed);
	}
}

#########################################################################
# This is the routine that copies the package from a repository to the	#
# spool directory. This is inteligent - if the spool directory already	#
# contains a package of the same name it will check the repository	#
# index and only download the file if the checksum differs - this	#
# can help saving bandwidth if copies of the files have been manually	#
# copied to the spool directory.					#
#########################################################################

sub spool_software($$$$$;$) {
my ($hashedinfo,$spooldir,$cpkgname,$traversed,$action,$logger)=@_;
my ($clist,@F,$crepos,$creposl,$str,$checksum,$ctx);
my ($ok,$msg,$fd2,$calculated);

	#################################################################
	# Abort if an error has occured...				#
	#################################################################

	if(exists($hashedinfo->{'%ERROR%'})) {
		return;
	}

	#################################################################
	# If this element has been visited before ignore it.		#
	#################################################################

	return if exists($traversed->{$cpkgname});

	#################################################################
	# If the current package has no dedendencies then just print	#
	# the name and do nothing further.				#
	#################################################################

	$traversed->{$cpkgname}=1;
	$clist=$hashedinfo->{$cpkgname}->[1];
	$crepos=$hashedinfo->{$cpkgname}->[2];
	$creposl=$hashedinfo->{$cpkgname}->[3];
	@F=split(/\|/,$creposl);
	my $ti=time;
	# if(-f "$spooldir/$F[0]") {
		# if($action eq "overwrite") {
			# rename "$spooldir/$F[0]","$spooldir/$F[0].$ti;
		# }
	# }
	$checksum="NONE";
	if(defined($F[9])) {
		$checksum=$F[9];
	}
	#################################################################
	# If the file already exists in the spool directory get the	#
	# SHA1 value and if the same as the remote repository indiex,	#
	# then we do not need to drag it across again.			#
	#################################################################

	if(-f "$spooldir/$F[0]") {
		$ctx=Digest::SHA1->new;
		if(open($fd2,"$spooldir/$F[0]")) {
			$ctx->addfile($fd2);
		}
		close($fd2);
		$calculated=$ctx->hexdigest;
		$ctx=undef;

		if($calculated eq $checksum) {
			if(defined($logger)) {
				&$logger("Matching Local copy of $F[0] version $F[2] found.");
			}
			for (@$clist) {
				my ($cn,$max,$min)=split(/,/,$_);
				spool_software($hashedinfo,$spooldir,$cn,$traversed,$action,$logger);
			}
			return;
		}
		unlink "$spooldir/$F[0]";
	}
		#########################################################
		# The local copy does not match the remote, so carry	#
		# on processing.					#
		#########################################################
		
	if(! -f "$spooldir/$F[0]") {
		if(defined($logger)) {
			&$logger(sprintf("Getting %-*s [file=$F[0] , version=$F[2]] from $crepos",20,$cpkgname));
		} else {
			logmsg(sprintf("Getting %-*s [file=$F[0] , version=$F[2]] from $crepos",20,$cpkgname));
		}

		($ok,$msg)=get_package_from_path($crepos,$F[0],"$spooldir/$F[0].$ti");
		if(!$ok) {
			$hashedinfo->{'%ERROR%'}=$msg;
			return;
		}
		unlink("$spooldir/$F[0]") if -f "$spooldir/$F[0]";
		rename "$spooldir/$F[0].$ti","$spooldir/$F[0]";
	}

	#################################################################
	# Generate the SHA1 sum for the package, and abort if different.#
	#################################################################
	$ctx=Digest::SHA1->new;
	if(open($fd2,"$spooldir/$F[0]")) {
		$ctx->addfile($fd2);
	}
	close($fd2);
	$calculated=$ctx->hexdigest;
	if($calculated ne $checksum) {
		$hashedinfo->{'%ERROR%'}="\nRepository/Spooled package checksum mismatch for $F[1]:\nRepository checksum: $checksum\nCalculated checksum: $calculated \n";
		return;
	}

	for (@$clist) {
		my ($cn,$max,$min)=split(/,/,$_);
		spool_software($hashedinfo,$spooldir,$cn,$traversed,$action,$logger);
	}
}

sub fs_impact_actual($) {
my $installed_ref=$_[0];

	#################################################################
	# Load in file system utilisation info if not yet defined.	#
	#################################################################
	my @FS_NAMES=();
	my $fsinfo={}; my $fsinfo2={};
	if($Config{osname} eq "hpux") {
		open($fd,"bdf|") or 
			errormsg("Unable to ascertain current disk utilisation.",1);
	} else {
		open($fd,"df -k|") or 
			errormsg("Unable to ascertain current disk utilisation.",1);
	}
	while(<$fd>) {
		chomp; 
		@F=split;
		if($#F>2 && $F[$#F] =~ /^\//) {
			$fsinfo->{$F[$#F]}=$F[$#F-2];
			$fsinfo2->{$F[$#F]}=0;
			push @FS_NAMES,$F[$#F];
		}
	}
	close($fd);

	#################################################################
	# Sort names longest first, so first match is always the one	#
	# we actually need to use.					#
	#################################################################

	@FS_NAMES=sort {length($b) <=> length($a)} @FS_NAMES;

	my $last_dir=""; my $last_fs="";
	for (sort(keys(%$installed_ref))) {
		my $cdir=dirname($_);
		my $lm="";
		#########################################################
		# Seach for longest directory in fs list matching...	#
		#########################################################
		if($cdir eq $last_dir) {
			$lm=$last_fs;
		} else {
			for $ckey (@FS_NAMES) {
				if($cdir =~ /^$ckey/) {
					$lm=$ckey; last;
				}
			}
			errormsg("Unable to ascertain file system for \"$cdir\"!",1) if $lm eq "";
			$last_fs=$lm; $last_dir=$cdir;
		}

		#########################################################
		# Update fsinfo2 ...					#
		#########################################################

		$fsinfo2->{$lm}+=$installed_ref->{$_};
	}
	close($fd);
	return ($fsinfo,$fsinfo2);
}

#########################################################################
# This routine takes a reference to a hash of file systems, a		#
# reference to a hash of storage used, a namespace object and an	#
# absolute path for a tp2 package.					#
#									#
# For each file it rounds up the size to the nearest 4kb, which will	#
# suit most environments, though may produce some false results if	#
# a package has a large number of small files.				#
#									#
# Returns undef on error, fsinfo reference updated with free space	#
# if ok.								#
#########################################################################

sub fs_impact($$$$$$;$) {
my ($fsinfo,$fsinfo2,$nsobj,$cpkgfile,$bsize,$frsize,$abort_on_error)=@_;
my ($fd,@F,$cdir,$cname,$ROOT,$fp);
my ($lm,$ckey,$cval,$sz);

	$abort_on_error=1 if !defined($abort_on_error);
	$bsize=8192 if !defined($bsize);
	$frsize=8192 if !defined($frsize);
	$fsinfo2={} if !defined($fsinfo2);
	$ROOT=$nsobj->{ROOT};
	#################################################################
	# Load in file system utilisation info if not yet defined.	#
	#################################################################
	my @FS_NAMES=();
	if(!defined($fsinfo)) {
		$fsinfo={};
		if($Config{osname} eq "hpux") {
			if(!open($fd,"bdf|")) {
				if($abort_on_error) {
					errormsg("Unable to ascertain current disk utilisation.",1);
				} else {
					return(undef,["Unable to ascertain current disk utilisation."]);
				}
			}
		} else {
			if(!open($fd,"df -k|")) {
				if($abort_on_error) {
					errormsg("Unable to ascertain current disk utilisation.",1);
				 } else {
				 	return(undef,["Unable to ascertain current disk utilisation."]);
				}
			}
		}
		while(<$fd>) {
			chomp; 
			@F=split;
			if($#F>2 && $F[$#F] =~ /^\//) {
				$fsinfo->{$F[$#F]}=$F[$#F-2];
				push @FS_NAMES,$F[$#F];
			}
		}
		close($fd);
	} else {
		@FS_NAMES=keys(%$fsinfo);
	}

	# Sort names longest first, so first match is always the one
	@FS_NAMES=sort {length($b) <=> length($a)} @FS_NAMES;

	if(! -f $cpkgfile || ! -r $cpkgfile) {
		if($abort_on_error) {
			errormsg("Unable to find or read package $cpkgfile.",1);
		} else {
			return(undef,["Unable to find or read package $cpkgfile."]);
		}
	}
	if(! open($fd,_get_tar_cmd()." tvzf $cpkgfile 2>/dev/null|")) {
		if($abort_on_error) {
			errormsg("Unable to run gtar: $!",1);
		} else {
			return(undef,["Unable to run gtar: $!"]);
		}
	}

	my $last_dir=""; my $last_fs="";
	while(<$fd>) {
		chomp;
		@F=split;
		$sz=$F[2];
		#########################################################
		# Since the name might include spaces we can't simply	#
		# take the last field, so we find the time and strip	#
		# off everything to that point.				#
		#########################################################

		s/.*[0-9]{2}:[0-9]{2}:[0-9]{2} //;
		$fp=dirname($_);
		$fp =~ s/^\.//;
		$cdir="$ROOT/$fp";
		$cdir =~ s/\/{2,}/\//g;	# remove duplicate slashes,
		$cdir =~ s/\/$//g;	# remove trailing slashes

		$lm="";
		#########################################################
		# Seach for longest directory in fs list matching...	#
		#########################################################
		if($cdir eq $last_dir) {
			$lm=$last_fs;
		} else {
			for $ckey (@FS_NAMES) {
				if($cdir =~ /^$ckey/) {
					$lm=$ckey; last;
				}
			}
			if($lm eq "") {
				if($abort_on_error) {
					errormsg("Unable to ascertain file system for \"$cdir\"!",1);
				} else {
					return(undef,["Unable to ascertain file system for \"$cdir\"!"]);
				}
			}
			$last_fs=$lm; $last_dir=$cdir;
		}

		#########################################################
		# Update fsinfo and fsinfo2 ...				#
		#########################################################

		$blocks=$sz/$bsize;
		if($blocks != int($blocks)) {
			my $xxx=$blocks-int($blocks);
			my $frs=$xxx/$frsize;
			if($frs!=int($frs)) {
				$frs++;
				$frs=int($frs);
			}
			#################################################
			# set real disk impact.				#
			#################################################
			$sz=int($blocks)*$bsize + $frs*$frsize;
		}
		$sz/=1024;
		$fsinfo->{$lm}-=$sz;
		if(!exists($fsinfo2->{$lm})) {
			$fsinfo2->{$lm}=0;
		}
		$fsinfo2->{$lm}+=$sz;
	}
	close($fd);

	return ($fsinfo,$fsinfo2);;
}

sub do_software_installs($$$$$$$$$) {
my ($hashedinfo,$spooldir,$nsobj,$preview,$force,$binpath,$preinstall,$postinstall,$verbose)=@_;
my ($cmd,$ckey,$cval,@F,$inst,$root,$n,$rc);

	# Reset all 'traversed' flags
	foreach (keys(%$hashedinfo)) {
		$hashedinfo->{$_}->[0]=0;
	}

	$root=$nsobj->{ROOT};
	$inst=1;
	while($inst) {
		$inst=0;
		foreach $ckey (keys(%$hashedinfo)) {
			$cval=$hashedinfo->{$ckey};
			next if $cval->[0] !=0;

			@F=split(/\|/,$cval->[3]);
			#################################################
			# Number of dependencies is 0, so install 	#
			# this item of software.			#
			#################################################

			$n="install+$ckey+" . time;
			logmsg("Installation of $ckey starting - please wait.");
			logmsg("Package file : $F[0]");
			logmsg("Log file     : $n");
			$inst=1;
			if($binpath ne "") {
				$cmd="$binpath/_tp2install_ll  --namespace=".$nsobj->{NAME}." ";
			} else {
				$cmd="_tp2install_ll  --namespace=".$nsobj->{NAME}." ";
			}
			$cmd.="--pkgfile $spooldir/$F[0] ";
			$cmd.="--verbose " if $verbose;
			$cmd.="--preview " if $preview;
			$cmd.="--logfile /var/adm/tp2/ns/".$nsobj->{NAME}."/log/$n ";
			$cmd.="--force " if $force;
			$cmd.="--nopreinstall " if $preinstall==0;
			$cmd.="--nopostinstall " if $postinstall==0;

			system("umask 022; $cmd");
			$rc=$?>>8;
			if($rc) {
				errormsg("Aborting after RC=$rc from installation of $ckey.");
				errormsg("[Use \"tp2log --namespace " . $nsobj->{NAME} . " --show $n\" for details]",2);
			}
			logmsg("Installation of $ckey was successful.");
			logmsg("[Use \"tp2log --namespace " . $nsobj->{NAME} . " --show $n\" for details]");
			logmsg("");
			#################################################
			# Now it has been successful, scan all packages	#
			# and remove this from this of dependencies.	#
			#################################################

			for (keys(%$hashedinfo)) {
				my ($cval2,$xx,$cc);

				$xx=$_;
				$cval2=$hashedinfo->{$xx};
				next if ! $cval2->[0];

				if(scalar(grep {/^$ckey,/} @{$cval2->[1]})) {
					$cval2->[0]--;
					@F=();
					foreach (@{$cval2->[1]}) {
						$cc=$_;
						push @F,$cc if $cc !~ /^$ckey,/;
					}
					$cval2->[1]=\@F;
				}
			}

			#################################################
			# Indicate this package has been installed..	#
			#################################################

			$cval->[0]=-1;
		}
	}
}

#########################################################################
# If logger is supplied then any messages and errors goes to that	#
# instead of the inbuilt errormsg routine...				#
#########################################################################

sub check_for_all_clashes($$$$;$) {
my ($hashedinfo,$spooldir,$nsobj,$strict,$logger)=@_;
my ($cpkg,$cval,@F,$pkgobj,$cmd,$msg,$clist,@mlist);

	$pwd=cwd;
	@mlist=();
	if(! mkdir("/tmp/tmpconfig.$$")) {
		if(!defined($logger)) {
			errormsg("Unable to create temporary directory '/tmp/tmpconfig.$$': $!",1);
		} else {
			&$logger("error","Unable to create directory '/tmp/tmpconfig.$$': $!");
			return undef;
		}
	}

	if(!chdir("/tmp/tmpconfig.$$")) {
		if(!defined($logger)) {
			errormsg("Unable to change directory to '/tmp/tmpconfig.$$': $!",1);
		} else {
			&$logger("error","Unable to change directory to '/tmp/tmpconfig.$$': $!");
			return undef;
		}
	}

	for $cpkg (keys %$hashedinfo) {
		$cval=$hashedinfo->{$cpkg};
		@F=split(/\|/,$cval->[3]);

		&$logger("note","Checking for namespace clashes in package '$cpkg' ...") if defined($logger);

		#########################################################
		# Extract the !!..config..!! file, and parse.		#
		#########################################################

		if(! -f "$spooldir/$F[0]") {
			if(!defined($logger)) {
				warnmsg("Unable to find file $spooldir/$F[0] - ignoring.");
			} else {
				&$logger("warning","Unable to find file $spooldir/$F[0] - ignoring.");
			}
			next;
		}
		$cmd=_get_tar_cmd()." xzf $spooldir/$F[0] ./!!..config..!!";
		system("$cmd 2>/dev/null >/dev/null");
		if($?>>8) {
			if(!defined($logger)) {
				warnmsg("Unable to extract config from $spooldir/$F[0] - ignoring.");
			} else {
				&$logger("warning","Unable to extract config from $spooldir/$F[0] - ignoring.");
			}
			next;
		}
		($pkgobj,$msg)=pkgfile_config_to_obj("!!..config..!!");
		if(!defined($pkgobj)) {
			if(!defined($logger)) {
				warnmsg("Unable to parse config file for $F[4] - ignoring.");
				warnmsg("Error was : $msg");
			} else {
				&$logger("warning","Unable to parse config file for $F[4] - ignoring.");
				&$logger("warning","Error was : $msg");
			}
			next;
		}
		$clist=file_clash($pkgobj,$nsobj,$strict);
		if(scalar(@$clist)) {
			push @mlist,[$cpkg,$clist];
		}
	}
	chdir $pwd;
	remove_dir_path("/tmp/tmpconfig.$$");
	return \@mlist;
}

sub get_owning_namespace($$) {
my ($dir,$ignore_root)=@_;
my ($dfd,$cent,$cline,$fd,%a);
my ($ml,$ml2,$ml3);

	$ml=0; $ml2=""; $ml3="";
	return undef if ! -d "/var/adm/tp2/ns";
	opendir($dfd,"/var/adm/tp2/ns") or return undef;
	while($cent=readdir $dfd) {
		next if $cent eq "." || $cent eq "..";
		next if $cent eq "root" && $ignore_root;
		next if ! -d "/var/adm/tp2/ns/$cent";
		next if ! -f "/var/adm/tp2/ns/$cent/config";
		next if ! -r "/var/adm/tp2/ns/$cent/config";
		open($fd,"/var/adm/tp2/ns/$cent/config") or next;
		%a=();
		while($cline=<$fd>) {
			chomp $cline;
			next if $cline !~ /^[A-Z]+/;
			@F=split(/\s+/,$cline);
			$a{$F[0]}=$F[1];
		}
		if(exists($a{ROOT})) {
			my $c=$a{ROOT};
			if(length($c)<=length($dir)) {
				if(substr($dir,0,length($c)) eq $c &&
					length($c)>$ml) {
					$ml=length($c);
					$ml2=$c;
					$ml3=$cent;
				}
			}
		}
		close($fd);
	}
	closedir($dfd);
	if($ml2 ne "") {
		return $ml3;
	}
}

sub get_owning_package($$) {
my ($ns_obj,$fullpath)=@_;
my ($moddedname,$sha1obj,$fpsha1,$cdir,$namespace);
my (@entries,$cfile,$fd,@F,@l);

	$moddedname=$fullpath;
	$moddedname=~ s/\/{2,}/\//g;
	$sha1obj=Digest::SHA1->new;
	$sha1obj->add($moddedname);
	$fpsha1=$sha1obj->hexdigest;
	$namespace=$ns_obj->{NAME};
	$cdir="/var/adm/tp2/ns/$namespace/meta/".substr($fpsha1,0,2);
	if(! -d "$cdir") {
		return(undef,"No such file in namespace.");
	}
	@entries=();
	$cfile="$cdir/$fpsha1";
	if(-f $cfile) {
		if(!open($fd,"$cfile")) {
			return(undef,"Unable to read existing meta data entry $cfile");
		}
		@entries=<$fd>; close($fd);
		chomp foreach (@entries);
	}
	@l=();
	foreach (@entries) {
		@F=split(/\|/,$_);
		if($F[0] ne $moddedname) {
			next;
		}
		push @l,$F[$#F];
	}
	return(\@l,undef);
}

#########################################################################
# Purpose:	Return a list of namespaces - each being represented	#
#		by a hash to the details.				#
# Return values:							#
# nslistref	- Ref to list of namespace hash refs			#
# warnings	- Ref to list of warnings				#
# errors	- Ref to list of errors					#
#########################################################################

sub get_ns_list() {
my ($dfd,$cent,@nslist,$nsobj);
my (@errors,@warnings,$obj,$err);

	@errors=(); @warnings=();
	@nslist=();
	$nsobj=undef;
	if(! -d "/var/adm/tp2/ns") {
		push @errors,"TP2 namespace directory \"/var/adm/tp2/ns\" does not exist.";
		return(\@nslist,\@warnings,\@errors);
	}
	if(!opendir($dfd,"/var/adm/tp2/ns")) {
		push @errors,"Unable to open \"/var/adm/tp2/ns\": $!";
		return(\@nslist,\@warnings,\@errors);
	}
	while($cent=readdir($dfd)) {
		next if $cent eq "." || $cent eq "..";
		next if ! -d "/var/adm/tp2/ns/$cent";
		if(! -f "/var/adm/tp2/ns/$cent/config") {
			push @warnings,"Directory $cent found with no config file!";
			next;
		}

		#########################################################
		# Found a namespace and config, so load it.		#
		#########################################################
		$obj=undef;
		($obj,$err)=namespace_to_obj($cent);
		if(!defined($obj)) {
			push @warnings,"Unable to decode namespace \"$cent\".";
			next;
		}
		push @nslist,$obj;
	}
	closedir($dfd);
	return(\@nslist,\@warnings,\@errors);
}


#########################################################################
# This returns a list where each element is a reference to a hash with	#
# the contents of the package config, with the additionel key 		#
# __STATUS__ containing "installed", "installing", "removing".		#
#									#
# Argument pased can be the name of the namespace, or a reference	#
# to a namespace object.						#
#									#
# 30/11/2006	Fixed a bug where a package in the installing state	#
#		is included twice in the returned details.		#
#########################################################################

sub get_ns_pkglist {
my $namespace=$_[0];
my $action=$_[1];
my ($ns_obj,$err,$cent,$dfd,$obj,$status);
my (@plist,$count);

	$action="detail" if !defined($action);
	@plist=(); $count=0;
	if(ref($namespace)) {
		$ns_obj=$namespace;
		$namespace=$ns_obj->{NAME};
	} else {
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(undef,$err);
		}
	}
	if(! -r "/var/adm/tp2/ns/$namespace/pkg") {
		return(undef,"Unable to read from namespace package directory: $!");
	}
	if(! -x "/var/adm/tp2/ns/$namespace/pkg") {
		return(undef,"Unable to read from namespace package directory: $!");
	}

	opendir($dfd,"/var/adm/tp2/ns/$namespace/pkg") or
		return(undef,"Unable to read from namespace package directory: $!");

	my $had_errors=0;
	my $had_ok=0;
	my $last_error="";
	my %dir_ends=();
	$dir_ends{$_}=1 foreach (readdir($dfd));
	my @ents=();
	closedir($dfd);
	for (keys(%dir_ends)) {
		next if /^\.first boot for /;
		next if /\$\$\$ trans$/;
		next if $_ eq "rm on reboot";
		next if ! -f "/var/adm/tp2/ns/$namespace/pkg/$_";
		next if exists($dir_ends{"$_ install \$\$\$"});
		push @ents,$_;
	}

	for $cent (@ents) {
		if($action eq "count") {
			$count++;
			next;
		}
		($obj,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$cent");
		if(defined($err)) {
			$had_errors++;
			$last_error=$err;
			next;
		}
		if(!defined($obj)) {
			warnmsg("Unable to parse config for package \"$cent\" object:\n$err");
			$had_errors++;
			$last_error=$err;
			next;

		}

		#########################################################
		# Use the pkg_install_state to get status.		#
		#########################################################
		my $ok2;
		($ok2,$status)=pkg_install_state($ns_obj,$cent);
		if($ok2==1) {
			$status="Installed";
		} 
		$obj->{__STATUS__}=$status;
		$had_ok++;
		push @plist,$obj;
	}
	closedir($dfd);
	return($count,undef) if $action eq "count";
	if($last_error ne "" && $had_ok==0) {
		return([],$last_error);
	}
	return(\@plist,undef);
}

sub get_ns_log_list($) {
my $namespace=$_[0];
my ($ns_obj,$err,$cent,$dfd,$obj,@F);
my (@llist,$type);

	@llist=();
	if(ref($namespace)) {
		$ns_obj=$namespace;
		$namespace=$ns_obj->{NAME};
	} else {
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(undef,$err);
		}
	}

	opendir($dfd,"/var/adm/tp2/ns/$namespace/log") or
		return(undef,"Unable to read from namespace log directory: $!");

	while($cent=readdir($dfd)) {
		next if ! -f "/var/adm/tp2/ns/$namespace/log/$cent";
		next if $cent eq ".lock";
		next if $cent !~ /.*\+.*/;
		@sb=stat(_);
		@F=split(/\+/,$cent);

		#########################################################
		# Attempt to open the log, handling the fact that the	#
		# log might actually be gzip compressed.		#
		#########################################################

		my (@FF,$fd,$fname,$preview,$result,$user,$sys,$lastline);
		$preview="Unknown"; $result="Unknown"; $user=0.0; $sys=0.0;
		if(lc($F[0]) eq "boot") {
			$preview="N/A";
		}
		if($cent =~ /\.gz$/) {
			$fname="gunzip -c /var/adm/tp2/ns/$namespace/log/$cent|";
		} elsif($cent =~ /\.Z$/) {
			$fname="zcat /var/adm/tp2/ns/$namespace/log/$cent|";
		} elsif($cent =~ /\.bz2$/) {
			$fname="bunzip2 -c /var/adm/tp2/ns/$namespace/log/$cent|";
		} else {
			$fname="/var/adm/tp2/ns/$namespace/log/$cent";
		}
		if(open($fd,$fname)) {
			$result="Fail";
			$lastline="";
			my @lines=();
			my $prevline="";
			while($cline=<$fd>) {
				chomp $cline;
				push @lines,$cline;
				if($cline =~ /^Log\s*:\s*Arguments:/ && $preview eq "Unknown") {
					$preview="Yes" if $cline =~ /--preview/;
					$preview="No" if $cline !~ /--preview/;
				}
			}
			$lastline=$lines[$#lines];
			$prevline=$lines[$#lines-1];

			# Success or failure is defined by the contents	#
			# of the very last line...			#

			if($lastline =~ /Package files for.*removed\.$/) {
				$result="Success";
			}
			if($lastline =~ /Installed package.*successfully\.$/) {
				$result="Success";
			}
			if($lastline =~ /^Log\s*:\s*Result:/) {
				if($lastline =~ /SUCCESS/) {
					$result="Success";
					@FF=split(/\s+/,$lastline);
					$user=$FF[5]; $sys=$FF[10];
				}
			}
			if(lc($F[0]) eq "boot") {
				if($prevline =~ /Log  : Result: SUCCESS/ || $lastline =~ /Log  : Result: SUCCESS/) {
					$result="Success";
				} else {
					$result="Fail";
				}
			}
			close($fd);
		}

		if(lc($F[0]) eq "boot") {
			$F[1]="N/A";
		}
		push @llist,[$cent,$F[0],$F[1],$sb[7],$sb[9],$preview,$result,$user,$sys];
	}
	closedir($dfd);
	return(\@llist,undef);
}

#########################################################################
# Remove any old logs from the specified namespace to ensure space used	#
# for logs fall into the configured guidelines. This requires the	#
# MAXLOGS and MAXLOGSPACE to be set - defaulting to 50 - rounded up	#
# if less, and 2(MB) - rounded up if less.				#
#									#
# Arguments are:							#
# namespace - The name of the namespace.				#
# ns_obj    - Reference to ns object. Can be undefined.			#
#########################################################################

sub manage_ns_logs($) {
my ($ns_obj)=@_;
my ($dfd,$err,%dhash,$cent,@sb,$tspace);
my ($maxlogspace,$namespace,$maxlogs);

	if(ref($ns_obj)) {
		$namespace=$ns_obj->{NAME};
	} else {
		$namespace=$on_obj;
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(0,$err);
		}
	}

	if(!defined($ns_obj)) {
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(0,$err);
		}
	}

	if(exists($ns_obj->{MAXLOGS})) {
		$maxlogs=$ns_obj->{MAXLOGS};
	} else {
		$maxlogs=50;
	}
	$maxlogs=50 if $maxlogs<50;

	if(exists($ns_obj->{MAXLOGSPACE})) {
		$maxlogspace=$ns_obj->{MAXLOGSPACE};
	} else {
		$maxlogspace=2;
	}
	$maxlogspace=2 if $maxlogspace<2;

	#################################################################
	# Open the log directory and include list of files...		#
	#################################################################

	opendir($dfd,"/var/adm/tp2/ns/$namespace/log") or
		return(0,"Unable to open directory /var/adm/tp2/ns/$namespace/log: $!");
	while($cent=readdir($dfd)) {
		next if ! -f "/var/adm/tp2/ns/$namespace/log/$cent";
		@sb=stat(_);
		$dhash{$cent}=[$sb[9],$sb[7]];
	}
	closedir($dfd);
	
	#################################################################
	# Check count of logs and get rid of any older ones required.	#
	#################################################################

	my @oldest=sort {$dhash{$a}->[0] cmp $dhash{$b}->[0]} (keys %dhash);
	my $rmi=0;
	while(scalar(keys(%dhash)) > $maxlogs) {
		return(1,undef) if $rmi > $#oldest;
		return(0,"Unable to remove file $oldest[$rmi]: $!") 
			if ! unlink("/var/adm/tp2/ns/$namespace/log/$oldest[$rmi]");
		delete $dhash{$oldest[$rmi]};
		$rmi++;
	}

	#################################################################
	# Work out the space occupied by the remaining log files.	#
	#################################################################

	$tspace=0;
	map {$tspace+=$dhash{$_}->[1]} (keys %dhash);

	while($tspace/1024 >1024*$maxlogspace) {
		return(1,undef) if $rmi > $#oldest;
		return(0,"Unable to remove file $oldest[$rmi]: $!")
			if ! unlink("/var/adm/tp2/ns/$namespace/log/$oldest[$rmi]");
		$tspace-=$dhash{$oldest[$rmi]}->[1];
		delete $dhash{$oldest[$rmi]};
		$rmi++;
	}
	return(1,undef);
}

#########################################################################
# Remove any old spoolled packages for the specified namespace.		#
# Based on the settings for MAXSPOOLSPACE. If not set will default to 0.#
# Value is givin in MB.							#
#									#
# Arguments are:							#
# namespace - The name or reference of the namespace.			#
# ns_obj    - Reference to ns object. Can be undefined.			#
#########################################################################

sub manage_ns_spool($) {
my ($ns_obj)=@_;
my ($dfd,$err,%dhash,$cent,@sb,$tspace);
my ($namespace,$maxtmpspace,$tmpdir);

	if(ref($ns_obj)) {
		$namespace=$ns_obj->{NAME};
	} else {
		$namespace=$on_obj;
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(0,$err);
		}
	}

	if(exists($ns_obj->{MAXSPOOLSPACE})) {
		$maxtmpspace=$ns_obj->{MAXSPOOLSPACE};
	} else {
		$maxtmpspace=0;
	}
	$maxtmpspace=0 if $maxtmpspace<0;

	if(exists($ns_obj->{TMPDIR})) {
		$tmpdir=$ns_obj->{TMPDIR};
	} else {
		$tmpdir="/var/adm/tp2/ns/$namespace/spool";
	}
	#################################################################
	# Open the spool directory and get list of file contents.	#
	#################################################################

	opendir($dfd,$tmpdir) or
		return(0,"Unable to open directory $tmpdir: $!");
	while($cent=readdir($dfd)) {
		next if ! -f "$tmpdir/$cent";
		@sb=stat(_);
		$dhash{$cent}=[$sb[9],1024*$sb[12]];
	}
	closedir($dfd);
	my @oldest=sort {$dhash{$a}->[0] cmp $dhash{$b}->[0]} (keys %dhash);
	
	#################################################################
	# Work out the space occupied by the remaining log files.	#
	#################################################################

	$tspace=0;
	map {$tspace+=$dhash{$_}->[1]} (keys %dhash);
	my $rmi=0;

	dprint "manage_ns_spool: Total space of spoolled files (MB): ",sprintf("%.2f\n",$tspace/1024/1024);
	dprint "manage_ns_spool: Maximum defined space (MB)        : $maxtmpspace\n";
	dprint "manage_ns_spool: Spoolled file count               : ",scalar(@oldest),"\n";

	while($tspace/1024/1024>$maxtmpspace) {
		return(1,undef) if $rmi > $#oldest;
		return(0,"Unable to remove file $oldest[$rmi]: $!")
			if ! unlink("$tmpdir/$oldest[$rmi]");
		$tspace-=$dhash{$oldest[$rmi]}->[1];
		dprint "manage_ns_spool: Deleted package $oldest[$rmi]\n";
		delete $dhash{$oldest[$rmi]};
		$rmi++;
	}
	return(1,undef);
}

#########################################################################
# By default a repackage directory has no limits, but if the namespace	#
# configuration defines a REPACKAGE_MAXSPACE setting then this routine	#
# will the oldest packages until they occupy less space than configured.#
#########################################################################

sub manage_repackage_space($) {
my ($ns_obj)=@_;
my ($dfd,$err,%dhash,$cent,@sb,$tspace);
my ($namespace,$maxtmpspace);

	return(1,undef);
	if(ref($ns_obj)) {
		$namespace=$ns_obj->{NAME};
	} else {
		$namespace=$ns_obj;
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(0,$err);
		}
	}

	if(exists($ns_obj->{REPACKAGE_MAXSPACE})) {
		$maxtmpspace=$ns_obj->{REPACKAGE_MAXSPACE};
	} else {
		$maxtmpspace=0;
	}
	$maxtmpspace=0 if $maxtmpspace<0;
	return(1,undef) if $maxtmpspace==0;

	#################################################################
	# If we get here we have to limit the space occupied.		#
	#################################################################
	my $repackage_dir="/var/adm/tp2/ns/$namespace/repackage";
	if(exists($ns_obj->{REPACKAGE_DIR})) {
		$repackage_dir=$ns_obj->{REPACKAGE_DIR};
	}
	return(1,undef) if ! -d $repackage_dir;

	#################################################################
	# Open the spool directory and get list of file contents.	#
	#################################################################

	opendir($dfd,$repackage_dir) or
		return(0,"Unable to open directory $repackage_dir: $!");
	while($cent=readdir($dfd)) {
		next if ! -f "$repackage_dir/$cent";
		@sb=stat(_);
		$dhash{$cent}=[$sb[9],1024*$sb[12]];
	}
	closedir($dfd);
	my @oldest=sort {$dhash{$a}->[0] cmp $dhash{$b}->[0]} (keys %dhash);
	
	#################################################################
	# Work out the space occupied by the remaining log files.	#
	#################################################################

	$tspace=0;
	map {$tspace+=$dhash{$_}->[1]} (keys %dhash);
	my $rmi=0;

	dprint "manage_repackage_space: Total space of spoolled files (MB): ",sprintf("%.2f\n",$tspace/1024/1024);
	dprint "manage_repackage_space: Maximum defined space (MB)        : $maxtmpspace\n";
	dprint "manage_repackage_space: Spoolled file count               : ",scalar(@oldest),"\n";

	while($tspace/1024/1024>$maxtmpspace) {
		return(1,undef) if $rmi > $#oldest;
		return(0,"Unable to remove file $oldest[$rmi]: $!")
			if ! unlink("$repackage_dir/$oldest[$rmi]");
		$tspace-=$dhash{$oldest[$rmi]}->[1];
		dprint "manage_repackage_space: Deleted package $oldest[$rmi]\n";
		delete $dhash{$oldest[$rmi]};
		print "rmi=$rmi\n";
		$rmi++;
	}
	return(1,undef);
}



sub file_installed_by($$$) {
my ($ns_obj,$cfile,$what)=@_;
my ($name,$moddedname,$sha1obj,$fpsha1,$cdir);
my ($namespace,$fd,@E,@F,@G,$root);

	$namespace=$ns_obj->{NAME};
	$root=$ns_obj->{ROOT};
	$moddedname="$root/$cfile";
	$moddedname=~ s/\/{2,}/\//g;
	if(exists($SHA1_MAP{"$moddedname"})) {
		@F=grep {/^$moddedname\|/} @{$SHA1_MAP{$cfile}}; close($fd);
	} else {
		$sha1obj=Digest::SHA1->new;
		$sha1obj->add($moddedname);
		$fpsha1=$sha1obj->hexdigest;
		dprint "file_installed_by: namespace:$namespace,cfile=$cfile,sha1=$fpsha1\n";
		$cdir="/var/adm/tp2/ns/$namespace/meta/".substr($fpsha1,0,2);
		if(! -d $cdir) {
			return (undef,"File $moddedname not in meta-data:\nNo directory $cdir exists.");
		}
		if(! -f "$cdir/$fpsha1") {
			return (undef,"File $moddedname not in meta-data:\nNo file $cdir/$fpsha1 exists.");
		}

		open($fd,"$cdir/$fpsha1") or
			return (undef,"File $moddedname could not be opened in meta-data!");
		@F=(map {chomp;$_;} <$fd>); close($fd);
		$SHA1_MAP{"$root/$cfile"}=[@F];
		@F=grep {/^$moddedname\|/} (@F);
	}
	return (undef,"File $moddedname not in meta-data:\nFile is $cdir/$fpsha1.") if !scalar(@F);
	if($what == TP2_LATEST_PACKAGE) {
		@E=split(/\|/,$F[$#F]);
		return($E[5],$F[$#F]);
	} else {
		@E=map {@G=split(/\|/); $G[5];} @F;
		return(\@E,undef);
	}
}

#########################################################################
# This routine will attempt to gain a lock on a specified namespace.	#
# [1] The namespace can be the name or the reference to hash details.	#
# [2] The type of lock to grant - can be TP2_READ or TP2_WRITE.		#
#									#
# Need to import :locking from tp2utils to get these functions and	#
# 'constants'.								#
# Will return (1,undef) if OK, or (0,msg) on failure.			#
# Will time-out after 5 seconds...					#
#########################################################################

sub TP2_READLOCK() {1}
sub TP2_WRITELOCK() {2}

my %locked_fds=();

sub gain_lock($$) {
my ($namespace,$type)=@_;
my ($ns_obj,$err,$loop,$l,$fd);

	if(ref($namespace)) {
		$ns_obj=$namespace;
		$namespace=$ns_obj->{NAME};
	} else {
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(undef,$err);
		}
	}

	#################################################################
	# Locks are granted by locking the .lock file for the 		#
	# log directory.						#
	#################################################################

	$loop=0;
	if($type == TP2_WRITELOCK) {
		open($fd,">>/var/adm/tp2/ns/$namespace/log/.lock") or
			return(0,"Unable to open config file for /var/adm/tp2/ns/$namespace/log/.lock:\n$!");
	} else {
		if(! -f "/var/adm/tp2/ns/$namespace/log/.lock") {
			open($fd,">/var/adm/tp2/ns/$namespace/log/.lock") or
				return(0,"Unable to write to /var/adm/tp2/ns/$namespace/log/.lock:\n$!");
			close($fd);
		}
		open($fd,"/var/adm/tp2/ns/$namespace/log/.lock") or
			return(0,"Unable to open config file for /var/adm/tp2/ns/$namespace/log/.lock:\n$!");
	}
	while($loop<10) {
		if($type == TP2_READLOCK) {
			$l=flock($fd,LOCK_SH|LOCK_NB);
		} else {
			$l=flock($fd,LOCK_EX|LOCK_NB);
		}
		if(!$l) {
			select(undef,undef,undef,1);
			$loop++;
			next;
		}
		$locked_fds{$namespace}=$fd;
		return(1,undef);
	}
	return(0,"Time-out attempting to gain lock.");
}

sub release_lock($) {
my ($namespace)=@_;
my ($ns_obj,$err);

	if(ref($namespace)) {
		$ns_obj=$namespace;
		$namespace=$ns_obj->{NAME};
	} else {
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(undef,$err);
		}
	}
	if(!exists($locked_fds{$namespace})) {
		return(0,"No current lock on namespace $namespace.");
	} 
	flock($locked_fds{$namespace},LOCK_UN);
	delete $locked_fds{$namespace};
	return(1,undef);
}

#########################################################################
# This routine will remove all the files for a package. As per the 	#
# design document it does so in a very controlled manner to allow a 	#
# recovery if a problem arises.						#
#									#
# Also it will attempt to remove the file ONLY IF THE FILE TO REMOVE	#
# IS NOT REFERENCED IN ANY OTHER PACKAGE.				#
#									#
# The routine uses keyname:value pairs since it requires a number of	#
# parameters:								#
# PACKAGE	Name of the package to remove.				#
# NAMESPACE	Namespace to install file into. The value of the	#
#		ROOTINSTALLABLE attribute is not checked here, the	#
#		caller is expected to do that.				#
# [OBJ]		Reference to package config object - optional.		#
# [HOOK]	The name of a routine to call after every successfull	#
#		file removal - files removed, space freed.		#
# [PREVIEW]	If exists and set to 1, then NO FILES are removed.	#
#		However, will call the HOOK function, if any file	#
#		does need to be removed.				#
# [LOGMSG,ERRORMSG,WARNMSG] - Override the default routines for 	#
#		logging information.					#
#									#
# Returns (1,undef)	- files removed as requested.			#
# 	  (0,msg)	- error ... see msg				#
#########################################################################

sub remove_pkg_files_1 {
my %args=@_;
my ($package,$namespace,$hook,$package_name);
my ($obj,@F,$ROOT);
my ($wfd,$cline,$cdir,$cfile,$xx);
my ($uname,$gname,$nsobj,$preview,$ip,$err,$fcount);


	my $logmsg=\&logmsg;
	my $warnmsg=\&warnmsg;
	my $errormsg=\&errormsg;
	$logmsg=$args{LOGMSG} if exists($args{LOGMSG});
	$warnmsg=$args{WARNMSG} if exists($args{WARNMSG});
	$errormsg=$args{ERRORMSG} if exists($args{ERRORMSG});

	$uname=getpwuid($UID); $gname=_getgrgid($GID);
	$hook=undef; $preview=0;
	if(!exists($args{NAMESPACE}) || !exists($args{PACKAGE})) {
		return(0,"Missing mandatory arguments.");
	}

	$preview=$args{PREVIEW} if exists($args{PREVIEW});
	$package=$args{PACKAGE};
	$namespace=$args{NAMESPACE};
	$hook=$args{HOOK} if exists($args{HOOK});

	if(exists($args{OBJ})) {
		$obj=$args{OBJ};
	} else {
		if(! -f "/var/adm/tp2/ns/$namespace/pkg/$package") {
			return(0,"Package \"$package\" not found in namespace \"$namespace\".");
		}
		($obj,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$package");
		return(0,$err) if defined($err);
	}

	($nsobj,$err)=namespace_to_obj($namespace);

	if(!exists($nsobj->{ROOT})) {
		return(0,"Missing ROOT setting for namespace $namespace.");
	}
	$ROOT=$nsobj->{ROOT};
	$ROOT=~ s/\/+$//; $ROOT="/" if $ROOT eq "";	# remove trailing /'s

	if(! -d $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace does not exist.");
	}
	if(! -w $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace not writable.");
	}

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$") {
		return(0,"Previous failed removal not cleaned up - use tp2clean first!");
	}

	#################################################################
	# First step is to rename the package file from pkg to		#
	# "pkg removing $$$".						#
	#################################################################

	if(!$preview) {
		rename("/var/adm/tp2/ns/$namespace/pkg/$package",
			"/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$") or
				return(0,"Unable to rename package file.");

		if(!open($wfd,">/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$ trans")) {
			rename("/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$",
				"/var/adm/tp2/ns/$namespace/pkg/$package");
			return(0,"Unable to open transaction file for package removal.");
		}
	}

	#################################################################
	# Ensure all changes to wfd are flushed ASAP			#
	#################################################################

	$xx=select $wfd;
	$|=1;
	select $xx;

	#################################################################
	# Now all directories are created, the order being based on	#
	# the shortest length paths first.				#
	#################################################################

	my $total_files=scalar(@{$obj->{FILES}});
	$fcount=0;
	for $cfile (@{$obj->{FILES}}) {
		my $t;

		@F=split(/\|/,$cfile);
		$t=$F[6];
		$fcount++;
		if(defined($hook)) {
			&$hook($fcount,0,$total_files,$t,"Staging");
		}

		#########################################################
		# If the file does not exist simply continue, since the	#
		# meta informatino might still be in place and need	#
		# removing.						#
		#########################################################

		# next if ! -f "$ROOT/$t";

		($ip,$err)=file_installed_by($nsobj,$t,TP2_ALL_PACKAGES);

		if(!defined($ip)) {
			return(0,"Unable to get details for file:\n$err");
		}

		#########################################################
		# If the file is in more than a single package, simply	#
		# update the meta info, otherwise rename.		#
		#########################################################

		if(scalar(@$ip)>1) {
			print $wfd "UM $t\n" if !$preview;
			&$logmsg("Updating: File '$t' not removed - multiple owners.");
			&$logmsg("          [Owners " . join(",",@$ip) . "]");
		} else {
			my ($curr,$om,@OM,$orig);
			my $t2=$ROOT . "/$t";

			&$logmsg("Removing: File '$t'.");
			next if $preview;
			$om=get_file_meta($t2,$namespace);

			if(!defined($om)) {
				#########################################
				# No meta info?? Remove file.		#
				#########################################
				if(-f $t2) {
					rename($t2,$t2.".remove.\$\$") or
						return(0,"Unable to temporarily rename $t2!");
				}
				print $wfd "RM $t\n";
			} else {
				dprint "om=$om\n";
				if(! -f $t2) {
					print $wfd "RM $t\n";
					next;
				}
				$curr=file_sha1($t2);
				@OM=split(/\|/,$om);
				$orig=$OM[1];
				# Handle configuration files in a specical manner
				if($orig =~ /^CFGFILE=/) {
					$orig =~ s/^CFGFILE=//;
					if($orig eq $curr) {
					#################################
					# Same as original, so remove.	#
					#################################
						rename($t2,$t2.".remove.\$\$") or
							return(0,"Unable to temporarily rename $t2!");
						print $wfd "RM $t\n";
					} else {
					#################################
					# Altered, so rename instead.	#
					#################################
						rename($t2,"$t2.old.$$") or
							return(0,"Unable to rename file $t2 to $t2.old.$$");
						&$logmsg("Renaming user-altered config file to $t2.old.$$");
						print $wfd "RS $t2.old.$$|$t2\n";
					}
				} else {
					#################################
					# Normal file, so remove.	#
					#################################
					rename($t2,$t2.".remove.\$\$") or
						return(0,"Unable to temporarily rename $t2!");
					print $wfd "RM $t\n";
				}
			}
		}
	}

	for $cdir (@{$obj->{DIRS}}) {
		my $t;

		@F=split(/\|/,$cdir);
		$t="$F[4]";
		next if ! -d "$ROOT/$t";		# Dir does not exist, so do nothing.
		&$logmsg("Removing: Directory '$t'.");
		if(!$preview) {
			print $wfd "RD $t\n";
		}
	}

	close($wfd) if !$preview;
	return(1,undef);

	#################################################################
	# Stage 2 of remove now starts...				#
	# called after a successful postremove is executed.		#
	#################################################################
}

{
my %removed_file_space;
my $files_removed=0;

sub remove_pkg_files_2 {
my %args=@_;
my ($namespace,$hook,$package);
my ($obj,@F,$ROOT,$wfd);
my ($fd,$cline,$cdir,$cfile,$err,$msg);
my ($uname,$gname,$nsobj,$preview,@ip);
my ($fcount,$fsize);

	$files_removed=0;
	%removed_file_space=();
	my $logmsg=\&logmsg;
	my $warnmsg=\&warnmsg;
	my $errormsg=\&errormsg;
	$logmsg=$args{LOGMSG} if exists($args{LOGMSG});
	$warnmsg=$args{WARNMSG} if exists($args{WARNMSG});
	$errormsg=$args{ERRORMSG} if exists($args{ERRORMSG});

	$uname=getpwuid($UID); $gname=_getgrgid($GID);
	$hook=undef; $preview=0;
	if(!exists($args{NAMESPACE}) || !exists($args{OBJ})) {
		return(0,"Missing mandatory arguments.");
	}

	$preview=$args{PREVIEW} if exists($args{PREVIEW});
	$namespace=$args{NAMESPACE};
	$hook=$args{HOOK} if exists($args{HOOK});

	$obj=$args{OBJ};
	$package=$obj->{NAME};

	($nsobj,$err)=namespace_to_obj($namespace);

	if(!exists($nsobj->{ROOT})) {
		return(0,"Missing ROOT setting for namespace $namespace.");
	}
	$ROOT=$nsobj->{ROOT};
	$ROOT=~ s/\/+$//; $ROOT="/" if $ROOT eq "";	# remove trailing /'s

	if(! -d $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace does not exist.");
	}
	if(! -w $ROOT) {
		return(0,"Specified ROOT ($ROOT) for namespace $namespace not writable.");
	}

	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$") {
		if(!rename("/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$",
			   "/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ removed")) {
			return(0,"Unable to rename package configuration file!");
		}
		dprint "Renamed removing file to removed.\n";
	}

	my $transactions=0;
	if(-f "/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$ trans") {
		open($fd,"/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$ trans") or
			return(0,"Unable to open package removal transaction file.");
		$transactions=1;
	}

	if(! $preview) {
		open($wfd,">>/var/adm/tp2/ns/$namespace/pkg/rm on reboot") or
			return(0,"Unable to open \"rm on reboot\" file.");
	}
	$fcount=0;
	my $r=$nsobj->{ROOT};
	if($transactions) {
		my ($total_files,$cfilename,$remove_action);
		$total_files++ foreach(<$fd>);
		close($fd);
		open($fd,"/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$ trans") or
			return(0,"Unable to open package removal transaction file.");
		while(<$fd>) {
			chomp;
			@F=split(/\s+/,$_);
			$_ = substr($_,3);
			$cfilename=$_;
			if($F[0] eq "RM") {
				if(! $preview) {
					($err,$msg)=rm_file_meta($nsobj,$_,$package);
					return(0,$msg) if $err==0;
				}
				if(! $preview) {
					if(-f "$r/$_.remove.\$\$") {
						my @sb=stat("$r/$_.remove.\$\$");
						if(!unlink("$r/$_.remove.\$\$")) {
							print $wfd "$_.remove.\$\$\n";
							&$warnmsg("Unable to remove $_.remove.\$\$ - will try on reboot.");
						} else {
							$files_removed++;
							$fcount++;
							$fsize=$sb[7] if scalar(@sb)>4;
							my $blocks=$fsize/$sb[11];
							my $kb;
							if($blocks!=int($blocks)) {
								$kb=(1+int($blocks)) * $sb[11];
							} else {
								$kb=$blocks * $sb[11];
							}
							$removed_file_space{"$r/$_.remove.\$\$"}=-$kb/1024;
						}
					}
				}
				if(!$preview && defined($hook)) {
					&$hook($fcount,$fsize,$total_files,$cfilename,"Remove");
				}

			} elsif($F[0] eq "UM") {
				if(! $preview) {
					($err,$msg)=rm_file_meta($nsobj,$_,$package);
					return(0,$msg) if $err==0;
					$fcount++;
					if(defined($hook)) {
						&$hook($fcount,$fsize,$total_files,$cfilename,"Update");
					}
				}
			} elsif($F[0] eq "RD") {
				if(! $preview) {
					if(!rmdir("$r/$_")) {
						&$warnmsg("Unable to remove directory $_ - not empty or no permissions.");
					}
					$fcount++;
					if(defined($hook)) {
						&$hook($fcount,$fsize,$total_files,$cfilename,"RM Dir");
					}
				}
			}
		}
		close($fd);
	}
	close($wfd) if !$preview;
	return(1,undef) if $preview;
	#################################################################
	# Check to see if a first boot file exists and if it does, 	#
	# then see about removing it - or issue a warning message 	#
	# if that is not possible.					#
	#################################################################
	if(-f "/var/adm/tp2/ns/$namespace/pkg/.first boot for $package") {
		unlink("/var/adm/tp2/ns/$namespace/pkg/.first boot for $package") or
			&$warnmsg("Unable to remove first boot flag for package.");
	}
	unlink("/var/adm/tp2/ns/$namespace/pkg/$package removing \$\$\$ trans") or
		&$warnmsg("Unable to remove transaction file for package removal.");
	unlink("/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ removed") or
		&$warnmsg("Unable to remove package configuration file.");

	return(1,undef);
}

sub _get_removed_space {
	return {%removed_file_space};
}

sub _get_summary_remove_counts {
	return ($files_removed);
}

}

#########################################################################
# This will indicate whether the package is installed in a sensible	#
# state or not.								#
# Arguments:								#
# 0 - Reference to hash of namespace details / or namespace name	#
# 1 - Name of the package to query					#
# Returns:								#
# (0,undef)	- package is not installed.				#
# (1,undef)	- package is installed without problems.		#
# (2,msg)	- package is installing/removing state - see msg.	#
# (3,msg)	- Unexpected error - see msg.				#
#########################################################################

sub pkg_install_state($$) {
my ($namespace,$pkgname)=@_;
my ($ns_obj,$err);

	if(ref($namespace)) {
		$ns_obj=$namespace;
		$namespace=$ns_obj->{NAME};
	} else {
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(undef,$err);
		}
	}
	if(! -d "/var/adm/tp2/ns/$namespace") {
		return(3,"Namespace \"$namespace\" does not exist.");
	}
	if(! -d "/var/adm/tp2/ns/$namespace/pkg") {
		return(3,"Missing \"pkg\" directory for namespace \"$namespace\".");
	}

	my $removing=0;
	my $installing=0;
	my $installed=0;
	my $state=0;
	if(-f "/var/adm/tp2/ns/$namespace/pkg/$pkgname removing \$\$\$" ||
		-f "/var/adm/tp2/ns/$namespace/pkg/$pkgname \$\$\$ removed" ||
		$pkgname =~ / \$\$\$ removed$/ ||
		$pkgname =~ / removing \$\$\$$/) {
		$removing=1;
	}
	if(-f "/var/adm/tp2/ns/$namespace/pkg/$pkgname install \$\$\$" ||
		-f "/var/adm/tp2/ns/$namespace/pkg/$pkgname \$\$\$ installed" ||
		$pkgname =~ / install \$\$\$$/ ||
		$pkgname =~ / \$\$\$ installed$/) {
		$installing=1;
	}
	$installed=1 if -f "/var/adm/tp2/ns/$namespace/pkg/$pkgname" &&
		$pkgname !~ /\$\$\$/;
	if($removing+$installing+$installed==0) {
		return(0,undef);
	}
	if($installed && $removing+$installing==0) {
		return(1,undef);
	}
	if($removing & $installing) {
		return(2,"Removing/Installing");
	}
	return(2,"Removing") if $removing;
	return(2,"Installing") if $installing;
}

#########################################################################
# The routine below is passed the name of a package in a namespace	#
# and returns an ordered list of packages that need to be removed	#
# (including the original package) if dependency information in the	#
# namespace is to be kept intact.					#
#									#
# Arguments are:							#
# 0 - namespace name or hash reference.					#
# 1 - package name to remove.						#
#									#
# Return values:							#
# (undef,$msg)	- Error has occured, see msg.				#
# (ref,undef)	- Ok - ref is a reference to list of packages.		#
#########################################################################

sub return_dependents_list($$) {
my ($namespace,$package)=@_;
my ($ns_obj,$err,$pkglist);
my (@target_list,$old_count,$new_count);

	if(ref($namespace)) {
		$ns_obj=$namespace;
		$namespace=$ns_obj->{NAME};
	} else {
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(undef,$err);
		}
	}

	#################################################################
	# Get a list of packages for this namespace.			#
	#################################################################

	($pkglist,$err)=get_ns_pkglist($ns_obj);
	if(!defined($pkglist)) {
		return(undef,"Unable to get package list for namespace:\n$err");
	}
	dprint "Package list for namespace \"$namespace\" is:\n";
	dprint $_->{NAME},"\n" foreach(@$pkglist);

	#################################################################
	# Generate a hash keyed on package name to add performance	#
	# during the potentially closely iteritive process of working	#
	# out the dependencies.						#
	#################################################################

	my %pkgs=();
	for (@$pkglist) {
		$pkgs{$_->{NAME}}=$_;
	}

	@target_list=("$package");
	my $n=1; my $cpkgname;
	do {
		my ($cfg,$cdep,$cpkg);
		$old_count=scalar(@target_list);

		#########################################################
		# Scan the packages in the list in REVERSE ORDER, and	#
		# if the current package is mentioned in the 		#
		# dependencies add it to the start of the list if not	#
		# present already.					#
		#########################################################

		dprint "\n";
		dprint "Iteration $n - current dependency list:\n";
		dprint join(",",@target_list),"\n\n";

		for $cpkg (reverse(@target_list)) {
			for $cfg (@$pkglist) {
				next if !exists($pkgs{$cfg->{NAME}}->{DEPENDENCIES});
				dprint "Dependencies for $cfg->{NAME} :: $pkgs{$cfg->{NAME}}->{DEPENDENCIES}\n";

				for $cdep (split(/\s+/,$pkgs{$cfg->{NAME}}->{DEPENDENCIES})) {
					$cpkgname=$cfg->{NAME};
					my @xx=split(/\|/,$cdep); my $xx0=$xx[0];
					if($xx0 eq $cpkg && 
						!scalar(grep {/^$cpkgname$/} @target_list)) {
						dprint "Package ",$cfg->{NAME}," requires $xx0.\n";
						unshift @target_list,$cpkgname;
					}
				}
			}
		}

		$new_count=scalar(@target_list);
		$n++;
		errormsg("Too many iterations attempting to determine dependencies: $n.",2) if $n>100;

	} until($new_count==$old_count);

	#################################################################
	# Rather than just return the names, return references to 	#
	# hashes for each package.					#
	#################################################################

	my @mlist=();
	push @mlist,$pkgs{$_} foreach (@target_list);
	return(\@mlist,undef);
}

#########################################################################
# This utility is typically called by the tp2admin command and is used	#
# to remove the inclusion of a specified file (relative to namespace	#
# root) from the specified package.					#
#									#
# Typically useful if you do not wish for the upgrade/downgrade to	#
# remove such files on the next install if they are no longer bundled	#
# with the package.							#
#									#
# Steps taken to perform this change...					#
#     Checks:								#
# [1] Validate namespace exists						#
# [2] Validate UID suitable for change					#
# [3] Validate specified package is installed correctly.		#
# [4] Validate specfied file is part of package.			#
#									#
#     Changes:								#

#########################################################################

sub remove_file_from_package {
my ($namespace,$package,$file)=@_;
my ($ns_obj,$err,$ok,$ok2,$status,$obj);
my ($found,@F);

	#################################################################
	# Resolve namespace to object if required.			#
	#################################################################
	
	if(ref($namespace)) {
		$ns_obj=$namespace;
		$namespace=$ns_obj->{NAME};
	} else {
		($ns_obj,$err)=namespace_to_obj($namespace);
		if(!defined($ns_obj)) {
			return(0,$err);
		}
	}
	($ok,$err)=check_namespace_write($namespace);
	if(!$ok) {
		return(0,$err);
	}

	#################################################################
	# Check that the package is installed, and abort if not.	#
	#################################################################

	($ok2,$status)=pkg_install_state($ns_obj,$package);
	if($ok2==1) {
		$status="Installed";
	} else {
		return(0,"Package \"$package\" is not currently installed.") if $ok2==0;
		return(0,"Package is in state \"$status\" - action not permitted!") if $ok2==2;
		return(0,"Unexpected error: $msg");
	}

	#################################################################
	# Package is in the correct state, so load in package		#
	# configuration.						#
	#################################################################

	($obj,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$package");
	if(!defined($obj)) {
		return(0,$err);
	}

	#################################################################
	# Scan files in package (if any) to ensure it includes the file	#
	# in question and if not abort.					#
	#################################################################

	if(!exists($obj->{FILES})) {
		return(0,"File \"$file\" not defined in package.");
	}

	$found=0; my $ind=0;
	foreach (@{$obj->{FILES}}) {
		@F=split(/\|/,$_);
		if($F[6] eq $file) {
			$found=1; last;
		}
		$ind++;
	}
	return(0,"File \"$file\" not defined in package.") if ! $found;

	my @L=@{$obj->{FILES}};
	splice(@L,$ind,1);
	$obj->{FILES}=[@L];

	#################################################################
	# Now the checks have been passed, remove the entry from	#
	# array and re-write.						#
	#################################################################

	if(!rename("/var/adm/tp2/ns/$namespace/pkg/$package",
		   "/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed")) {
		   return(0,"Unable to rename package configuration file!");
	}
	my ($wfd,$fd,$cline,$infiles);

	if(!open($fd,"/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed")) {
		rename("/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed",
			   "/var/adm/tp2/ns/$namespace/pkg/$package");
		return(0,"Unable to open temporary package file for writing:\n$!");
	}

	if(!open($wfd,">/var/adm/tp2/ns/$namespace/pkg/$package.$$")) {
		rename("/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed",
			   "/var/adm/tp2/ns/$namespace/pkg/$package");
		return(0,"Unable to open temporary package file for writing:\n$!");
	}

	#################################################################
	# Both files open as required, so squirt one into the other	#
	# apart from the required file entry.				#
	#################################################################

	$infiles=0;
	while($cline=<$fd>) {
		chomp $cline;
		if($cline =~ /^__FILELIST__$/) {
			$infiles=1;
		}
		if($infiles) {
			@F=split(/\|/,$cline);
			next if defined($F[6]) && $F[6] eq $file;
		}
		if(!print $wfd "$cline\n") {
			close($fd); close($wfd);
			unlink("/var/adm/tp2/ns/$namespace/pkg/$package.$$");
			rename("/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed",
				   "/var/adm/tp2/ns/$namespace/pkg/$package");
			return(0,"Unable to write to temporary package file:\n$!");
		}
	}
	close($fd);
	if(!close($wfd)) {
		unlink("/var/adm/tp2/ns/$namespace/pkg/$package.$$");
		rename("/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed",
			   "/var/adm/tp2/ns/$namespace/pkg/$package");
		return(0,"Unable to write to temporary package file:\n$!");
	}

	#################################################################
	# Update (if necessary) has been applied to package config	#
	# file so remove entries from the meta data now...		#
	#################################################################

	($ok,$err)=rm_file_meta($ns_obj,$file,$package);
	if(!$ok) {
		unlink("/var/adm/tp2/ns/$namespace/pkg/$package.$$");
		rename("/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed",
			   "/var/adm/tp2/ns/$namespace/pkg/$package");
		return(0,$err);
	}

	#################################################################
	# Ok - file has been removed from the meta information, so	#
	# do the final bit.						#
	#################################################################

	if(!rename("/var/adm/tp2/ns/$namespace/pkg/$package.$$",
		   "/var/adm/tp2/ns/$namespace/pkg/$package")) {
		unlink("/var/adm/tp2/ns/$namespace/pkg/$package.$$");
		return(0,"Unable to rename temporary package file:\n$!");
	}
	unlink("/var/adm/tp2/ns/$namespace/pkg/$package \$\$\$ installed");
	return(1,undef);
}

sub repackage {
my %args=@_;
my ($err,$pkg_obj);

	if(!exists($args{pkgname}) || !exists($args{ns})) {
		return(0,"Missing mandatory arguments.");
	}

	my $pkgname=$args{pkgname};
	my $ns=$args{ns};
	my $hook=$args{hook} if exists($args{hook});
	my $newver=$args{newver} if exists($args{newver});

	my ($namespace,$ns_obj);
	if(ref($ns)) {
		$ns_obj=$ns;
		$namespace=$ns_obj->{NAME};
	} else {
		($ns_obj,$err)=namespace_to_obj($ns);
		if(!defined($ns_obj)) {
			return(0,$err);
		}
		$namespace=$ns;
	}
	#################################################################
	# Before doing anything check for a REPACKAGE_DISALLOW var and	#
	# if present abort with an error indicating repackaging is	#
	# not supported.						#
	#################################################################
	if(exists($ns_obj->{REPACKAGE_DISALLOW})) {
		return (0,"Repackaging is not supported for namespace '$namespace'.");
	}
	my $repackage_dir="/var/adm/tp2/ns/$namespace/repackage";
	if(exists($ns_obj->{REPACKAGE_DIR})) {
		$repackage_dir=$ns_obj->{REPACKAGE_DIR};
	}
	if(! -e $repackage_dir) {
		mkdir $repackage_dir,0755 or
			return(0,"Unable to create directory to repackage to: '$repackage_dir'.");
	}
	if(! -d $repackage_dir) {
		return(0,"Location to repackage to ['$repackage_dir'] is not a directory!");
	}
		
	#################################################################
	# We have the namespace details, so check to see if specified	#
	# package has a configuration file, and abort if not...		#
	#################################################################

	if(! -f "/var/adm/tp2/ns/$namespace/pkg/$pkgname") {
		return(0,"Missing package configuration file for '$pkgname' in namespace '$namespace'.");
	}

	#################################################################
	# Load in the configuration file, and create a temporary 	#
	# directory in the TMPDIR or spool directory for the package.	#
	#################################################################

	($pkg_obj,$err)=pkgfile_config_to_obj("/var/adm/tp2/ns/$namespace/pkg/$pkgname");
	if(!defined($pkg_obj)) {
		return(0,$err);
	}

	#################################################################
	# Change the package description, if the new version has 	#
	# been provided.						#
	#################################################################

	if(defined($newver)) {
		$pkg_obj->{DESCRIPTION}="Repackaged $pkg_obj->{PKGVER} during $newver install - $pkg_obj->{DESCRIPTION}";
	}
	
	my $tmpdir;
	my $root=$ns_obj->{ROOT};
	if(exists($ns_obj->{TMPDIR})) {
		$tmpdir=$ns_obj->{TMPDIR};
	} else {
		$tmpdir=$root;
	}
	$tmpdir=~ s/\/+$//;	# remove trailing /'s

	my $ctime=time;
	my $_t="$tmpdir/$pkgname-$ctime";
	mkdir($_t) or return(0,"Unable to create temporary directory: $!");
	
	my @new_dirs=();
	my @new_files=();
	my $cent;
	for $cent (@{$pkg_obj->{DIRS}}) {
		my @F=split(/\|/,$cent);
		if(! -e "$root/$F[4]") {
			next;
		}
		if(-d _) {
			@s=stat(_);
			my ($uid,$gid);
			$uid=getpwuid($s[4]);
			$gid=getgrgid($s[5]);
			$uid=$s[4] if ! defined($uid);
			$gid=$s[5] if ! defined($gid);
			push @new_dirs,sprintf("D|%o|%s|%s|$F[4]",$s[2] & 07777,$uid,$gid);
			next;
		}
		rmdir($_t);
		return(0,"Package entry directory '$F[4]' is no longer a directory.");
	}

	#################################################################
	# Now do the same thing, but for files...			#
	#################################################################

	$total=@{$pkg_obj->{FILES}};
	my $cfile=0;
	for $cent (@{$pkg_obj->{FILES}}) {
		my @F=split(/\|/,$cent);
		if(! -e "$root/$F[6]") {
			next;
		}
		if(! -f _) {
			rmdir($_t);
			return(0,"Package entry directory '$F[6]' is no longer a regular file.");
		}
		@s=stat(_);
		my ($uid,$gid,$size);
		$uid=getpwuid($s[4]);
		$gid=getgrgid($s[5]);
		$size=$s[7];
		$uid=$s[4] if ! defined($uid);
		$gid=$s[5] if ! defined($gid);
		my ($e_sha1,$n_sha1);
		$cfile++;
		if(defined($hook)) {
			&$hook($F[6],$cfile,$total,"Checking");
		}
		if($F[4] =~ /^[a-f0-9]{40}$/) {
			#################################################
			# Get the SHA1 of the current file contents.	#
			#################################################
			$n_sha1=file_sha1("$root/$F[6]");
		} else {
			$n_sha1=$F[4];
		}
		push @new_files,sprintf("F|%o|%s|%s|$n_sha1|$size|$F[6]",$s[2] & 07777,$uid,$gid);
	}
	
	#################################################################
	# We now copy all files in the new_files array to the temporary	#
	# folder, making directories as and when required. We also	#
	# call the hook function if available with the details of the	#
	# copy process to provide feedback.				#
	#################################################################

	$total=@new_files;
	$cfile=0;
	for $cent (@new_files) {
		my @F=split(/\|/,$cent);
		my $np="$_t/$F[6]";
		my $nd=dirname($np);
		if(! -d $nd) {
			make_dir_path($nd);
		}

		#########################################################
		# Copy the file to the necessary directory.		#
		#########################################################

		my ($ok,$msg)=copyfile("$root/$F[6]",$np);
		if(!defined($ok)) {
			remove_dir_path("$_t");
			return(0,"Unable to copy '$F[6]' for repackaging:\n$msg");
		}
		#########################################################
		# Call the hook if one is present...			#
		#########################################################
		$cfile++;
		if(defined($hook)) {
			&$hook($F[6],$cfile,$total);
		}
	}

	#################################################################
	# All files have been copied to the temporary directory so the	#
	# next step is to generate the !!..config..!! file.		#
	#################################################################

	$pkg_obj->{DIRS}=[@new_dirs];
	$pkg_obj->{FILES}=[@new_files];
	$pkg_obj->{REPACKAGED}=hostname;
	my $ok=regenerate_pkg_config($pkg_obj,$_t);
	if(! $ok) {
		remove_dir_path($_t);
		return(0,"Unable to generate package configuration file  '$_t/!!..config..!!'.");
	}
	
	#################################################################
	# Everything should be in place now, so generate the package -	#
	# and stuff it in the pkg/repackage directory if it exists,	#
	# /tmp otherwise.						#
	#################################################################

	my $destdir=$repackage_dir;
	my $dummy={name=>"$pkgname-repackaged",version=>$pkg_obj->{PKGVER}};
	$dummy->{os}=$pkg_obj->{OS} if exists($pkg_obj->{OS});
	$dummy->{architecture}=$pkg_obj->{ARCHITECTURE} if exists($pkg_obj->{ARCHITECTURE});
	my $newname=get_pkg_name($dummy);

	my $pwd=cwd;
	if(! chdir($_t)) {
		remove_dir_path($_t);
		return(0,"Unable to change to temporary directory '$_t': $!.");

	}
	$cmd=_get_tar_cmd()." czf $destdir/$newname .";
	system($cmd);
	if($?>>8) {
		chdir $pwd;
		remove_dir_path($_t);
		return(0,"Unable to generate repackaged package '$destdir/$newname'.");
	}
	chdir $pwd;
	remove_dir_path($_t);
	return(1,"$destdir/$newname");
}

1;

